'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2009
' by DotNetNuke Corporation
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'
Imports System
Imports System.ComponentModel
Imports System.Text
Imports System.Web
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports DotNetNuke.UI.Utilities

Namespace DotNetNuke.UI.WebControls

    <TypeConverter(GetType(ExpandableObjectConverter))> _
    Public Class DNNAnimation
        Implements IStateManager, IUrlResolutionService

#Region "Member Variables"
        Private m_Marked As Boolean = False
        Private m_State As StateBag
#End Region

#Region "Properties"

        <ClientProperty(), ClientPropertyName("anim"), DefaultValue(Animation.AnimationType.None), NotifyParentProperty(True)> _
        Public Property AnimationType() As Animation.AnimationType
            Get
                If ClientAPI.BrowserSupportsFunctionality(Utilities.ClientAPI.ClientFunctionality.Motion) Then
                    If Len(ViewState("AnimationType")) = 0 Then
                        Return Animation.AnimationType.None
                    Else
                        Return CType(ViewState("AnimationType"), Animation.AnimationType)
                    End If
                Else
                    Return Animation.AnimationType.None
                End If
            End Get
            Set(ByVal value As Animation.AnimationType)
                ViewState("AnimationType") = value
            End Set
        End Property

        <ClientProperty(), ClientPropertyName("easeDir"), DefaultValue(Animation.EasingDirection.Out), NotifyParentProperty(True)> _
        Public Property EasingDirection() As Animation.EasingDirection
            Get
                If Len(ViewState("EasingDirection")) = 0 Then
                    Return Animation.EasingDirection.In
                Else
                    Return CType(ViewState("EasingDirection"), Animation.EasingDirection)
                End If
            End Get
            Set(ByVal value As Animation.EasingDirection)
                ViewState("EasingDirection") = value
            End Set
        End Property

        <ClientProperty(), ClientPropertyName("easeType"), DefaultValue(Animation.EasingType.Expo), NotifyParentProperty(True)> _
        Public Property EasingType() As Animation.EasingType
            Get
                If Len(ViewState("EasingType")) = 0 Then
                    Return Animation.EasingType.Expo
                Else
                    Return CType(ViewState("EasingType"), Animation.EasingType)
                End If
            End Get
            Set(ByVal value As Animation.EasingType)
                ViewState("EasingType") = value
            End Set
        End Property

        <ClientProperty(), ClientPropertyName("animLen"), DefaultValue(1), NotifyParentProperty(True)> _
        Public Property Length() As Integer
            Get
                If Len(ViewState("Length")) = 0 Then
                    Return 1
                Else
                    Return CInt(ViewState("Length"))
                End If
            End Get
            Set(ByVal value As Integer)
                ViewState("Length") = value
            End Set
        End Property

        <ClientProperty(), ClientPropertyName("animInt"), DefaultValue(10), NotifyParentProperty(True)> _
        Public Property Interval() As Integer
            Get
                If Len(ViewState("Interval")) = 0 Then
                    Return 10
                Else
                    Return CInt(ViewState("Interval"))
                End If
            End Get
            Set(ByVal value As Integer)
                ViewState("Interval") = value
            End Set
        End Property


#End Region

#Region "State Management"
        Protected ReadOnly Property ViewState() As StateBag
            Get
                If m_State Is Nothing Then
                    m_State = New StateBag(True)
                    If IsTrackingViewState Then
                        CType(m_State, IStateManager).TrackViewState()
                    End If
                End If
                Return m_State
            End Get
        End Property

        Protected ReadOnly Property IsTrackingViewState() As Boolean Implements System.Web.UI.IStateManager.IsTrackingViewState
            Get
                Return m_Marked
            End Get
        End Property

        Protected Sub LoadViewState(ByVal state As Object) Implements System.Web.UI.IStateManager.LoadViewState
            If Not (state Is Nothing) Then
                CType(ViewState, IStateManager).LoadViewState(state)
            End If
        End Sub

        Protected Function SaveViewState() As Object Implements System.Web.UI.IStateManager.SaveViewState
            Dim oState As Object = Nothing
            If Not (m_State Is Nothing) Then
                oState = CType(m_State, IStateManager).SaveViewState()
            End If
            Return oState
        End Function

        Protected Sub TrackViewState() Implements System.Web.UI.IStateManager.TrackViewState
            m_Marked = True
        End Sub
#End Region

#Region "IUrlResolutionService"
        Public Function ResolveClientUrl(ByVal Url As String) As String Implements IUrlResolutionService.ResolveClientUrl
            'no url properties at the moment, so don't need anything here
            Return Url
        End Function
#End Region

    End Class
End Namespace