'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2009
' by DotNetNuke Corporation
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System
Imports System.ComponentModel
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Xml

Namespace DotNetNuke.UI.WebControls
	Public Class DNNNode

#Region "Member Variables"
		Private m_objXMLDoc As XmlDocument
		Private m_objXMLNode As XmlNode
		Private m_strParentNS As String
		Private m_objNodes As DNNNodeCollection
		Private m_objHashAttributes As Hashtable
#End Region

#Region "Constructors"
		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Default constructor
		''' </summary>
		''' <remarks>
		''' When using this constructor your node will not have properties that are related
		''' to the xml node hierarchy until you associate the node with the hierarchy.  This is 
		''' accomplished by adding it to the DNNNodeCollection or by calling Import
		''' directly
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Sub New()
			Me.New(New XmlDocument().CreateNode(XmlNodeType.Element, "n", ""))
		End Sub

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Constructor providing the defaulting of the Text property
		''' </summary>
		''' <param name="strText">Text for the Node</param>
		''' <remarks>
		''' When using this constructor your node will not have properties that are related
		''' to the xml node hierarchy until you associate the node with the hierarchy.  This is 
		''' accomplished by adding it to the DNNNodeCollection or by calling Import
		''' directly
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Sub New(ByVal strText As String)
			Me.New()
			Me.Text = strText
		End Sub

		Public Sub New(ByVal NodeText As String, ByVal navigateUrl As String)
			If NodeText Is Nothing OrElse navigateUrl Is Nothing Then
				Throw New ArgumentNullException
			End If
			Text = NodeText
			navigateUrl = navigateUrl
		End Sub

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Constructor to create a Node already associated to a hierarchy
		''' </summary>
		''' <param name="objXmlNode"></param>
		''' <remarks>
		''' Preferred method for creating a node.  Since the node wraps the XmlNode object
		''' there will be no need for an intermediate holder of the attributes, they can
		''' be directly written to the XmlNode object that belongs to the hierarchy.
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Sub New(ByVal objXmlNode As XmlNode)
			m_objXMLNode = objXmlNode
			m_objXMLDoc = objXmlNode.OwnerDocument
		End Sub

#End Region

#Region "Node Properties"
		''' -----------------------------------------------------------------------------
		''' <summary>
		''' A node that is instantiated without knowledge of its parent is created "stand-alone",
		''' therefore, it does not belong to an xmldocument, until associated with a DNNNodeCollection
		''' Until this association takes place, the node will not be able to 
		''' expose a property/method that requires the hierarchy.  This property indicates
		''' the node's membership to a hierarchy.
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public ReadOnly Property IsInHierarchy() As Boolean
			Get
				'Return Not XmlNode Is Nothing
				Return Not XmlNode.ParentNode Is Nothing
			End Get
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Exposes the XmlDocument that the OwnerDocument of the XmlNode the class is wrapping
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Friend ReadOnly Property XMLDoc() As XmlDocument
			Get
				Return m_objXMLDoc
			End Get
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Exposes the XmlNode the node is wrapping
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Friend ReadOnly Property XmlNode() As XmlNode
			Get
				Return m_objXMLNode
			End Get
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Exposes the root XmlNode
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Private ReadOnly Property RootNode() As XmlNode
			Get
				Return XMLDoc.ChildNodes(0)
			End Get
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Returns the parent DNNNode
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' Requires node to be associated or nothing will return
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public ReadOnly Property ParentNode() As DNNNode
			Get
				If Not Me.XmlNode.ParentNode Is Nothing AndAlso Me.XmlNode.ParentNode.NodeType <> System.Xml.XmlNodeType.Document Then
					Return New DNNNode(Me.XmlNode.ParentNode)
				Else
					Return Nothing
				End If
			End Get
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Returns a collection of the children DNNNodes
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public ReadOnly Property DNNNodes() As DNNNodeCollection
			Get
				If m_objNodes Is Nothing Then
					m_objNodes = New DNNNodeCollection(Me.XmlNode)
				End If
				Return m_objNodes
			End Get
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Returns a colleciton of the children XmlNodes
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public ReadOnly Property XmlNodes() As XmlNodeList
			Get
				Return Me.XmlNode.ChildNodes
			End Get
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Returns a flag indicating whether the node has children
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Property HasNodes() As Boolean
			Get
				Dim blnHas As Boolean = CBool(CustomAttribute("hasNodes", "0"))
				If blnHas = False Then
					Return Me.DNNNodes.Count > 0
				Else
					Return blnHas					  'False
				End If
			End Get
			Set(ByVal Value As Boolean)
				'CustomAttribute("hasNodes", 0) = Value
				Me.CustomAttribute("hasNodes") = CStr(IIf(Value, "1", "0"))
			End Set
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Gets the node's parent namespace by walking the chain up until it reaches the 
		''' root.  This name plus a unique identifier for the node will be unique.
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public ReadOnly Property ParentNameSpace() As String
			Get
				If Len(m_strParentNS) = 0 Then
					m_strParentNS = ""
                    If Not XmlNode.ParentNode Is Nothing AndAlso TypeOf (XmlNode.ParentNode) Is XmlElement AndAlso Not XmlNode.ParentNode.Attributes.GetNamedItem("id") Is Nothing Then
                        m_strParentNS = XmlNode.ParentNode.Attributes.GetNamedItem("id").Value
                    End If
				End If
				Return m_strParentNS
			End Get
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Returns the level (depth) of the node within the hierarchy
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public ReadOnly Property Level() As Integer
			Get
				'If IsInHierarchy AndAlso Not Me.ParentNode Is Nothing Then
				If Not Me.ParentNode Is Nothing Then
					Dim objParent As XmlNode = Me.XmlNode
					Dim intLevel As Integer = -1
					While Not objParent Is Nothing AndAlso TypeOf (objParent) Is XmlElement
						intLevel += 1
						objParent = objParent.ParentNode
						If Not objParent Is Nothing AndAlso objParent.Name = "root" Then Exit While
					End While
					Return intLevel
				Else

					Return -1
				End If
			End Get
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Property to access attribute collection for the node
		''' </summary>
		''' <param name="Key">Attribute Name</param>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history> 
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Property CustomAttribute(ByVal Key As String) As String
            Get
                If Not XmlNode.Attributes.GetNamedItem(Key) Is Nothing Then
                    Return XmlNode.Attributes.GetNamedItem(Key).Value
                Else
                    Return Nothing
                End If
            End Get
            Set(ByVal Value As String)
                Try
                    If Not XmlNode.Attributes.GetNamedItem(Key) Is Nothing Then
                        If Value Is Nothing Then
                            XmlNode.Attributes.Remove(CType(XmlNode.Attributes.GetNamedItem(Key), XmlAttribute))
                        Else
                            XmlNode.Attributes.GetNamedItem(Key).Value = Value
                        End If
					ElseIf Not Value Is Nothing Then
						Dim objAttr As System.Xml.XmlAttribute = XMLDoc.CreateAttribute(Key)
						objAttr.Value = Value
						XmlNode.Attributes.Append(objAttr)
					End If
				Catch ex As Exception
                    Throw ex
                End Try
            End Set
        End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Property to access attribute collection for the node.  If property is not set, 
		''' the passed in default value will be returned
		''' </summary>
		''' <param name="Key">Attribute Name</param>
		''' <param name="DefaultValue">Value to return when attribute not set</param>
		''' <value></value>
		''' <remarks>
		''' Until the node is in a hierarchy, thus allowing the creation of an XmlNode,
		''' the attributes will be stored in a hashtable.  Once the node is associated
		''' with a hierarchy, these attributes will be transferred to the XmlNode
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Property CustomAttribute(ByVal Key As String, ByVal DefaultValue As String) As String
			Get
				If Len(CustomAttribute(Key)) = 0 Then
					Return DefaultValue
				Else
					Return CustomAttribute(Key)
				End If

			End Get
			Set(ByVal Value As String)
				If Value = DefaultValue Then Value = ""
				CustomAttribute(Key) = Value
			End Set
		End Property

#End Region

#Region "Base Attributes"

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' ID of Node
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Property ID() As String
			Get
				Return CStr(CustomAttribute("id"))
			End Get
			Set(ByVal Value As String)
				CustomAttribute("id") = Value
			End Set
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' ClientID of Node
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' Since the client browsers do not like :, we are replacing with _
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public ReadOnly Property ClientID() As String
			Get
				Return Me.ID.Replace(":", "_")
			End Get
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Key to identify the node
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Property Key() As String
			Get
				Return CustomAttribute("key")
			End Get
			Set(ByVal Value As String)
				CustomAttribute("key") = Value
			End Set
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Text for the node to display
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Property Text() As String
			Get
				Return CustomAttribute("txt")
			End Get
			Set(ByVal Value As String)
				CustomAttribute("txt") = Value
			End Set
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' URL to navigate to when node is selected
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		'''		[Jon Henning]	11/14/2005  When assigning a NavigateUrl, set clickaction to navigate to maintain backwards compat
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Property NavigateURL() As String
			Get
				Return CustomAttribute("url")
			End Get
			Set(ByVal Value As String)
				If Len(Value) > 0 Then
					Me.ClickAction = eClickAction.Navigate
					CustomAttribute("url") = Value
				Else
					CustomAttribute("url") = Nothing				'don't render attribute
				End If
			End Set
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Function to execute when node is selected
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Property JSFunction() As String
			Get
				Return CustomAttribute("js")
			End Get
			Set(ByVal Value As String)
				CustomAttribute("js") = Value
			End Set
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Target frame to do the navigation
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Property Target() As String
			Get
				Return CustomAttribute("tar")
			End Get
			Set(ByVal Value As String)
				CustomAttribute("tar") = Value
			End Set
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' ToolTip for the node to display
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Property ToolTip() As String
			Get
				Return CustomAttribute("tTip")
			End Get
			Set(ByVal Value As String)
				If Len(Value) = 0 Then
					CustomAttribute("tTip") = Nothing				'don't render attribute
				Else
					CustomAttribute("tTip") = Value
				End If
			End Set
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Flag to determine if node is enabled
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Property Enabled() As Boolean
			Get
				Return CBool(CustomAttribute("enabled", "1"))
			End Get
			Set(ByVal Value As Boolean)
				CustomAttribute("enabled") = CStr(IIf(Value, "1", "0"))
			End Set
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' CSS Class Name of node
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Property CSSClass() As String
			Get
				Return CustomAttribute("css")
			End Get
			Set(ByVal Value As String)
				CustomAttribute("css") = Value
			End Set
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' CSS Class Name of node when selected
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Property CSSClassSelected() As String
			Get
				Return CustomAttribute("cssSel")
			End Get
			Set(ByVal Value As String)
				CustomAttribute("cssSel") = Value
			End Set
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' CSS Class Name of node when hovered
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Property CSSClassHover() As String
			Get
				Return CustomAttribute("cssHover")
			End Get
			Set(ByVal Value As String)
				CustomAttribute("cssHover") = Value
			End Set
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' CSS Class Name of Icon
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Property CSSIcon() As String
			Get
				Return CustomAttribute("cssIcon")
			End Get
			Set(ByVal Value As String)
				CustomAttribute("cssIcon") = Value
			End Set
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' image for the node to display
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	4/7/2005	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Property Image() As String
			Get
				Return CustomAttribute("img")
			End Get
			Set(ByVal Value As String)
				CustomAttribute("img") = Value
			End Set
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Determines if node is selected
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	4/7/2005	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Property Selected() As Boolean
			Get
				Return CBool(Me.CustomAttribute("selected", "0"))
			End Get
			Set(ByVal Value As Boolean)
				Me.CustomAttribute("selected") = CStr(IIf(Value, "1", "0"))
			End Set
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Determines if node is part of the breadcrumb
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	4/7/2005	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Property BreadCrumb() As Boolean
			Get
				Return CBool(Me.CustomAttribute("bcrumb", "0"))
			End Get
			Set(ByVal Value As Boolean)
				Me.CustomAttribute("bcrumb") = CStr(IIf(Value, "1", "0"))
			End Set
		End Property

		Public Property ClickAction() As eClickAction
			Get
				If Len(Me.CustomAttribute("ca")) > 0 Then
					Dim eAction As eClickAction = CType(Me.CustomAttribute("ca"), eClickAction)
					Return (eAction)
				Else
					Return eClickAction.PostBack
				End If
			End Get
			Set(ByVal Value As eClickAction)
				Me.CustomAttribute("ca") = CStr(Value)
			End Set
		End Property

		Public Property IsBreak() As Boolean
			Get
                Return CBool(Me.CustomAttribute("isBreak", "0"))
			End Get
			Set(ByVal Value As Boolean)
                Me.CustomAttribute("isBreak") = CStr(IIf(Value, "1", "0"))
			End Set
		End Property

#End Region

#Region "Methods"

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Retrieves Xml for the node
		''' </summary>
		''' <returns>Xml of current node and all its children</returns>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Function ToXML() As String
			Return XmlNode.OuterXml
		End Function

		Public Function ToJSON() As String
			Return ToJSON(True)
		End Function

		Public Function ToJSON(ByVal blnDeep As Boolean) As String

            Dim sb As System.Text.StringBuilder = New System.Text.StringBuilder()
			Dim oAttr As XmlAttribute
			For Each oAttr In Me.XmlNode.Attributes
				If sb.Length = 0 Then
					sb.Append("{")
				Else
					sb.Append(",")
                End If
                'making assumption here, that only when we are not looking for a deep json representation do we want the nice text properties
                If blnDeep = False Then
                    'xml contains abbreviations... but when we are using this on the client we want to use it as if it was a real DNNNode (DNNTreeNode) object so we want the long names
                    Select Case oAttr.Name
                        Case "txt"
                            sb.Append("text")
                        Case "tar"
                            sb.Append("target")
                        Case "tTip"
                            sb.Append("toolTip")
                        Case Else
                            sb.Append(oAttr.Name)
                    End Select
                Else
                    sb.Append(oAttr.Name)
                End If
                sb.Append(":")
                sb.Append("""" & DotNetNuke.UI.Utilities.ClientAPI.GetSafeJSString(oAttr.Value) & """")
            Next
            If blnDeep Then
                sb.Append(",")
                sb.Append(Me.DNNNodes.ToJSON())
            End If
            sb.Append("}")
			Return sb.ToString

		End Function

        'Friend Sub AssociateXmlNode(ByVal objXmlNode As XmlNode)
		'	m_objXMLNode = objXmlNode
		'	m_objXMLDoc = objXmlNode.OwnerDocument

		'	'copy all the values from the cache over
		'	Dim strKey As String
		'	If Not m_objHashAttributes Is Nothing Then
		'		For Each strKey In m_objHashAttributes.Keys
		'			CustomAttribute(strKey) = m_objHashAttributes(strKey)
		'		Next
		'	End If
		'End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Since an XmlNode cannot exist without a document, and since the node's 
        ''' interface allows the creation of a node without a document, we need 
        ''' to expose a way for the node to be added to a document.
        ''' </summary>
        ''' <param name="objXmlNode">XmlNode to associate with the DNNNode object</param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	12/22/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Friend Sub AssociateXmlNode(ByVal objXmlNode As XmlNode)
            m_objXMLNode = objXmlNode
            m_objXMLDoc = objXmlNode.OwnerDocument
        End Sub


        Public Function Clone() As DNNNode
            Return Clone(True)
        End Function

        Public Function Clone(ByVal blnDeep As Boolean) As DNNNode
            Dim objXmlNode As XmlNode = Me.XmlNode.CloneNode(blnDeep)
            Return New DNNNode(objXmlNode)
        End Function

#End Region

	End Class
End Namespace