'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2009
' by DotNetNuke Corporation
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'
Imports System.Xml
Imports DotNetNuke.UI.WebControls
Imports System.Web.UI
Imports System.Collections
Imports System.Collections.Generic

Namespace DotNetNuke.UI.WebControls

	Public Class DNNNodeCollection : Inherits XmlCollectionBase

#Region "Constructors"
		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Constructor to call when creating a Root Node
		''' </summary>
		''' <param name="strNamespace">Namespace of node hierarchy</param>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Sub New(ByVal strNamespace As String)
			MyBase.New(strNamespace)
		End Sub

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Loads node collection based off of XML string
		''' </summary>
		''' <param name="strXML">XML string</param>
		''' <param name="strXSLFile">XSL FileName.  Leave empty if no transform needed</param>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Sub New(ByVal strXML As String, ByVal strXSLFile As String)
			MyBase.New(strXML, strXSLFile)
		End Sub

        Public Sub New(ByVal RootNamespace As String, ByVal JSONDict As Dictionary(Of String, Object))
            MyBase.New(RootNamespace)
            LoadJSON(CType(JSONDict("nodes"), ArrayList))
        End Sub

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Constructor for all nodes that are not the root.  
		''' </summary>
		''' <param name="objXmlNode">Node whose children will be exposed by this class</param>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	12/22/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Sub New(ByVal objXmlNode As XMLNode)
			MyBase.New(objXmlNode)
		End Sub

#Region "TreeView Backwards Compatibility"
		'In order to maintain backwards compatibility with the tree from versions before 3.2 we need to allow the 
		'baseclass to return the treenodeenumerator, thus the need to pass the tree to the new base collection class
		'Yes, this is a hack!
		Public Sub New(ByVal strNamespace As String, ByVal objTreeControl As DotNetNuke.UI.WebControls.DnnTree)
			MyBase.New(strNamespace, objTreeControl)
		End Sub

		Public Sub New(ByVal objXmlNode As XMLNode, ByVal objTreeControl As DotNetNuke.UI.WebControls.DnnTree)
			MyBase.New(objXmlNode, objTreeControl)
		End Sub
#End Region

#End Region

		Public ReadOnly Property XMLNode() As Xml.XmlNode
			Get
				Return MyBase.InnerXMLNode
			End Get
		End Property

		Public ReadOnly Property XMLDoc() As Xml.XmlDocument
			Get
				Return MyBase.InnerXMLDoc
			End Get
		End Property

		Public Function Add() As Integer		  ' DNNNode
			Dim objNode As DNNNode = New DNNNode
			Me.XMLNode.AppendChild(Me.XMLDoc.ImportNode(objNode.XmlNode, False))
			objNode.ID = objNode.ParentNameSpace & "_" & XMLNode.ChildNodes.Count
			Return Me.XMLNode.ChildNodes.Count - 1			 'Return objNode
		End Function

		Public Function Add(ByVal objNode As DNNNode) As Integer		  ' DNNNode
			Dim objXmlNode As Xml.XmlNode = Me.XMLDoc.ImportNode(objNode.XmlNode, True)
			Me.XMLNode.AppendChild(objXmlNode)
			objNode.AssociateXmlNode(objXmlNode)
			Return Me.XMLNode.ChildNodes.Count - 1
		End Function

		Public Function AddBreak() As Integer		  ' DNNNode
			Dim intIndex As Integer = Me.Add()
			Dim objNode As DNNNode = Me(intIndex)
			objNode.IsBreak = True
			Return intIndex
		End Function

		Public Function Add(ByVal strID As String, ByVal strKey As String, ByVal strText As String, ByVal strNavigateURL As String, _
		 ByVal strJSFunction As String, ByVal strTarget As String, ByVal strToolTip As String, ByVal blnEnabled As Boolean, _
		 ByVal strCSSClass As String, ByVal strCSSClassSelected As String, ByVal strCSSClassHover As String) As Integer		  'DNNNode

			Dim intIndex As Integer = Add()
			Dim objNode As DNNNode = Me.Item(intIndex)

			If Len(strID) > 0 Then
				objNode.ID = strID
			End If

			objNode.Key = strKey
			objNode.Text = strText
			objNode.NavigateURL = strNavigateURL
			objNode.JSFunction = strJSFunction
			objNode.Target = strTarget
			objNode.ToolTip = strToolTip
			objNode.Enabled = blnEnabled
			objNode.CSSClass = strCSSClass
			objNode.CSSClassSelected = strCSSClassSelected
			objNode.CSSClassHover = strCSSClassHover

			Return intIndex			'objNode

		End Function

		Public Function Import(ByVal objNode As DNNNode) As Integer
			Return Import(objNode, True)
		End Function

		Public Function Import(ByVal objNode As DNNNode, ByVal blnDeep As Boolean) As Integer		' DNNNode
			Dim objXmlNode As Xml.XmlNode = Me.XMLDoc.ImportNode(objNode.XmlNode, blnDeep)
			Me.XMLNode.AppendChild(objXmlNode)

			Return Me.XMLNode.ChildNodes.Count - 1
		End Function

		Default Public Property Item(ByVal index As Integer) As DNNNode
			Get
				Return New DNNNode(MyBase.InnerXMLNode.ChildNodes(index))
			End Get
			Set(ByVal Value As DNNNode)
				'MyBase.InnerXMLNode.ChildNodes(index) = Value.XmlNode
				Throw New Exception("Cannot Assign Node Directly")
			End Set
		End Property

		Public Function IndexOf(ByVal value As DNNNode) As Integer
			Dim i As Integer
			For i = 0 To Me.XMLNode.ChildNodes.Count - 1
				If New DNNNode(Me.XMLNode.ChildNodes(i)).ID = value.ID Then
					Return i
				End If
			Next
		End Function

		Public Sub InsertAfter(ByVal index As Integer, ByVal value As DNNNode)
			Dim objXmlNode As XmlNode = Me.XMLDoc.ImportNode(value.XmlNode, True)
			Me.XMLNode.InsertAfter(objXmlNode, Me.XMLNode.ChildNodes(index))

		End Sub

		Public Sub InsertBefore(ByVal index As Integer, ByVal value As DNNNode)
			Dim objXmlNode As XmlNode = Me.XMLDoc.ImportNode(value.XmlNode, True)
			Me.XMLNode.InsertBefore(objXmlNode, Me.XMLNode.ChildNodes(index))
		End Sub

		Public Sub Remove(ByVal value As DNNNode)
			Me.XMLNode.RemoveChild(value.XmlNode)
		End Sub

		Public Sub Remove(ByVal index As Integer)
			Me.XMLNode.RemoveChild(Me.XMLNode.ChildNodes(index))
		End Sub

		'Protected Overrides Sub OnInsert(ByVal index As Integer, ByVal value As Object)
		'	If m_bNodeCollectionInterfaceCall = False Then Me.InsertAfter(index, CType(value, DNNNode))
		'End Sub

		'Protected Overrides Sub OnRemove(ByVal index As Integer, ByVal value As Object)
		'	If m_bNodeCollectionInterfaceCall = False Then Me.Remove(CType(value, DNNNode))
		'End Sub

		'Protected Overrides Sub OnSet(ByVal index As Integer, ByVal oldValue As Object, ByVal newValue As Object)
		'	Me.Item(index) = CType(newValue, DNNNode)
		'End Sub

		'Protected Overrides Sub OnClear()
		'	Me.Clear()
		'End Sub

		'Public Shadows Sub Clear()
		'	Dim i As Integer
		'	For i = Me.XMLNode.ChildNodes.Count - 1 To 0 Step -1
		'		Me.XMLNode.RemoveChild(Me.XMLNode.ChildNodes(i))
		'	Next
		'End Sub

		Public Function Contains(ByVal value As DNNNode) As Boolean
			If Me.FindNode(value.ID) Is Nothing Then
				Return False
			Else
				Return True
			End If
		End Function

        Friend Function FindFast(ByVal Key As String, ByVal Value As String, ByVal Parent As XMLNode, ByVal OptimizedForSmallData As Boolean) As XMLNode
            Try
                If OptimizedForSmallData Then
                    Dim objNode As XMLNode = Parent.FirstChild
                    While Not objNode Is Nothing
                        If Not objNode.Attributes(Key) Is Nothing AndAlso objNode.Attributes(Key).Value = Value Then
                            Return objNode
                        End If
                        If Parent.HasChildNodes Then
                            Dim objFound As XMLNode = FindFast(Key, Value, objNode, True)
                            If Not objFound Is Nothing Then Return objFound
                        End If
                        objNode = objNode.NextSibling
                    End While
                Else
                    Return Parent.SelectSingleNode(String.Format(".//n[@{0}='{1}']", Key, Value))
                End If
                Return Nothing
            Catch ex As Exception
                Throw ex
            End Try
        End Function

		Public Function FindNode(ByVal ID As String) As DNNNode
            Dim objNode As XMLNode = FindFast("id", ID, Me.XMLNode, True)
			If Not objNode Is Nothing Then Return New DNNNode(objNode)
			Return Nothing
		End Function

		Public Function FindNodeByKey(ByVal Key As String) As DNNNode
            Dim objNode As XMLNode = FindFast("key", Key, Me.XMLNode, True)
			If Not objNode Is Nothing Then Return New DNNNode(objNode)
			Return Nothing
		End Function


		Public Overridable Function FindSelectedNodes() As Collection
			Dim colNodes As Collection = New Collection
			If Not Me.XMLNode Is Nothing Then
				Dim objNodeList As XmlNodeList = Me.XMLNode.SelectNodes("//n[@selected='1']")
				Dim objNode As XMLNode
				For Each objNode In objNodeList
					colNodes.Add(New DNNNode(objNode))
				Next
			End If
			Return colNodes
		End Function

		Public Function ToXml() As String
			Return XMLDoc.OuterXml
		End Function

        Public Function ToJSON() As String
            Dim sb As System.Text.StringBuilder = New System.Text.StringBuilder()
            Dim hasOne As Boolean
            sb.Append("nodes:[")
            For Each node As DNNNode In Me
                If hasOne Then sb.Append(",")
                hasOne = True
                sb.Append(node.ToJSON(True))
            Next
            sb.Append("]")
            Return sb.ToString()
        End Function

        <Obsolete("Use LoadJSON IEnumerable instead")> _
        Public Sub LoadJSON(ByVal JSONNodes As ArrayList)
            Dim node As DNNNode
            For Each JSONNode As Dictionary(Of String, Object) In JSONNodes
                node = New DNNNode()
                For Each key As String In JSONNode.Keys
                    If key = "nodes" Then
                        node.DNNNodes.LoadJSON(CType(JSONNode("nodes"), IEnumerable))
                    Else
                        node.CustomAttribute(key) = JSONNode(key).ToString()
                    End If
                Next
                Me.Add(node)
            Next
        End Sub

        Public Sub LoadJSON(ByVal JSONNodes As IEnumerable)
            Dim node As DNNNode
            For Each JSONNode As Dictionary(Of String, Object) In JSONNodes
                node = New DNNNode()
                For Each key As String In JSONNode.Keys
                    If key = "nodes" Then
                        node.DNNNodes.LoadJSON(CType(JSONNode("nodes"), IEnumerable))
                    Else
                        node.CustomAttribute(key) = JSONNode(key).ToString()
                    End If
                Next
                Me.Add(node)
            Next
        End Sub


#Region "ICollection Implementation"
		Public Sub CopyTo(ByVal myArr As Array, ByVal index As Integer)		  'Implements ICollection.CopyTo
			Dim objNode As Xml.XmlNode
			For Each objNode In MyBase.InnerXMLNode.ChildNodes
				'myArr(index) = objNode
				myArr.SetValue(objNode, index)
				index = index + 1
			Next
		End Sub
		Public Shadows Sub RemoveAt(ByVal index As Integer)
			Me.Remove(index)
			Dim o As DNNNodeCollection
		End Sub

		Public Shadows Function GetEnumerator() As IEnumerator		  'Implements ICollection.GetEnumerator
			Return New DNNNodeEnumerator(MyBase.InnerXMLNode)
		End Function

		'The IsSynchronized Boolean property returns True if the 
		'collection is designed to be thread safe; otherwise, it returns False.
		'ReadOnly Property IsSynchronized() As Boolean		  'Implements ICollection.IsSynchronized
		'	Get
		'		Return False
		'	End Get
		'End Property

		'The SyncRoot property returns an object, which is used to synchronize 
		'the collection. This should return the instance of the object or return the 
		'SyncRoot of another collection if the collection contains other collections.
		'ReadOnly Property SyncRoot() As Object		  'Implements ICollection.SyncRoot
		'	Get
		'		Return Me
		'	End Get
		'End Property

		'The ReadOnly property Count returns the number 
		'of items in the custom collection.
		Shadows ReadOnly Property Count() As Integer		  'Implements ICollection.Count
			Get
				Return MyBase.InnerXMLNode.ChildNodes.Count
			End Get
		End Property
#End Region

#Region "Utility Functions"
		Private Function DoTransform(ByVal XML As String, ByVal XSL As String) As String
			Return DoTransform(XML, XSL, Nothing)
		End Function

		Private Function DoTransform(ByVal XML As String, ByVal XSL As String, ByVal Params As Xsl.XsltArgumentList) As String
			Try
				Dim oXsl As New System.Xml.Xsl.XslTransform
				oXsl.Load(XSL)
				Dim oXR As XmlTextReader = New XmlTextReader(XML, XmlNodeType.Document, Nothing)

				Dim oXml As XPath.XPathDocument = New XPath.XPathDocument(oXR)
				Dim oResolver As New XmlUrlResolver

				Dim oSB As System.Text.StringBuilder = New System.Text.StringBuilder
				Dim oWriter As System.io.StringWriter = New System.Io.StringWriter(oSB, Nothing)
				oXsl.Transform(oXml, Params, oWriter, oResolver)
				oWriter.Close()
				Return oSB.ToString

			Catch ex As Exception
				Throw ex
			End Try
		End Function

#End Region

	End Class

	Class DNNNodeEnumerator
		Implements IEnumerator

		Private m_objXMLNode As Xml.XmlNode
		Private m_intCursor As Integer

		Sub New(ByVal objRoot As Xml.XmlNode)
			m_objXMLNode = objRoot
			m_intCursor = -1
		End Sub

		Public Sub Reset() Implements IEnumerator.Reset
			m_intCursor = -1
		End Sub

		Public Function MoveNext() As Boolean Implements IEnumerator.MoveNext
			If m_intCursor < m_objXMLNode.ChildNodes.Count Then
				m_intCursor = m_intCursor + 1
			End If

			If (m_intCursor = m_objXMLNode.ChildNodes.Count) Then
				Return False
			Else
				Return True
			End If
		End Function

		Public ReadOnly Property Current() As Object Implements IEnumerator.Current
			Get
				If ((m_intCursor < 0) Or (m_intCursor = m_objXMLNode.ChildNodes.Count)) Then
					Throw New InvalidOperationException
				Else
					Return New DNNNode(m_objXMLNode.ChildNodes(m_intCursor))
				End If
			End Get
		End Property

	End Class
End Namespace