'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2009
' by DotNetNuke Corporation
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System
Imports System.Collections
Imports System.ComponentModel
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Diagnostics
Imports System.Xml
Imports DotNetNuke.UI.Utilities

Namespace DotNetNuke.UI.WebControls
	Public Class DNNLabelEdit
		Inherits Label
		Implements IPostBackEventHandler
		Implements DotNetNuke.UI.Utilities.IClientAPICallbackEventHandler
		Implements DotNetNuke.UI.WebControls.IDNNToolBar
		Implements DotNetNuke.UI.WebControls.IDNNToolBarSupportedActions

#Region "Events / Delegates"
		Public Delegate Sub DNNLabelEditEventHandler(ByVal source As Object, ByVal e As DNNLabelEditEventArgs)
		Public Event UpdateLabel As DNNLabelEditEventHandler
#End Region

#Region "Member Variables"
#End Region

#Region "Constructors"
		''' -----------------------------------------------------------------------------
		''' <summary>
		''' 
		''' </summary>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	10/10/2005	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Sub New()
		End Sub
#End Region

#Region "Properties"
        <Bindable(True), DefaultValue(""), ClientProperty(), ClientPropertyName("bid")> _
        Public Property BehaviorID() As String
            Get
                Return CType(ViewState("BehaviorID"), String)
            End Get
            Set(ByVal Value As String)
                ViewState("BehaviorID") = Value
            End Set
        End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Returns whether the LabelEdit will save when contol looses focus
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	2/22/2005	Created
		''' </history>
		''' -----------------------------------------------------------------------------
        <Category("Behavior"), DefaultValue(True), ClientProperty(), ClientPropertyName("blursave")> _
        Public Property LostFocusSave() As Boolean
            Get
                If Len(Me.ToolBarId) > 0 Then            'if using toolbar do not cause save to occur on lostfocus... defeats purpose.
                    Return False
                ElseIf Len(CStr(ViewState("LostFocusSave"))) = 0 Then
                    Return True
                Else
                    Return CType(ViewState("LostFocusSave"), Boolean)
                End If
            End Get
            Set(ByVal value As Boolean)
                ViewState("LostFocusSave") = value
            End Set
        End Property


		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Returns whether the LabelEdit will render DownLevel or not
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	2/22/2005	Commented
		'''		[Jon Henning]	3/9/2005	Requiring XML support on client for uplevel
		''' </history>
		''' -----------------------------------------------------------------------------
		<Browsable(False), Category("Appearance")> _
		  Public ReadOnly Property IsDownLevel() As Boolean
			Get
				If EditEnabled = False OrElse DotNetNuke.UI.Utilities.ClientAPI.BrowserSupportsFunctionality(Utilities.ClientAPI.ClientFunctionality.DHTML) = False OrElse _
				 DotNetNuke.UI.Utilities.ClientAPI.BrowserSupportsFunctionality(Utilities.ClientAPI.ClientFunctionality.XML) = False _
				Then
					Return True
				Else
					Return False
				End If
			End Get
		End Property

		<DefaultValue(True), PersistenceMode(PersistenceMode.Attribute), Category("Behavior")> _
		Public Property EditEnabled() As Boolean
			Get
				If Len(ViewState("EditEnabled")) > 0 Then
					Return CType(ViewState("EditEnabled"), Boolean)
				Else
					Return True
				End If
			End Get
			Set(ByVal Value As Boolean)
				ViewState("EditEnabled") = Value
			End Set
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Location of ClientAPI js files 
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	2/22/2005	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		<Category("Paths")> _
		Public Property ClientAPIScriptPath() As String
			Get
				Return DotNetNuke.UI.Utilities.ClientAPI.ScriptPath
			End Get
			Set(ByVal Value As String)
				' Commented out because it was breaking the build.. (SM 26 Oct)  
				' uncommented out, don't understand how this could have broke build - JH 22 Feb 2005
				DotNetNuke.UI.Utilities.ClientAPI.ScriptPath = Value
			End Set
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Location of dnn.controls.DNNLabelEdit.js file
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' Since 1.1 this path will be the same as the ClientAPI path.
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	2/22/2005	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		<Category("Paths")> _
		Public Property LabelEditScriptPath() As String
			Get
				If Len(ViewState("LabelEditScriptPath")) = 0 Then
					Return ClientAPIScriptPath
				Else
					Return CStr(ViewState("LabelEditScriptPath"))
				End If
			End Get
			Set(ByVal Value As String)
				ViewState("LabelEditScriptPath") = Value
			End Set
		End Property


		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Location of system images (i.e. spacer.gif)
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	2/22/2005	Created
		''' </history>
		''' -----------------------------------------------------------------------------
        <Description("Directory to find the images for the LabelEdit.  Need to have spacer.gif here!"), DefaultValue("images/"), Category("Paths"), ClientProperty(), ClientPropertyName("sysimgpath")> _
        Public Property SystemImagesPath() As String
            Get
                If Len(ViewState("SystemImagesPath")) = 0 Then
                    Return "images/"
                Else
                    Return CStr(ViewState("SystemImagesPath"))
                End If
            End Get
            Set(ByVal Value As String)
                ViewState("SystemImagesPath") = Value
            End Set
        End Property

        <Bindable(True), DefaultValue("onclick"), PersistenceMode(PersistenceMode.Attribute), Category("Behavior"), Description("Client-side event that will trigger an edit.  (onclick, ondblclick, none)"), ClientProperty(), ClientPropertyName("eventName")> _
        Public Property EventName() As String
            Get
                If Len(ViewState("EventName")) = 0 Then
                    Return "onclick"
                Else
                    Return CStr(ViewState("EventName"))
                End If
            End Get
            Set(ByVal Value As String)
                ViewState("EventName") = Value
            End Set
        End Property

        <DefaultValue(True), PersistenceMode(PersistenceMode.Attribute), Category("Behavior"), ClientProperty(), ClientPropertyName("saveonenter")> _
        Public Property SaveOnEnter() As Boolean
            Get
                If Len(ViewState("SaveOnEnter")) = 0 Then
                    Return True
                Else
                    Return CBool(ViewState("SaveOnEnter"))
                End If
            End Get
            Set(ByVal Value As Boolean)
                ViewState("SaveOnEnter") = Value
            End Set
        End Property

        <DefaultValue(False), PersistenceMode(PersistenceMode.Attribute), Category("Behavior"), ClientProperty(), ClientPropertyName("multiline")> _
        Public Property MultiLine() As Boolean
            Get
                If Len(ViewState("MultiLine")) = 0 Then
                    Return False
                Else
                    Return CBool(ViewState("MultiLine"))
                End If
            End Get
            Set(ByVal Value As Boolean)
                ViewState("MultiLine") = Value
            End Set
        End Property

        <DefaultValue(UrlFormatType.AbsoluteWithoutServer), PersistenceMode(PersistenceMode.Attribute), Category("Behavior")> _
         Public Property UrlFormat() As UrlFormatType
            Get
                If Len(ViewState("UrlFormat")) = 0 Then
                    Return UrlFormatType.AbsoluteWithoutServer
                Else
                    Return CType(ViewState("UrlFormat"), UrlFormatType)
                End If
            End Get
            Set(ByVal Value As UrlFormatType)
                ViewState("UrlFormat") = Value
            End Set
        End Property

        <DefaultValue(False), PersistenceMode(PersistenceMode.Attribute), Category("Behavior"), ClientProperty(), ClientPropertyName("richtext")> _
         Public Property RichTextEnabled() As Boolean
            Get
                If Len(ViewState("RichTextEnabled")) = 0 Then
                    Return False
                Else
                    Return CBool(ViewState("RichTextEnabled"))
                End If
            End Get
            Set(ByVal Value As Boolean)
                ViewState("RichTextEnabled") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' If callbacks are supported/enabled, this javascript function will be invoked
        ''' with each status change of the xmlhttp request.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' The Javascript transport does not raise any events.
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	4/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), PersistenceMode(PersistenceMode.Attribute), Category("Behavior"), ClientProperty(), ClientPropertyName("callbackSF")> _
        Public Property CallbackStatusFunction() As String
            Get
                Return (CType(ViewState("CallbackStatusFunction"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("CallbackStatusFunction") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' This property determines what type of callback is used
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' The Simple callback short-circuits at the Page Init event while the ProcessPage
        ''' will run the page through the entire page's lifecycle.
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	9/14/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), DefaultValue(ClientAPICallBackResponse.CallBackTypeCode.Simple), Description("Image to display during a callback"), Category("Behavior")> _
        Public Property CallBackType() As ClientAPICallBackResponse.CallBackTypeCode
            Get
                If Len(ViewState("CallBackTypeCode")) > 0 Then
                    Return CType(ViewState("CallBackTypeCode"), ClientAPICallBackResponse.CallBackTypeCode)
                Else
                    Return ClientAPICallBackResponse.CallBackTypeCode.Simple
                End If
            End Get
            Set(ByVal Value As ClientAPICallBackResponse.CallBackTypeCode)
                ViewState("CallBackTypeCode") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' This property allows for data on page to be posted in callback
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	9/17/2008	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), Description("This property allows for data on page to be posted in callback"), Category("Behavior")> _
        Public Property CallBackPostChildrenOf() As String
            Get
                If Len(ViewState("CallBackPostChildrenOf")) > 0 Then
                    Return CStr(ViewState("CallBackPostChildrenOf"))
                Else
                    Return ""
                End If
            End Get
            Set(ByVal Value As String)
                ViewState("CallBackPostChildrenOf") = Value
            End Set
        End Property


        <Category("Behavior")> _
        Public Property ToolBarId() As String Implements IDNNToolBar.ToolBarId
            Get
                Return (CType(ViewState("ToolBarId"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("ToolBarId") = Value
                TypeDescriptor.Refresh(Me)     'make sure we refresh the property grid
            End Set
        End Property

        <Browsable(False)> _
         Friend ReadOnly Property ToolBarSupportedActions() As String() Implements IDNNToolBarSupportedActions.Actions
            Get
                Return New String() {"edit", "save", "cancel", "bold", _
                  "italic", "underline", "justifyleft", "justifycenter", "justifyright", _
                  "insertorderedlist", "insertunorderedlist", "outdent", "indent", "createlink"}
            End Get
        End Property

        <DefaultValue("onmousemove"), Category("Behavior"), Description("Allows the client-side event that displays the toolbar to be configured (onmouseover, onclick)"), ClientProperty(), ClientPropertyName("tbEvent")> _
        Public Property ShowToolBarEventName() As String
            Get
                If Len(ViewState("ShowToolBarEventName")) = 0 Then
                    Return "onmousemove"
                Else
                    Return CStr(ViewState("ShowToolBarEventName"))
                End If
            End Get
            Set(ByVal Value As String)
                ViewState("ShowToolBarEventName") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' This javascript function will be invoked just before persisting the data
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' To cancel a tab from changing return false from this function
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/7/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), PersistenceMode(PersistenceMode.Attribute), Category("Behavior"), ClientProperty(), ClientPropertyName("beforeSaveF")> _
        Public Property BeforeSaveFunction() As String
            Get
                Return (CType(ViewState("BeforeSaveFunction"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("BeforeSaveFunction") = Value
            End Set
        End Property

        <Bindable(True), DefaultValue(""), PersistenceMode(PersistenceMode.Attribute), Category("Appearance"), ClientProperty(), ClientPropertyName("cssEdit")> _
         Public Property LabelEditCssClass() As String
            Get
                Return (CType(ViewState("LabelEditCssClass"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("LabelEditCssClass") = Value
            End Set
        End Property

        <Bindable(True), DefaultValue(""), PersistenceMode(PersistenceMode.Attribute), Category("Appearance"), ClientProperty(), ClientPropertyName("cssWork")> _
         Public Property WorkCssClass() As String
            Get
                Return (CType(ViewState("WorkCssClass"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("WorkCssClass") = Value
            End Set
        End Property

        <DefaultValue(""), PersistenceMode(PersistenceMode.Attribute), Category("Appearance"), ClientProperty(), ClientPropertyName("cssOver")> _
        Public Property MouseOverCssClass() As String
            Get
                Return (CType(ViewState("MouseOverCssClass"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("MouseOverCssClass") = Value
            End Set
        End Property

        <DefaultValue(False), Category("Misc")> _
        Public Property RenderAsDiv() As Boolean
            Get
                Return (CType(ViewState("RenderAsDiv"), Boolean))
            End Get
            Set(ByVal Value As Boolean)
                ViewState("RenderAsDiv") = Value
            End Set
        End Property

        Protected Overrides ReadOnly Property TagName() As String
            Get
                If Me.RenderAsDiv Then
                    Return "div"
                Else
                    Return "span"
                End If
            End Get
        End Property

        Protected Overrides ReadOnly Property TagKey() As HtmlTextWriterTag
            Get
                If Me.RenderAsDiv Then
                    Return HtmlTextWriterTag.Div
                Else
                    Return HtmlTextWriterTag.Span
                End If
            End Get
        End Property

#End Region

#Region "OverRidden Methods"

        'Protected Overrides Sub AddAttributesToRender(ByVal writer As System.Web.UI.HtmlTextWriter)
        '	MyBase.AddAttributesToRender(writer)
        '	'writer.AddAttribute(HtmlTextWriterAttribute.Class, Me.CssClass)
        '	If Me.EditEnabled Then
        '              'writer.AddAttribute("sysimgpath", Me.SystemImagesPath)
        '              'If Len(Me.LabelEditCssClass) > 0 Then
        '              '	writer.AddAttribute("cssEdit", Me.LabelEditCssClass)
        '              'End If
        '              'If Len(Me.WorkCssClass) > 0 Then
        '              '	writer.AddAttribute("cssWork", Me.WorkCssClass)
        '              'End If

        '              'If Len(Me.MouseOverCssClass) > 0 Then
        '              '	writer.AddAttribute("cssOver", Me.MouseOverCssClass)
        '              'End If

        '              'If Me.EventName <> "onclick" Then
        '              '	writer.AddAttribute("eventName", Me.EventName)
        '              'End If

        '              'If Me.MultiLine Then
        '              '	writer.AddAttribute("multiline", "1")
        '              'End If

        '              'If Me.RichTextEnabled Then
        '              '	writer.AddAttribute("richtext", "1")
        '              'End If

        '		If DotNetNuke.UI.Utilities.ClientAPI.BrowserSupportsFunctionality(Utilities.ClientAPI.ClientFunctionality.XMLHTTP) Then
        '			writer.AddAttribute("callback", DotNetNuke.UI.Utilities.ClientAPI.GetCallbackEventReference(Me, "'[TEXT]'", "this.callBackSuccess", "this", "this.callBackFail", "this.callBackStatus", CallBackType))
        '		End If

        '              'If Len(Me.CallbackStatusFunction) > 0 Then
        '              '	writer.AddAttribute("callbackSF", Me.CallbackStatusFunction)
        '              'End If

        '              'If Len(Me.BeforeSaveFunction) > 0 Then
        '              '	writer.AddAttribute("beforeSaveF", BeforeSaveFunction)
        '              'End If

        '		If Len(Me.ToolBarId) > 0 Then
        '			writer.AddAttribute("tbId", ToolBarId)
        '			If Len(Me.UniqueID.Replace(Me.ID, "")) > 0 Then
        '				'writer.AddAttribute("nsPrefix", Me.UniqueID.Replace(Me.ID, ""))
        '				writer.AddAttribute("nsPrefix", Me.NamingContainer.UniqueID)						 'TEST ???
        '			End If
        '		End If
        '              'If Me.ShowToolBarEventName <> "onmousemove" Then
        '              '	writer.AddAttribute("tbEvent", Me.ShowToolBarEventName)
        '              'End If

        '              'If Me.LostFocusSave = False Then
        '              '	writer.AddAttribute("blursave", "0")
        '              'End If
        '	End If
        'End Sub

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' 
		''' </summary>
		''' <returns></returns>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	10/10/2005	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Protected Overloads Overrides Function SaveViewState() As Object
			Dim _baseState As Object = MyBase.SaveViewState
			Dim objState(0) As Object
			objState(0) = _baseState
			Return objState
		End Function

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' 
		''' </summary>
		''' <param name="state"></param>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	10/10/2005	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Protected Overloads Overrides Sub LoadViewState(ByVal state As Object)
			If Not (state Is Nothing) Then
				Dim objState As Object() = CType(state, Object())
				If Not (objState(0) Is Nothing) Then
					MyBase.LoadViewState(objState(0))
				End If
			End If
		End Sub

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' 
		''' </summary>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	10/10/2005	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Protected Overloads Overrides Sub TrackViewState()
			MyBase.TrackViewState()
			'CType(Root, IStateManager).TrackViewState()
		End Sub
#End Region

#Region "Event Handlers"

		Public Overridable Sub RaisePostBackEvent(ByVal eventArgument As String) Implements IPostBackEventHandler.RaisePostBackEvent
			RaiseEvent UpdateLabel(Me, New DNNLabelEditEventArgs(eventArgument))
		End Sub

        Public Function RaiseClientAPICallbackEvent(ByVal eventArgument As String) As String Implements Utilities.IClientAPICallbackEventHandler.RaiseClientAPICallbackEvent
            If Me.UrlFormat <> UrlFormatType.None Then
                eventArgument = WebControls.StripUrlPaths(eventArgument, Me.UrlFormat, Me.Page)
            End If
            Dim args As DNNLabelEditEventArgs = New DNNLabelEditEventArgs(eventArgument)
            RaiseEvent UpdateLabel(Me, args)
            Return args.ReturnedText
        End Function

#End Region

#Region "Methods"

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Determines if client supports an uplevel rendering, and if so it registers
		''' the appropriate scripts.
		''' </summary>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	4/6/2005	Commented
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Sub RegisterClientScript()
			If IsDownLevel = False AndAlso Me.EditEnabled Then
				DotNetNuke.UI.Utilities.ClientAPI.RegisterClientReference(Me.Page, DotNetNuke.UI.Utilities.ClientAPI.ClientNamespaceReferences.dnn_dom)
                'DotNetNuke.UI.Utilities.ClientAPI.RegisterClientReference(Me.Page, DotNetNuke.UI.Utilities.ClientAPI.ClientNamespaceReferences.dnn_xml)
				DotNetNuke.UI.Utilities.ClientAPI.RegisterClientReference(Me.Page, DotNetNuke.UI.Utilities.ClientAPI.ClientNamespaceReferences.dnn_dom_positioning)
				DotNetNuke.UI.Utilities.ClientAPI.RegisterClientReference(Me.Page, DotNetNuke.UI.Utilities.ClientAPI.ClientNamespaceReferences.dnn_xmlhttp)

                'If Not ClientAPI.IsClientScriptBlockRegistered(Me.Page, "dnn.controls.dnnlabeledit.js") Then
                '	ClientAPI.RegisterClientScriptBlock(Me.Page, "dnn.controls.dnnlabeledit.js", "<script src=""" & LabelEditScriptPath & "dnn.controls.dnnlabeledit.js""></script>")
                'End If
                RegisterClientScriptBlock(Me.Page, "dnn.controls.js")
                RegisterClientScriptBlock(Me.Page, "dnn.controls.dnnlabeledit.js")
                If ClientAPI.UseExternalScripts = False Then
                    ClientAPI.RegisterEmbeddedResource(Me.Page, "dnn.controls.dnninputtext.js", Me.GetType())
                    ClientAPI.RegisterEmbeddedResource(Me.Page, "dnn.controls.dnnrichtext.js", Me.GetType())
                End If

				'ClientAPI.RegisterStartUpScript(Page, Me.ClientID & "_startup", "<script type="text/javascript">dnn.controls.initLabelEdit(dnn.dom.getById('" & Me.ClientID & "'));</script>")				   'wrong place
			End If
		End Sub

#End Region

		Private Sub DNNLabelEdit_PreRender(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.PreRender
			RegisterClientScript()
		End Sub

        Protected Overrides Sub Render(ByVal writer As System.Web.UI.HtmlTextWriter)
            MyBase.Render(writer)
            If IsDownLevel = False AndAlso Me.EditEnabled Then
                Dim properties As Hashtable = Me.MarshalledProperties() 'get runtime generated properties (non attribute based)
                Dim json As String = ScriptGenerator.GetMarshalledPropertyJSON(Me, properties)
                BaseWebControl.RegisterInitialize(Me, "initLabelEdit", json)
            End If
        End Sub

        Protected Function MarshalledProperties() As Hashtable
            Dim properties As Hashtable = New Hashtable
            If Me.EditEnabled Then
                If DotNetNuke.UI.Utilities.ClientAPI.BrowserSupportsFunctionality(Utilities.ClientAPI.ClientFunctionality.XMLHTTP) Then
                    properties.Add("callback", DotNetNuke.UI.Utilities.ClientAPI.GetCallbackEventReference(Me, "'[TEXT]'", "this.callBackSuccess", "this", "this.callBackFail", "this.callBackStatus", CallBackPostChildrenOf, CallBackType))
                End If

                If Len(Me.ToolBarId) > 0 Then
                    properties.Add("tbId", ToolBarId)
                    If Len(Me.UniqueID.Replace(Me.ID, "")) > 0 Then
                        properties.Add("nsPrefix", Me.NamingContainer.UniqueID)                         'TEST ???
                    End If
                End If
            End If
            Return properties
        End Function


    End Class
End Namespace
