'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2009
' by DotNetNuke Corporation
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System
Imports System.Collections
Imports System.ComponentModel
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Diagnostics
Imports System.Xml
Imports System.Collections.Generic
Imports DotNetNuke.UI.Utilities

Namespace DotNetNuke.UI.WebControls
	<Designer(GetType(DNNMenuDesigner))> _
	 Public Class DNNMenu
		Inherits WebControl
		Implements IPostBackEventHandler
		Implements IPostBackDataHandler
		Implements DotNetNuke.UI.Utilities.IClientAPICallbackEventHandler

        Public Enum MenuRenderMode
            Normal
            NoTables
            UnorderedList
            DownLevel
        End Enum

#Region "Events / Delegates"
		Public Delegate Sub DNNMenuEventHandler(ByVal source As Object, ByVal e As DNNMenuEventArgs)
		Public Delegate Sub DNNMenuNodeClickHandler(ByVal source As Object, ByVal e As DNNMenuNodeClickEventArgs)

		Public Event NodeClick As DNNMenuNodeClickHandler
		Public Event PopulateOnDemand As DNNMenuEventHandler
#End Region

#Region "Member Variables"
        Private m_Nodes As MenuNodeCollection
        Private m_Images As NodeImageCollection
        Private m_Writer As IDNNMenuWriter
        Private m_DNNAnimation As DNNAnimation = New DNNAnimation
#End Region

#Region "Constructors"
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Sub New()
        End Sub
#End Region

#Region "Properties"
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Allows developer to force the rendering of the Menu in DownLevel mode
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/22/2005	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Property ForceDownLevel() As Boolean
            Get
                Return CType(ViewState("ForceDownLevel"), Boolean)
            End Get
            Set(ByVal Value As Boolean)
                ViewState("ForceDownLevel") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns whether the Menu will render DownLevel or not
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/22/2005	Commented
        '''		[Jon Henning]	3/9/2005	Requiring XML support on client for uplevel
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Browsable(False)> _
          Public ReadOnly Property IsDownLevel() As Boolean
            Get
                If ForceDownLevel OrElse RenderMode = MenuRenderMode.DownLevel OrElse ClientAPI.BrowserSupportsFunctionality(Utilities.ClientAPI.ClientFunctionality.DHTML) = False Then
                    Return True
                Else
                    Return False
                End If
            End Get
        End Property

        Public Property IsCrawler() As Boolean
            Get
                If Len(ViewState("IsCrawler")) = 0 Then
                    Return System.Web.HttpContext.Current.Request.Browser.Crawler
                Else
                    Return CBool(ViewState("IsCrawler"))
                End If

            End Get
            Set(ByVal Value As Boolean)
                ViewState("IsCrawler") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Location of ClientAPI js files 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/22/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Property ClientAPIScriptPath() As String
            Get
                Return ClientAPI.ScriptPath
            End Get
            Set(ByVal Value As String)
                ' Commented out because it was breaking the build.. (SM 26 Oct)  
                ' uncommented out, don't understand how this could have broke build - JH 22 Feb 2005
                ClientAPI.ScriptPath = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Location of dnn.controls.DNNMenu.js file
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' Since 1.1 this path will be the same as the ClientAPI path.
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/22/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Property MenuScriptPath() As String
            Get
                If Len(ViewState("MenuScriptPath")) = 0 Then
                    Return ClientAPIScriptPath
                Else
                    Return CStr(ViewState("MenuScriptPath"))
                End If
            End Get
            Set(ByVal Value As String)
                ViewState("MenuScriptPath") = Value
            End Set
        End Property


        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Location of system images (i.e. spacer.gif)
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/22/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Description("Directory to find the images for the menu.  Need to have spacer.gif here!"), DefaultValue("images/"), ClientProperty(), ClientPropertyName("sysimgpath"), UrlProperty()> _
        Public Property SystemImagesPath() As String
            Get
                If Len(ViewState("SystemImagesPath")) = 0 Then
                    Return "images/"
                Else
                    Return CStr(ViewState("SystemImagesPath"))
                End If
            End Get
            Set(ByVal Value As String)
                ViewState("SystemImagesPath") = Value
            End Set
        End Property


        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Browsable(True), PersistenceMode(PersistenceMode.InnerProperty)> _
        Public ReadOnly Property MenuNodes() As MenuNodeCollection
            Get
                If m_Nodes Is Nothing Then
                    m_Nodes = New MenuNodeCollection(Me.ClientID, Me)
                End If
                Return m_Nodes
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Holds collection of selected node objects
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	4/6/2005	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Browsable(False)> _
        Public ReadOnly Property SelectedMenuNodes() As Collection
            Get
                Return Me.MenuNodes.FindSelectedNodes()
            End Get
        End Property


        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), DefaultValue(""), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("rarrowimg"), UrlProperty()> _
        Public Property RootArrowImage() As String
            Get
                Return CType(ViewState("RootArrowImage"), String)
            End Get
            Set(ByVal Value As String)
                ViewState("RootArrowImage") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), DefaultValue(""), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("carrowimg"), UrlProperty()> _
        Public Property ChildArrowImage() As String
            Get
                Return CType(ViewState("ChildArrowImage"), String)
            End Get
            Set(ByVal Value As String)
                ViewState("ChildArrowImage") = Value
            End Set
        End Property

        <Bindable(True), DefaultValue("dnnanim.gif"), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("workimg")> _
        Public Property WorkImage() As String
            Get
                Return CType(ViewState("WorkImage"), String)
            End Get
            Set(ByVal Value As String)
                ViewState("WorkImage") = Value
            End Set
        End Property

        <Bindable(True), DefaultValue(""), ClientProperty(), ClientPropertyName("bid")> _
        Public Property BehaviorID() As String
            Get
                Return CType(ViewState("BehaviorID"), String)
            End Get
            Set(ByVal Value As String)
                ViewState("BehaviorID") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), DefaultValue(""), ClientProperty(), ClientPropertyName("target")> _
        Public Property Target() As String
            Get
                Return (CType(ViewState("Target"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("Target") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Browsable(True), PersistenceMode(PersistenceMode.InnerProperty)> _
          Public ReadOnly Property ImageList() As NodeImageCollection
            Get
                If m_Images Is Nothing Then
                    m_Images = New NodeImageCollection
                    If IsTrackingViewState Then
                        CType(m_Images, IStateManager).TrackViewState()
                    End If
                End If
                Return m_Images
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), DefaultValue(""), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("css")> _
        Public Property DefaultNodeCssClass() As String
            Get
                Return Me.CssClass
            End Get
            Set(ByVal Value As String)
                Me.CssClass = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), DefaultValue(""), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("csschild")> _
        Public Property DefaultChildNodeCssClass() As String
            Get
                Return (CType(ViewState("DefaultChildNodeCssClass"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("DefaultChildNodeCssClass") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), DefaultValue(""), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("csshover")> _
        Public Property DefaultNodeCssClassOver() As String
            Get
                Return (CType(ViewState("DefaultNodeCssClassOver"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("DefaultNodeCssClassOver") = Value
            End Set
        End Property

        <Bindable(True), DefaultValue(""), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("csssel")> _
        Public Property DefaultNodeCssClassSelected() As String
            Get
                Return (CType(ViewState("DefaultNodeCssClassSelected"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("DefaultNodeCssClassSelected") = Value
            End Set
        End Property

        <Bindable(True), DefaultValue(""), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("cssicon")> _
        Public Property DefaultIconCssClass() As String
            Get
                Return (CType(ViewState("DefaultIconCssClass"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("DefaultIconCssClass") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), DefaultValue(""), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("mbcss")> _
        Public Property MenuBarCssClass() As String
            Get
                Return (CType(ViewState("MenuBarCssClass"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("MenuBarCssClass") = Value
            End Set
        End Property

        <Bindable(True), DefaultValue(""), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("mcss")> _
        Public Property MenuCssClass() As String
            Get
                Return (CType(ViewState("MenuCssClass"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("MenuCssClass") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private ReadOnly Property MenuWriter() As IDNNMenuWriter
            Get
                If m_Writer Is Nothing Then
                    If Me.IsDownLevel Then
                        'RenderMode = MenuRenderMode.DownLevel
                        RenderMode = MenuRenderMode.UnorderedList
                    ElseIf Me.IsCrawler Then
                        RenderMode = MenuRenderMode.UnorderedList
                    End If
                    Select Case RenderMode
                        Case MenuRenderMode.Normal, MenuRenderMode.NoTables
                            m_Writer = New DNNMenuUpLevelWriter
                        Case MenuRenderMode.UnorderedList
                            m_Writer = New DNNMenuListWriter()
                        Case MenuRenderMode.DownLevel
                            m_Writer = New DNNMenuWriter(Me.IsCrawler)
                    End Select
                End If
                Return m_Writer
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Allows you to have a common JS function be invoked for all nodes, unless
        ''' a different JS function is provided on the node level.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' If the client-side function returns false, the action associated to the node 
        ''' selection will be canceled
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	4/6/2005	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <DefaultValue(""), ClientProperty(), ClientPropertyName("js")> _
        Public Property JSFunction() As String
            Get
                Return (CType(ViewState("JSFunction"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("JSFunction") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Allows the nodes to be populated on the client 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	4/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(False), DefaultValue(True), PersistenceMode(PersistenceMode.Attribute)> _
          Public Property PopulateNodesFromClient() As Boolean
            Get
                'only normal and notable support pod
                Select Case RenderMode
                    Case MenuRenderMode.DownLevel, MenuRenderMode.UnorderedList
                        Return False
                End Select
                If ViewState("PopNodesFromClient") Is Nothing Then
                    Return True
                Else
                    Return (CType(ViewState("PopNodesFromClient"), Boolean))
                End If
            End Get
            Set(ByVal Value As Boolean)
                ViewState("PopNodesFromClient") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' If callbacks are supported/enabled, this javascript function will be invoked
        ''' with each status change of the xmlhttp request.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' The Javascript transport does not raise any events.
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	4/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), PersistenceMode(PersistenceMode.Attribute)> _
        Public Property CallbackStatusFunction() As String
            Get
                Return (CType(ViewState("CallbackStatusFunction"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("CallbackStatusFunction") = Value
            End Set
        End Property

        <Bindable(True), DefaultValue(DotNetNuke.UI.WebControls.Orientation.Horizontal), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("orient")> _
        Public Property Orientation() As DotNetNuke.UI.WebControls.Orientation
            Get
                If Len(ViewState("Orientation")) > 0 Then
                    Return (CType(ViewState("Orientation"), Orientation))                     '?
                Else
                    Return Orientation.Horizontal
                End If
            End Get
            Set(ByVal Value As DotNetNuke.UI.WebControls.Orientation)
                ViewState("Orientation") = Value
            End Set
        End Property

        <Bindable(True), DefaultValue(DotNetNuke.UI.WebControls.Orientation.Vertical), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("suborient")> _
        Public Property SubMenuOrientation() As DotNetNuke.UI.WebControls.Orientation
            Get
                If Len(ViewState("SubMenuOrientation")) > 0 Then
                    Return (CType(ViewState("SubMenuOrientation"), Orientation))                   '?
                Else
                    Return Orientation.Vertical
                End If
            End Get
            Set(ByVal Value As DotNetNuke.UI.WebControls.Orientation)
                ViewState("SubMenuOrientation") = Value
            End Set
        End Property

        <DefaultValue(MenuRenderMode.Normal), ClientProperty(), ClientPropertyName("rmode")> _
        Public Property RenderMode() As MenuRenderMode
            Get
                If Len(CStr(ViewState("RenderMode"))) = 0 Then
                    Return MenuRenderMode.Normal
                Else
                    Return CType(ViewState("RenderMode"), MenuRenderMode)
                End If
            End Get
            Set(ByVal Value As MenuRenderMode)
                ViewState("RenderMode") = Value
            End Set
        End Property

        <PersistenceMode(PersistenceMode.Attribute), DefaultValue(True), Obsolete("Use RenderMode")> _
        Public Property UseTables() As Boolean
            Get
                RenderMode = MenuRenderMode.Normal
            End Get
            Set(ByVal Value As Boolean)
                If Value Then
                    RenderMode = MenuRenderMode.Normal
                Else
                    RenderMode = MenuRenderMode.NoTables
                End If
            End Set
        End Property

        <Description("Number of milliseconds to wait befor hiding sub-menu on mouse out"), Category("Behavior"), PersistenceMode(PersistenceMode.Attribute), DefaultValue(500), ClientProperty(), ClientPropertyName("moutdelay")> _
          Public Property MouseOutDelay() As Integer
            Get
                If Len(CStr(ViewState("MouseOutDelay"))) = 0 Then
                    Return 500
                Else
                    Return CType(ViewState("MouseOutDelay"), Integer)
                End If
            End Get
            Set(ByVal Value As Integer)
                ViewState("MouseOutDelay") = Value
            End Set
        End Property

        <Description("Number of milliseconds to wait befor displaying sub-menu on mouse over"), Category("Behavior"), PersistenceMode(PersistenceMode.Attribute), DefaultValue(250), ClientProperty(), ClientPropertyName("mindelay")> _
          Public Property MouseInDelay() As Integer
            Get
                If Len(CStr(ViewState("MouseInDelay"))) = 0 Then
                    Return 250
                Else
                    Return CType(ViewState("MouseInDelay"), Integer)
                End If
            End Get
            Set(ByVal Value As Integer)
                ViewState("MouseInDelay") = Value
            End Set
        End Property

        <PersistenceMode(PersistenceMode.Attribute), DefaultValue(False), ClientProperty(), ClientPropertyName("enablepbstate")> _
          Public Property EnablePostbackState() As Boolean
            Get
                If Len(CStr(ViewState("EnablePostbackState"))) = 0 Then
                    Return False
                Else
                    Return CType(ViewState("EnablePostbackState"), Boolean)
                End If
            End Get
            Set(ByVal Value As Boolean)
                ViewState("EnablePostbackState") = Value
            End Set
        End Property

        <Category("Appearance"), Description("The animation to be applied to showing/hiding the menus"), DesignerSerializationVisibility(DesignerSerializationVisibility.Content), NotifyParentProperty(True)> _
        Public ReadOnly Property Animation() As DNNAnimation
            Get
                Return m_DNNAnimation
            End Get
        End Property

#End Region

#Region "OverLoaded Methods"

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Responsible for rendering the DNNMenu and in an uplevel rendering of the Menu, sending down the xml for the child nodes in a ClientAPI variable.
        ''' </summary>
        ''' <param name="writer"></param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        '''		[Jon Henning 4/6/2005	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Protected Overloads Overrides Sub Render(ByVal writer As HtmlTextWriter)
            MenuWriter.RenderMenu(writer, Me)
            'added back...??? not sure if this is ok... - urllist needs it
            'If Me.IsDownLevel = False Then ClientAPI.RegisterClientVariable(Me.Page, Me.ClientID & "_xml", Me.MenuNodes.ToXml, True)
            Dim oCtl As Control
            For Each oCtl In Me.Controls
                oCtl.RenderControl(writer)
            Next

            If IsDownLevel = False Then
                Dim properties As Hashtable = Me.MarshalledProperties() 'get runtime generated properties (non attribute based)
                ScriptGenerator.GetMarshalledPropertyJSON(m_DNNAnimation, properties)  'also include animation properties
                Dim json As String = ScriptGenerator.GetMarshalledPropertyJSON(Me, properties)
                BaseWebControl.RegisterInitialize(Me, "initMenu", json)
            End If
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <param name="e"></param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Protected Overloads Overrides Sub OnInit(ByVal e As EventArgs)
            MyBase.OnInit(e)
            If ClientAPI.NeedsDNNVariable(Me) Then
                'This is to allow us to add a control to our parent's control collection...  kindof a hack
                AddHandler Me.Page.Load, AddressOf ParentOnLoad
            Else
                'LoadPostedXML()
                LoadPostedJSON()
            End If
        End Sub

        Protected Sub ParentOnLoad(ByVal Sender As Object, ByVal e As System.EventArgs)
            ClientAPI.RegisterDNNVariableControl(Me)

            'LoadPostedXML()
            LoadPostedJSON()
        End Sub


        Protected Overrides Sub OnLoad(ByVal e As System.EventArgs)
            If IsDownLevel AndAlso Len(ViewState("xml")) > 0 Then m_Nodes = New MenuNodeCollection(CStr(ViewState("xml")), "", Me)

            'RegisterClientScript()
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Protected Overloads Overrides Function SaveViewState() As Object
            Dim _baseState As Object = MyBase.SaveViewState
            Dim _imagesState As Object = CType(ImageList, IStateManager).SaveViewState
            Dim _newState(1) As Object
            _newState(0) = _baseState
            _newState(1) = _imagesState
            Return _newState
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <param name="state"></param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Protected Overloads Overrides Sub LoadViewState(ByVal state As Object)
            If Not (state Is Nothing) Then
                Dim _newState As Object() = CType(state, Object())
                If Not (_newState(0) Is Nothing) Then
                    MyBase.LoadViewState(_newState(0))
                End If
                If Not (_newState(1) Is Nothing) Then
                    CType(ImageList, IStateManager).LoadViewState(_newState(1))
                End If
            End If
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Protected Overloads Overrides Sub TrackViewState()
            MyBase.TrackViewState()
            'CType(Root, IStateManager).TrackViewState()
        End Sub
#End Region

#Region "Event Handlers"
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <param name="eventArgument"></param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Overridable Sub RaisePostBackEvent(ByVal eventArgument As String) Implements IPostBackEventHandler.RaisePostBackEvent
            Dim args() As String = Split(eventArgument, ClientAPI.COLUMN_DELIMITER)

            Dim Node As MenuNode = MenuNodes.FindNode(args(0))

            If Not (Node Is Nothing) Then
                If args.Length > 1 Then
                    Select Case args(1)
                        Case "Click"
                            Node.Click()
                        Case "Checked"
                            Node.Selected = Not Node.Selected
                        Case "OnDemand"
                            RaiseEvent PopulateOnDemand(Me, New DNNMenuEventArgs(Node))
                    End Select
                Else
                    'assume an event with no specific argument to be a click 
                    Node.Click()
                End If
            End If
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <param name="e"></param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	10/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Overridable Sub OnNodeClick(ByVal e As DNNMenuNodeClickEventArgs)
            RaiseEvent NodeClick(Me, e)
        End Sub

        Private Sub DNNMenu_PreRender(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.PreRender
            RegisterClientScript()
            Page.RegisterRequiresPostBack(Me)
            UpdateNodes(Me.MenuNodes)            'update all imageindex properties

            If Me.IsDownLevel = False Then
                'ClientAPI.RegisterClientVariable(Me.Page, Me.ClientID & "_xml", Me.MenuNodes.ToXml(), True)
                ClientAPI.RegisterClientVariable(Me.Page, Me.ClientID & "_json", "{" & Me.MenuNodes.ToJSON() & "}", True)
            Else         'If Me.EnablePostbackState Then
                ViewState("xml") = Me.MenuNodes.ToXml
            End If

        End Sub

        Private Sub LoadPostedJSON()
            Dim json As String = ""
            If Me.IsDownLevel = False Then json = ClientAPI.GetClientVariable(Me.Page, Me.ClientID & "_json")
            If String.IsNullOrEmpty(json) = False Then
                'Dim ser As System.Web.Script.Serialization.JavaScriptSerializer = New System.Web.Script.Serialization.JavaScriptSerializer()
                'Dim dict As Dictionary(Of String, Object) = ser.Deserialize(Of Dictionary(Of String, Object))(json)
                Dim dict As Dictionary(Of String, Object) = MSAJAX.Deserialize(Of Dictionary(Of String, Object))(json)
                If dict.ContainsKey("nodes") Then LoadJSON(dict)
                'Me.MenuNodes.LoadJSON(dict)
            End If
        End Sub

        Private Sub LoadPostedXML()
            Dim strXML As String = ""
            'If Me.EnablePostbackState AndAlso Me.IsDownLevel = False Then strXML = ClientAPI.GetClientVariable(Me.Page, Me.ClientID & "_xml")
            If Me.IsDownLevel = False Then strXML = ClientAPI.GetClientVariable(Me.Page, Me.ClientID & "_xml")
            If Len(strXML) > 0 Then LoadXml(strXML)
        End Sub

        Public Function LoadPostData(ByVal postDataKey As String, ByVal postCollection As System.Collections.Specialized.NameValueCollection) As Boolean Implements System.Web.UI.IPostBackDataHandler.LoadPostData
            'We need to process the individual checkboxes
            'If Me.CheckBoxes Then SelectNodes(Nothing, postCollection)

        End Function

        Public Sub RaisePostDataChangedEvent() Implements System.Web.UI.IPostBackDataHandler.RaisePostDataChangedEvent

        End Sub

        Public Function RaiseClientAPICallbackEvent(ByVal eventArgument As String) As String Implements Utilities.IClientAPICallbackEventHandler.RaiseClientAPICallbackEvent
            Dim aryArgs As String() = Split(eventArgument, ClientAPI.COLUMN_DELIMITER)
            Dim strNodeXml As String = "<root>" & aryArgs(0) & "</root>"
            Dim objNodes As MenuNodeCollection = New MenuNodeCollection(strNodeXml, "", Me)

            Dim objNode As MenuNode = objNodes.Item(0)
            If Not objNode Is Nothing Then
                RaiseEvent PopulateOnDemand(Me, New DNNMenuEventArgs(objNode))
                Dim objTNode As MenuNode = Me.FindNode(objNode.ID)              'if whole Menu was populated (i.e. LoadXml, then use the node from the Menu

                If Not objTNode Is Nothing Then
                    Return objTNode.MenuNodes.ToJSON()
                Else                 'if only passed in node object was updated then use that xml.
                    Return objNode.MenuNodes.ToJSON()
                End If
            Else
                Return Nothing
            End If
        End Function

        'Public Function RaiseClientAPICallbackEvent(ByVal eventArgument As String) As String Implements Utilities.IClientAPICallbackEventHandler.RaiseClientAPICallbackEvent
        '    Dim aryArgs As String() = Split(eventArgument, ClientAPI.COLUMN_DELIMITER)
        '    Dim strNodeXml As String = "<root>" & aryArgs(0) & "</root>"
        '    Dim objNodes As MenuNodeCollection = New MenuNodeCollection(strNodeXml, "", Me)

        '    Dim objNode As MenuNode = objNodes.Item(0)
        '    If Not objNode Is Nothing Then
        '        RaiseEvent PopulateOnDemand(Me, New DNNMenuEventArgs(objNode))
        '        Dim objTNode As MenuNode = Me.FindNode(objNode.ID)              'if whole Menu was populated (i.e. LoadXml, then use the node from the Menu

        '        If Not objTNode Is Nothing Then
        '            Return objTNode.XmlNode.InnerXml
        '        Else                 'if only passed in node object was updated then use that xml.
        '            Return objNode.XmlNode.InnerXml
        '        End If
        '    Else
        '        Return Nothing
        '    End If
        'End Function

#End Region

#Region "Methods"
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Responsible for inputting the hidden field necessary for the ClientAPI to pass variables back in forth
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	4/6/2005	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Obsolete("RegisterDNNVariableControl on control level is now obsolete.  Use RegisterDNNVariableControl.WebControls")> _
        Public Sub RegisterDNNVariableControl()
            ClientAPI.RegisterDNNVariableControl(Me)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Determines if client supports an uplevel rendering, and if so it registers
        ''' the appropriate scripts.
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	4/6/2005	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Sub RegisterClientScript()
            If IsDownLevel = False Then
                ClientAPI.RegisterClientReference(Me.Page, ClientAPI.ClientNamespaceReferences.dnn_dom)
                'ClientAPI.RegisterClientReference(Me.Page, ClientAPI.ClientNamespaceReferences.dnn_xml)
                ClientAPI.RegisterClientReference(Me.Page, ClientAPI.ClientNamespaceReferences.dnn_dom_positioning)
                If Me.PopulateNodesFromClient Then               'AndAlso ClientAPI.BrowserSupportsFunctionality(Utilities.ClientAPI.ClientFunctionality.XMLHTTP) Then
                    ClientAPI.RegisterClientReference(Me.Page, ClientAPI.ClientNamespaceReferences.dnn_xmlhttp)
                End If
                If Me.Animation.AnimationType <> UI.Utilities.Animation.AnimationType.None Then
                    ClientAPI.RegisterClientReference(Me.Page, ClientAPI.ClientNamespaceReferences.dnn_motion)
                End If
                RegisterSubmitComponent(Me.Page)
                RegisterClientScriptBlock(Me.Page, "dnn.controls.js")
                RegisterClientScriptBlock(Me.Page, "dnn.controls.dnnmenu.js")
                'If Not ClientAPI.IsClientScriptBlockRegistered(Me.Page, "dnn.controls.dnnmenu.js") Then
                '    ClientAPI.RegisterClientScriptBlock(Me.Page, "dnn.controls.dnnmenu.js", "<script src=""" & MenuScriptPath & "dnn.controls.dnnmenu.js""></script>")
                'End If
                'If Me.Visible Then
                '	ClientAPI.RegisterStartUpScript(Me.Page, Me.ClientID & "_startup", "<script type="text/javascript">dnn.controls.initMenu($get('" & Me.ClientID & "'));</script>")					  'wrong place
                'End If
            End If
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Finds Node by passed in Key. 
        ''' </summary>
        ''' <param name="strID">ID of node</param>
        ''' <returns></returns>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	11/17/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Function FindNode(ByVal strID As String) As MenuNode
            Return Me.MenuNodes.FindNode(strID)
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Finds Node by passed in Key. 
        ''' </summary>
        ''' <param name="strKey">Key of node</param>
        ''' <returns></returns>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	11/17/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Function FindNodeByKey(ByVal strKey As String) As MenuNode
            Return Me.MenuNodes.FindNodeByKey(strKey)
        End Function

        Public Sub LoadXml(ByVal strXml As String)
            m_Nodes = New MenuNodeCollection(strXml, "", Me)
        End Sub

        Public Sub LoadJSON(ByVal dict As Dictionary(Of String, Object))
            m_Nodes = New MenuNodeCollection(dict, Me)
        End Sub

        Private Sub UpdateNodes(ByVal objNodes As MenuNodeCollection)
            Dim objNode As MenuNode
            'todo: xpath lookup for img attribute so we don't waste time looping.
            For Each objNode In objNodes
                If Len(objNode.Image) > 0 Then
                    If Me.ImageList.Contains(New NodeImage(objNode.Image)) = False Then
                        objNode.ImageIndex = Me.ImageList.Add(objNode.Image)
                    Else
                        objNode.ImageIndex = Me.ImageList.IndexOf(New NodeImage(objNode.Image))
                    End If
                    objNode.Image = Nothing
                End If
                If objNode.DNNNodes.Count > 0 Then UpdateNodes(objNode.MenuNodes)
            Next
        End Sub

        Private Function MarshalledProperties() As Hashtable
            Dim properties As Hashtable = New Hashtable
            '--- imagelist logic ---
            If Me.ImageList.Count > 0 Then
                Dim objImagePaths As Collections.SortedList = New Collections.SortedList
                Dim strList As String = ""
                Dim strImagePathList As String = ""
                Dim objNodeImage As NodeImage
                For Each objNodeImage In Me.ImageList
                    If objNodeImage.ImageUrl.IndexOf("/") > -1 Then
                        Dim strPath As String = objNodeImage.ImageUrl.Substring(0, objNodeImage.ImageUrl.LastIndexOf("/") + 1)
                        Dim strImage As String = objNodeImage.ImageUrl.Substring(objNodeImage.ImageUrl.LastIndexOf("/") + 1)
                        If objImagePaths.ContainsValue(strPath) = False Then
                            objImagePaths.Add(objImagePaths.Count, strPath)
                        End If
                        objNodeImage.ImageUrl = String.Format("[{0}]{1}", objImagePaths.IndexOfValue(strPath).ToString, strImage)
                    End If
                    strList &= CStr(IIf(Len(strList) > 0, ",", "")) & objNodeImage.ImageUrl
                Next
                For intPaths As Integer = 0 To objImagePaths.Count - 1
                    strImagePathList &= CStr(IIf(Len(strImagePathList) > 0, ",", "")) & CStr(objImagePaths.GetByIndex(intPaths))
                Next
                properties.Add("imagelist", strList)
                properties.Add("imagepaths", strImagePathList)
            End If

            properties.Add("postback", ClientAPI.GetPostBackEventReference(Me, "[NODEID]" & ClientAPI.COLUMN_DELIMITER & "Click"))

            If Me.PopulateNodesFromClient Then
                If DotNetNuke.UI.Utilities.ClientAPI.BrowserSupportsFunctionality(Utilities.ClientAPI.ClientFunctionality.XMLHTTP) Then
                    properties.Add("callback", DotNetNuke.UI.Utilities.ClientAPI.GetCallbackEventReference(Me, "'[NODEXML]'", "this.callBackSuccess", "mNode", "this.callBackFail", "this.callBackStatus"))
                Else
                    properties.Add("callback", ClientAPI.GetPostBackClientHyperlink(Me, "[NODEID]" & ClientAPI.COLUMN_DELIMITER & "OnDemand"))
                End If
                If Len(Me.CallbackStatusFunction) > 0 Then
                    properties.Add("callbacksf", Me.CallbackStatusFunction)
                End If
            End If

            Return properties
        End Function

#End Region

        Protected Overrides Sub Finalize()
            MyBase.Finalize()
        End Sub
    End Class
End Namespace
