'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2009
' by DotNetNuke Corporation
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System
Imports System.ComponentModel
Imports System.Web.UI
Imports System.Web.UI.WebControls
Namespace DotNetNuke.UI.WebControls

	Public Class MenuNode : Inherits DNNNode

		Private m_objNodes As MenuNodeCollection
        Private m_DNNMenu As DNNMenu

        Public Sub New()
            MyBase.New()
        End Sub

        Public Sub New(ByVal strText As String)
            MyBase.New(strText)
        End Sub

        Friend Sub New(ByVal objXmlNode As System.Xml.XmlNode, ByVal ctlOwner As Control)
            MyBase.New(objXmlNode)
            m_DNNMenu = CType(ctlOwner, DNNMenu)
        End Sub

        Friend Sub New(ByVal ctlOwner As Control)
            MyBase.New()
            m_DNNMenu = CType(ctlOwner, DNNMenu)
        End Sub


        <Browsable(True), PersistenceMode(PersistenceMode.InnerProperty)> _
          Public ReadOnly Property MenuNodes() As MenuNodeCollection
            Get
                If m_objNodes Is Nothing Then
                    m_objNodes = New MenuNodeCollection(Me.XmlNode, Me.DNNMenu)
                End If
                Return m_objNodes
            End Get
        End Property

        <Browsable(False)> _
          Public ReadOnly Property Parent() As MenuNode
            Get
                Return CType(Me.ParentNode, MenuNode)
            End Get
        End Property

        <Browsable(False)> _
          Public ReadOnly Property DNNMenu() As DNNMenu
            Get
                Return CType(m_DNNMenu, DNNMenu)
            End Get
        End Property

        <Bindable(True), DefaultValue(""), PersistenceMode(PersistenceMode.Attribute)> _
          Public Property CssClassOver() As String
            Get
                Return Me.CSSClassHover
            End Get
            Set(ByVal Value As String)
                Me.CSSClassHover = Value
            End Set
        End Property

        <Bindable(True), DefaultValue(-1), PersistenceMode(PersistenceMode.Attribute)> _
          Public Property ImageIndex() As Integer
            Get
                If Len(Me.CustomAttribute("iIdx")) > 0 Then
                    Return CInt(Me.CustomAttribute("iIdx"))
                Else
                    Return -1
                End If
            End Get
            Set(ByVal Value As Integer)
                Me.CustomAttribute("iIdx") = CStr(Value)
            End Set
        End Property

        <Bindable(True), DefaultValue(-1), PersistenceMode(PersistenceMode.Attribute)> _
          Public Property UrlIndex() As Integer
            Get
                If Len(Me.CustomAttribute("uIdx")) > 0 Then
                    Return CInt(Me.CustomAttribute("uIdx"))
                Else
                    Return -1
                End If
            End Get
            Set(ByVal Value As Integer)
                Me.CustomAttribute("uIdx") = CStr(Value)
            End Set
        End Property


        'Public Property PopulateOnDemand() As Boolean
        '	Get
        '		Return CBool(Me.CustomAttribute("pond", False))
        '	End Get
        '	Set(ByVal Value As Boolean)
        '		Me.CustomAttribute("pond") = Value
        '	End Set
        'End Property

        Public Property LeftHTML() As String
            Get
                Return Me.CustomAttribute("lhtml", "")
            End Get
            Set(ByVal Value As String)
                Me.CustomAttribute("lhtml") = Value
            End Set
        End Property

        Public Property RightHTML() As String
            Get
                Return Me.CustomAttribute("rhtml", "")
            End Get
            Set(ByVal Value As String)
                Me.CustomAttribute("rhtml") = Value
            End Set
        End Property

        Public Shadows ReadOnly Property ParentNode() As MenuNode
            Get
                If Not Me.XmlNode.ParentNode Is Nothing AndAlso Me.XmlNode.ParentNode.NodeType <> System.Xml.XmlNodeType.Document Then Return New MenuNode(Me.XmlNode.ParentNode, m_DNNMenu) Else Return Nothing
            End Get
        End Property

        Private ReadOnly Property NodeWriter() As IMenuNodeWriter
            Get
                Select Case m_DNNMenu.RenderMode
                    Case UI.WebControls.DNNMenu.MenuRenderMode.DownLevel
                        Return New MenuNodeWriter(m_DNNMenu.IsCrawler, m_DNNMenu.Orientation)
                    Case UI.WebControls.DNNMenu.MenuRenderMode.UnorderedList
                        Return New MenuNodeListWriter(m_DNNMenu.Orientation)
                End Select
                Return New MenuNodeListWriter(m_DNNMenu.Orientation)
            End Get
        End Property

        Public Sub Click()
            Me.Selected = Not Me.Selected
            If DNNMenu.IsDownLevel Then
                Dim objNode As MenuNode
                For Each objNode In DNNMenu.SelectedMenuNodes
                    If objNode.Level >= Me.Level AndAlso objNode.ID <> Me.ID Then objNode.Selected = False
                Next
            End If
            DNNMenu.OnNodeClick(New DNNMenuNodeClickEventArgs(Me))
        End Sub

        Public Overridable Sub Render(ByVal writer As HtmlTextWriter)
            NodeWriter.RenderNode(writer, Me)
        End Sub

        Friend Sub SetDNNMenu(ByVal objMenu As DNNMenu)
            m_DNNMenu = objMenu
        End Sub

	End Class
End Namespace
