'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2009
' by DotNetNuke Corporation
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System
Imports System.Collections
Imports System.ComponentModel
Imports System.ComponentModel.Design
Imports System.Web.UI
Imports System.Web.UI.Design
Imports System.Web.UI.WebControls
Imports System.Diagnostics
Imports System.Xml
Imports DotNetNuke.UI.Utilities
Imports DotNetNuke.UI.WebControls

Namespace DotNetNuke.UI.WebControls
    <ParseChildren(True, "States"), PersistChildren(False), Designer(GetType(Design.WebControls.DNNMultiStateBoxDesigner)), ToolboxData("<{0}:DNNMultiStateBox runat=server></{0}:DNNMultiStateBox>")> _
     Public Class DNNMultiStateBox
        Inherits CheckBox

#Region "Events / Delegates"
        'Public Delegate Sub DNNMultiStateBoxEventHandler(ByVal source As Object, ByVal e As DNNMultiStateBoxEventArgs)

#End Region

#Region "Member Variables"
        Private m_States As DNNMultiStateCollection = Nothing
        Private m_Enabled As Boolean = True
        Private m_ValueSet As Boolean = False
#End Region

#Region "Constructors"
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/4/2008	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Sub New()

        End Sub
#End Region

#Region "Properties"
        <DefaultValue(""), ClientProperty(), ClientPropertyName("state")> _
        Public Property SelectedStateKey() As String
            Get
                'do the assignment here as init not always called (permissions grid)
                If m_ValueSet = False AndAlso Me.Page.IsPostBack AndAlso Len(Me.Page.Request.Form(Me.UniqueID)) > 0 Then
                    Me.SelectedStateKey = Me.Page.Request.Form(Me.UniqueID)
                End If
                If Len(Me.Attributes("value")) > 0 AndAlso Me.Attributes("value") <> "on" Then
                    Return Me.Attributes("value")
                End If
                If Me.States.Count > 0 Then
                    Return Me.States(0).Key
                End If
                Return ""
            End Get
            Set(ByVal Value As String)
                m_ValueSet = True
                Me.Attributes("value") = Value
            End Set
        End Property

        Public ReadOnly Property SelectedState() As DNNMultiState
            Get
                Dim key As String = SelectedStateKey
                If String.IsNullOrEmpty(key) = False Then
                    For Each state As DNNMultiState In States
                        If state.Key = key Then
                            Return state
                        End If
                    Next
                End If
                Return Nothing
            End Get
        End Property

        <DefaultValue(""), ClientProperty(), ClientPropertyName("bid")> _
        Public Property BehaviorID() As String
            Get
                Return CType(ViewState("BehaviorID"), String)
            End Get
            Set(ByVal Value As String)
                ViewState("BehaviorID") = Value
            End Set
        End Property


        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Location of ClientAPI js files 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/4/2008	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Paths"), Description("Location of ClientAPI js files"), DefaultValue("")> _
        Public Property ClientAPIScriptPath() As String
            Get
                Return ClientAPI.ScriptPath
            End Get
            Set(ByVal Value As String)
                ClientAPI.ScriptPath = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Location of dnn.controls.DNNMultiStateBox.js file
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/4/2008	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Paths"), Description("Location of dnn.controls.DNNMultiStateBox.js file"), DefaultValue("")> _
        Public Property MultiStateBoxScriptPath() As String
            Get
                If Len(ViewState("MultiStateBoxScriptPath")) = 0 Then
                    Return ClientAPIScriptPath
                Else
                    Return CStr(ViewState("MultiStateBoxScriptPath"))
                End If
            End Get
            Set(ByVal Value As String)
                ViewState("MultiStateBoxScriptPath") = Value
            End Set
        End Property


        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Location of system images (i.e. spacer.gif)
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/4/2008	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Paths"), Description("Directory to find the images for the MultiStateBox."), DefaultValue("images/"), ClientProperty(), ClientPropertyName("imgpath"), UrlProperty()> _
        Public Property ImagePath() As String
            Get
                If Len(ViewState("ImagePath")) = 0 Then
                    Return "images/"
                Else
                    Return CStr(ViewState("ImagePath"))
                End If
            End Get
            Set(ByVal Value As String)
                ViewState("ImagePath") = Value
            End Set
        End Property

        <PersistenceMode(PersistenceMode.InnerProperty)> _
        Public Overridable ReadOnly Property States() As DNNMultiStateCollection
            Get
                If m_States Is Nothing Then
                    m_States = New DNNMultiStateCollection(Me)
                End If
                Return m_States
            End Get
        End Property

        <ClientProperty(), ClientPropertyName("enabled"), DefaultValue(True)> _
        Public Shadows Property Enabled() As Boolean
            Get
                Return m_Enabled
            End Get
            Set(ByVal value As Boolean)
                m_Enabled = value
            End Set
        End Property

#End Region

#Region "Event Handlers"

        'Private Sub DNNMultiStateBox_Init(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Init
        '    If Me.Page.IsPostBack AndAlso Len(Me.Page.Request.Form(Me.UniqueID)) > 0 Then
        '        Me.SelectedStateKey = Me.Page.Request.Form(Me.UniqueID)
        '    End If
        'End Sub

        'Private Sub DNNMultiStateBox_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        '    If Me.Page.IsPostBack AndAlso Len(Me.Page.Request.Form(Me.UniqueID)) > 0 Then
        '        SelectedStateKey = Me.Page.Request.Form(Me.UniqueID)
        '    End If
        'End Sub

        Private Sub DNNMultiStateBox_PreRender(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.PreRender
            RegisterClientScript()
            If Not SelectedState Is Nothing Then
                Me.Text = SelectedState.Text
            End If
            'Page.RegisterRequiresPostBack(Me)
        End Sub

        Protected Overrides Sub Render(ByVal writer As System.Web.UI.HtmlTextWriter)
            writer.AddAttribute("value", SelectedStateKey)
            MyBase.Render(writer)

            Dim properties As Hashtable = MarshalledProperties() 'get runtime generated properties (non attribute based)
            Dim json As String = ScriptGenerator.GetMarshalledPropertyJSON(Me, properties)
            BaseWebControl.RegisterInitialize(Me, "initMultiStateBox", json)
        End Sub

#End Region

#Region "Methods"
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Determines if client supports an uplevel rendering, and if so it registers
        ''' the appropriate scripts.
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/4/2008	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Sub RegisterClientScript()
            ClientAPI.RegisterClientReference(Me.Page, ClientAPI.ClientNamespaceReferences.dnn_dom)
            RegisterClientScriptBlock(Me.Page, "dnn.controls.js")
            RegisterClientScriptBlock(Me.Page, "dnn.controls.dnnmultistatebox.js")
        End Sub
#End Region

        Protected Function MarshalledProperties() As Hashtable
            Dim properties As Hashtable = New Hashtable
            properties.Add("states", MSAJAX.Serialize(Me.States))
            Return properties
        End Function

        Friend Sub NotifyDesigner()
            If (MyBase.DesignMode AndAlso (Not Me.States Is Nothing)) Then
                Dim Host As IDesignerHost = TryCast(MyBase.Site.Container, IDesignerHost)
                Dim Designer As ControlDesigner = TryCast(Host.GetDesigner(Me), ControlDesigner)
                Dim Descriptor As PropertyDescriptor = Nothing
                Try
                    Descriptor = TypeDescriptor.GetProperties(Me).Item("States")
                Catch ex As Exception
                    Return
                End Try
                Dim ccea As New ComponentChangedEventArgs(Me, Descriptor, Nothing, Me.States)
                Designer.OnComponentChanged(Me, ccea)
            End If
        End Sub



    End Class
End Namespace
