'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2009
' by DotNetNuke Corporation
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'
Imports System
Imports System.ComponentModel
Imports System.Text
Imports System.Web
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports DotNetNuke.UI.Utilities

Namespace DotNetNuke.UI.WebControls

	<ParseChildren(False), PersistChildren(True), TypeConverter(GetType(ExpandableObjectConverter))> _
	Public Class DNNTab : Inherits WebControl
		Implements INamingContainer

		Public Enum eTabCallbackPostMode
			None
			DNNVariable
			TabStrip
			Form
		End Enum


#Region "Public Events"
        Public Event SetupDefaults()
        Public Event PreLoadPostData()
#End Region

#Region "Member Variables"

		Private m_oParentTabStrip As DNNTabStrip
		Private m_oDNNTabLabel As DNNTabLabel = New DNNTabLabel
		Private m_blnIsPostBack As Boolean
		Private m_blnIsCallBack As Boolean
#End Region

#Region "Constructors"
		Public Sub New()
			MyBase.New(HtmlTextWriterTag.Div)
		End Sub

		Public Sub New(ByVal LabelText As String)
			Me.New()
			Me.Label.Text = LabelText
		End Sub
#End Region

#Region "Properties"

        <ClientProperty(), ClientPropertyName("enabled"), DefaultValue(True)> _
        Public Overrides Property Enabled() As Boolean
            Get
                Return MyBase.Enabled
            End Get
            Set(ByVal value As Boolean)
                MyBase.Enabled = value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' This property allows the developer to determine whether or not the controls
        ''' contained within the tab are part of the postback.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' The controls within a tab that is rendered on demand will need to use this
        ''' property instead of the Page's IsPostback method.
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/7/2006	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
		<DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
		Public ReadOnly Property IsPostBack() As Boolean
			Get
				Return m_blnIsPostBack
			End Get
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' This property allows the developer to determine whether or not the controls
		''' contained within the tab are part of the callback.
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' The controls within a tab that is rendered on demand will possibly use this
		''' property for defaulting data.
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	8/7/2006	Commented
		''' </history>
		''' -----------------------------------------------------------------------------
		<DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
		Public ReadOnly Property IsCallBack() As Boolean
			Get
				Return m_blnIsCallBack
			End Get
		End Property


		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Access to the control containing this tab is necessary in order to pull
		''' properties pertaining to the tab.  For example, css.
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	8/7/2006	Commented
		''' </history>
		''' -----------------------------------------------------------------------------
		<DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
		Protected ReadOnly Property ParentControl() As DNNTabStrip
			Get
				If m_oParentTabStrip Is Nothing AndAlso TypeOf Me.Parent Is DNNTabStrip Then
					m_oParentTabStrip = CType(Me.Parent, DNNTabStrip)
				End If
				Return m_oParentTabStrip
			End Get
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Allows developer to determine if this tab is currently selected or not
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	8/7/2006	Commented
		''' </history>
		''' -----------------------------------------------------------------------------
		<DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
		Friend ReadOnly Property IsSelected() As Boolean
			Get
				If ParentControl Is Nothing Then
					Return True
				ElseIf Not ParentControl.SelectedTab Is Nothing AndAlso Me.ID = ParentControl.SelectedTab.ID AndAlso Not Me.ID Is Nothing Then
					Return True
				End If
				Return False
			End Get
		End Property

		<Category("Style"), Description("The style to be applied to Label"), DesignerSerializationVisibility(DesignerSerializationVisibility.Content), NotifyParentProperty(True)> _
		Public ReadOnly Property Label() As DNNTabLabel		'HtmlControls.HtmlGenericControl
			Get
				Return m_oDNNTabLabel
			End Get
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Assigns how the tabs will be rendered.
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' All - All tabs along with the html of the controls are sent down to the client.   
		''' When user changes the tab dhtml is used to display
		''' Postback - Only the selected tab and the html of the controls within it are sent 
		''' to the client.  When the user clicks another tab a postback occurs causing a new tab 
		''' to display.
		''' Callback - Only the selected tab and the html of the controls within it are sent 
		''' to the client.  When the user clicks another tab a callback occurs to retrieve the html
		''' for the new tab.
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	8/7/2006	Commented
		''' </history>
		''' -----------------------------------------------------------------------------
		<DefaultValue(DNNTabStrip.eTabRenderMode.CallBack), Category("Behavior")> _
		Public Property TabRenderMode() As DNNTabStrip.eTabRenderMode
			Get
				If ViewState("TabRenderMode") Is Nothing Then
					If Me.ParentControl Is Nothing Then
						Return DNNTabStrip.eTabRenderMode.CallBack
					Else
						Return Me.ParentControl.TabRenderMode
					End If
				Else
					Return (CType(ViewState("TabRenderMode"), DNNTabStrip.eTabRenderMode))
				End If
			End Get
			Set(ByVal Value As DNNTabStrip.eTabRenderMode)
				ViewState("TabRenderMode") = Value
			End Set
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' If tab is rendered through callback, this property determines what data is available
		''' to the callback handler
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' None - Only the minimal information to perform a callback is passed.   
		''' DNNVariable - Only the variables set through the ClientAPI are posted
		''' Tab - All form data from the rendered tab's conrols is posted
		''' Form - The entire form's data is posted
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	8/23/2006	Commented
		''' </history>
		''' -----------------------------------------------------------------------------
		<DefaultValue(eTabCallbackPostMode.None), Category("Behavior")> _
		Public Property TabCallbackPostMode() As eTabCallbackPostMode
			Get
				If ViewState("TabCallbackPostMode") Is Nothing Then
					Return eTabCallbackPostMode.None
				Else
					Return (CType(ViewState("TabCallbackPostMode"), eTabCallbackPostMode))
				End If
			End Get
			Set(ByVal Value As eTabCallbackPostMode)
				ViewState("TabCallbackPostMode") = Value
			End Set
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' This property determines what type of callback is used
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' The Simple callback short-circuits at the Page Init event while the ProcessPage
		''' will run the page through the entire page's lifecycle.
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	8/9/2006	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		<Bindable(True), DefaultValue(ClientAPICallBackResponse.CallBackTypeCode.Simple), Description("Image to display during a callback")> _
		Public Property CallBackType() As ClientAPICallBackResponse.CallBackTypeCode
			Get
				If Len(ViewState("CallBackTypeCode")) > 0 Then
                    Return CType(ViewState("CallBackTypeCode"), ClientAPICallBackResponse.CallBackTypeCode)
                ElseIf Not ParentControl Is Nothing Then
                    Return ParentControl.CallBackType
                Else
                    Return ClientAPICallBackResponse.CallBackTypeCode.Simple
                End If
			End Get
			Set(ByVal Value As ClientAPICallBackResponse.CallBackTypeCode)
				ViewState("CallBackTypeCode") = Value
			End Set
		End Property

#End Region

#Region "Overrides"

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Adds HTML attributes and styles that need to be rendered to the specified System.Web.UI.HtmlTextWriter. 
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' The attributes handled by this method include:  display, label id, tabid, and class (css)
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	8/7/2006	Commented
		''' </history>
		''' -----------------------------------------------------------------------------
		Protected Overrides Sub AddAttributesToRender(ByVal writer As System.Web.UI.HtmlTextWriter)
			If Me.IsSelected = False Then
				Me.Style.Add("display", "none")
			End If
			MyBase.AddAttributesToRender(writer)
            'writer.AddAttribute("label", Me.ClientID & "_l")
            'writer.AddAttribute("tid", Me.ID)

			If Len(Me.ParentControl.DefaultContainerCssClass) > 0 AndAlso Len(Me.CssClass) = 0 Then
				writer.AddAttribute("class", Me.ParentControl.DefaultContainerCssClass)
			End If

		End Sub

		'Necessary???
		'Protected Overrides Sub RenderContents(ByVal writer As HtmlTextWriter)
		'	MyBase.RenderContents(writer)
		'End Sub
#End Region

#Region "Internal (Friend) Methods"
		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Allows tab to have its postback property set
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' Assigned based off of posted data during tabstrip load
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	8/7/2006	Commented
		''' </history>
		''' -----------------------------------------------------------------------------
		Friend Sub SetTabState(ByVal IsPostBack As Boolean)
			m_blnIsPostBack = IsPostBack
		End Sub

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Allows tab to be assigned a parent.  
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' Needed for callback scenerios where the tab is passed as a parameter
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	8/7/2006	Commented
		''' </history>
		''' -----------------------------------------------------------------------------
		Friend Sub SetParent(ByVal Parent As DNNTabStrip)
			m_oParentTabStrip = Parent
		End Sub

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Need a way to raise the event 
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' Couldn't think of a better name
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	8/7/2006	Commented
		''' </history>
		''' -----------------------------------------------------------------------------
		Friend Sub RaiseSetupDefaultsEvent(ByVal IsCallBack As Boolean)
			m_blnIsCallBack = IsCallBack
			RaiseEvent SetupDefaults()
		End Sub

        Friend Sub RaisePreLoadPostData()
            RaiseEvent PreLoadPostData()
        End Sub


        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' There are 2 parts to render for each tab.  The Label and the panel with the contents.
        ''' The contents are rendered through the normal rendering methods, however, an explicit call
        ''' is made to render the label since we may render the label but not the contents
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/7/2006	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Friend Sub RenderLabel(ByVal writer As HtmlTextWriter)
            Dim blnHasParent As Boolean = Not Me.ParentControl Is Nothing

            Dim oLabel As Label = New Label
            oLabel.Controls.Clear()
            oLabel.ID = Me.ClientID & "_l"
            'm_oLabel.ApplyStyle(Me.LabelStyle)
            'Me.LabelStyle.AddAttributesToRender(writer, m_oLabel)

            Dim sCss As String
            If Me.IsSelected Then
                sCss = Me.Label.CssClassSelected
                If Len(sCss) = 0 AndAlso blnHasParent Then sCss = Me.ParentControl.DefaultLabel.CssClassSelected 'if not set default to strip's selected class
            ElseIf Me.Enabled = False Then
                sCss = Me.Label.CssClassDisabled
            Else
                sCss = Me.Label.CssClass
            End If
            If blnHasParent Then
                If Me.Enabled Then
                    If Len(sCss) = 0 Then sCss = Me.ParentControl.CssClass 'if not set default to tabs normal class
                    If Len(sCss) = 0 Then sCss = Me.ParentControl.DefaultLabel.CssClass 'if not set default to strip's default label class
                Else
                    If Len(sCss) = 0 Then sCss = Me.ParentControl.DefaultLabel.CssClassDisabled 'if not set default to strip's default label class
                End If
            End If
            If Len(sCss) > 0 Then oLabel.CssClass = sCss

            'oLabel.Attributes.Add("tid", Me.ID)
            'Me.Label.WriteAttributes(oLabel)
            'If Len(Me.Label.CssClass) > 0 Then oLabel.Attributes.Add("css", Me.Label.CssClass)
            'If Len(Me.Label.CssClassSelected) > 0 Then oLabel.Attributes.Add("csssel", Me.Label.CssClassSelected)
            'If Len(Me.Label.CssClassHover) > 0 Then oLabel.Attributes.Add("csshover", Me.Label.CssClassHover)
            'If Len(Me.Label.CssClassDisabled) > 0 Then oLabel.Attributes.Add("cssdisabled", Me.Label.CssClassDisabled)

            'If Me.Enabled = False Then oLabel.Attributes.Add("enabled", "0")
            If Me.Visible = False Then oLabel.Style.Add("display", "none")

            'If WebControls.IsDesignMode() = False Then      'messes up design time view when locating form, so bypass in designer
            '    If Me.ParentControl.IsDownLevel = False Then
            '        If Me.TabRenderMode = DNNTabStrip.eTabRenderMode.CallBack Then
            '            Select Case Me.TabCallbackPostMode
            '                Case eTabCallbackPostMode.DNNVariable
            '                    oLabel.Attributes.Add("postmode", DotNetNuke.UI.Utilities.ClientAPI.DNNVARIABLE_CONTROLID)
            '                Case eTabCallbackPostMode.Form
            '                    Dim oCtl As Control = Me.ParentControl
            '                    While Not TypeOf oCtl Is HtmlControls.HtmlForm
            '                        oCtl = oCtl.Parent
            '                        If oCtl Is Nothing Then
            '                            Exit While
            '                        End If
            '                    End While
            '                    If Not oCtl Is Nothing Then
            '                        oLabel.Attributes.Add("postmode", oCtl.ClientID)
            '                    Else
            '                        Throw New Exception("Could not find form control")
            '                    End If

            '                Case eTabCallbackPostMode.TabStrip
            '                    oLabel.Attributes.Add("postmode", Me.ParentControl.ClientID)
            '            End Select
            '        End If

            '        If Me.CallBackType <> ParentControl.CallBackType Then
            '            oLabel.Attributes.Add("cbtype", CInt(Me.CallBackType).ToString)
            '        End If
            '    Else
            If Me.ParentControl.IsDownLevel Then
                oLabel.Attributes.Add("onclick", ClientAPI.GetPostBackClientHyperlink(Me.ParentControl, Me.ID & ClientAPI.COLUMN_DELIMITER & "OnDemand"))
            End If
            '    End If
            'End If

            If Len(Me.Label.ImageUrl) > 0 OrElse (blnHasParent AndAlso (Len(Me.ParentControl.DefaultLabel.ImageUrl) > 0)) Then
                Dim oImg As Image = New Image
                oImg.ID = Me.ClientID & "_i"
                oImg.ImageUrl = Me.Label.ImageUrl
                If Len(oImg.ImageUrl) = 0 Then oImg.ImageUrl = Me.ParentControl.DefaultLabel.ImageUrl
                oLabel.Controls.Add(oImg)
            End If
            If blnHasParent AndAlso Len(Me.ParentControl.WorkImage) > 0 Then
                Dim oImg As Image = New Image
                oImg.ID = Me.ClientID & "_w"
                oImg.ImageUrl = Me.ParentControl.WorkImage
                oImg.Style.Add("display", "none")
                oLabel.Controls.Add(oImg)
            End If

            oLabel.Controls.Add(New LiteralControl(Me.Label.Text))
            oLabel.RenderControl(writer)

        End Sub

        Friend Function MarshalledProperties() As Hashtable
            Dim properties As Hashtable = New Hashtable
            properties.Add("tid", Me.ID)
            properties.Add("tcid", Me.ClientID)

            If WebControls.IsDesignMode() = False Then      'messes up design time view when locating form, so bypass in designer
                If Me.ParentControl.IsDownLevel = False Then
                    If Me.TabRenderMode = DNNTabStrip.eTabRenderMode.CallBack Then
                        Select Case Me.TabCallbackPostMode
                            Case eTabCallbackPostMode.DNNVariable
                                properties.Add("postmode", DotNetNuke.UI.Utilities.ClientAPI.DNNVARIABLE_CONTROLID)
                            Case eTabCallbackPostMode.Form
                                Dim oCtl As Control = Me.ParentControl
                                While Not TypeOf oCtl Is HtmlControls.HtmlForm
                                    oCtl = oCtl.Parent
                                    If oCtl Is Nothing Then
                                        Exit While
                                    End If
                                End While
                                If Not oCtl Is Nothing Then
                                    properties.Add("postmode", oCtl.ClientID)
                                Else
                                    Throw New Exception("Could not find form control")
                                End If

                            Case eTabCallbackPostMode.TabStrip
                                properties.Add("postmode", Me.ParentControl.ClientID)
                        End Select
                    End If

                    If Me.CallBackType <> ParentControl.CallBackType Then
                        properties.Add("cbtype", CInt(Me.CallBackType).ToString)
                    End If
                End If
            End If


            Return properties
        End Function


#End Region

#Region "State Management"
		Protected Overrides Sub LoadViewState(ByVal savedState As Object)
			If savedState Is Nothing Then
				' Always invoke LoadViewState on the base class, even if there is no saved state, because the base class might have implemented some logic that needs to be executed even if there is no state to restore.
				MyBase.LoadViewState(savedState)
				Return
			Else
				Dim oState() As Object = CType(savedState, Object())
				If oState.Length <> 2 Then
					Throw New ArgumentException("Invalid View State")
				End If
				If Not oState(0) Is Nothing Then MyBase.LoadViewState(oState(0))
				If Not oState(1) Is Nothing Then CType(Label, IStateManager).LoadViewState(oState(1))
			End If

		End Sub

		Protected Overrides Function SaveViewState() As Object
			Dim oState(1) As Object
			oState(0) = MyBase.SaveViewState()
			If Not m_oDNNTabLabel Is Nothing Then
				oState(1) = CType(Label, IStateManager).SaveViewState
			Else
				oState(1) = Nothing
			End If

			For i As Integer = 0 To oState.Length - 1
				If Not oState(i) Is Nothing Then
					Return oState
				End If
			Next

			Return Nothing			  'If there is no saved state it is more performant to return nothing than to return an array of nothing values.
		End Function

		Protected Overrides Sub TrackViewState()
			MyBase.TrackViewState()

			If Not Label Is Nothing Then
				CType(Label, IStateManager).TrackViewState()

			End If
		End Sub
#End Region

	End Class
End Namespace
