'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2009
' by DotNetNuke Corporation
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'
Imports System
Imports System.ComponentModel
Imports System.Text
Imports System.Web
Imports System.Web.UI
Imports System.Web.UI.WebControls

Namespace DotNetNuke.UI.WebControls
	Public Class DNNTabLabel
        Implements IStateManager, IUrlResolutionService

#Region "Member Variables"
		Private m_bMarked As Boolean = False
		Private m_oState As StateBag
#End Region

#Region "Properties"

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Text to display on the Tab
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' NotifyParentPropertyAttribute - (with the constructor argument equal to true) applied to the Text property causes the editor to serialize changes in these properties into their parent property, an instance of the Tab class.
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	8/7/2006	Commented
		''' </history>
		''' -----------------------------------------------------------------------------
		<NotifyParentProperty(True), Localizable(True), DefaultValue(""), Category("Appearance")> _
		Public Overridable Property [Text]() As String
			Get
				Return CStr(ViewState("Text"))
			End Get
			Set(ByVal value As String)
				ViewState("Text") = value
			End Set
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Css class name to use for a selected tab
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	8/7/2006	Commented
		''' </history>
        ''' -----------------------------------------------------------------------------
        <ClientProperty(), ClientPropertyName("csssel")> _
        Public Property CssClassSelected() As String
            Get
                Return CStr(ViewState("CssClassSelected"))
            End Get
            Set(ByVal value As String)
                ViewState("CssClassSelected") = value
            End Set
        End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Css class name to use for a hovered tab
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	8/7/2006	Commented
		''' </history>
		''' -----------------------------------------------------------------------------
        <ClientProperty(), ClientPropertyName("csshover")> _
        Public Property CssClassHover() As String
            Get
                Return CStr(ViewState("CssClassHover"))
            End Get
            Set(ByVal value As String)
                ViewState("CssClassHover") = value
            End Set
        End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Css class name to use for a tab
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	8/7/2006	Commented
		''' </history>
        ''' -----------------------------------------------------------------------------
        <ClientProperty(), ClientPropertyName("css")> _
        Public Property CssClass() As String
            Get
                Return CStr(ViewState("CssClass"))
            End Get
            Set(ByVal value As String)
                ViewState("CssClass") = value
            End Set
        End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Css class name to use for a tab
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	8/7/2006	Commented
		''' </history>
		''' -----------------------------------------------------------------------------
        <ClientProperty(), ClientPropertyName("cssdisabled")> _
        Public Property CssClassDisabled() As String
            Get
                Return CStr(ViewState("CssClassDisabled"))
            End Get
            Set(ByVal value As String)
                ViewState("CssClassDisabled") = value
            End Set
        End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Url of image to display next to text of tab
		''' </summary>
		''' <value></value>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	8/7/2006	Commented
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Property ImageUrl() As String
			Get
				Return CStr(ViewState("ImageUrl"))
			End Get
			Set(ByVal value As String)
				ViewState("ImageUrl") = value
			End Set
        End Property

#End Region

#Region "State Management"
		Protected ReadOnly Property ViewState() As StateBag
			Get
				If m_oState Is Nothing Then
					m_oState = New StateBag(True)
					If IsTrackingViewState Then
						CType(m_oState, IStateManager).TrackViewState()
					End If
				End If
				Return m_oState
			End Get
		End Property

		Protected ReadOnly Property IsTrackingViewState() As Boolean Implements System.Web.UI.IStateManager.IsTrackingViewState
			Get
				Return m_bMarked
			End Get
		End Property

		Protected Sub LoadViewState(ByVal state As Object) Implements System.Web.UI.IStateManager.LoadViewState
			If Not (state Is Nothing) Then
				CType(ViewState, IStateManager).LoadViewState(state)
			End If
		End Sub

		Protected Function SaveViewState() As Object Implements System.Web.UI.IStateManager.SaveViewState
			Dim oState As Object = Nothing
			If Not (m_oState Is Nothing) Then
				oState = CType(m_oState, IStateManager).SaveViewState()
			End If
			Return oState
		End Function

		Protected Sub TrackViewState() Implements System.Web.UI.IStateManager.TrackViewState
			m_bMarked = True
		End Sub
#End Region

#Region "IUrlResolutionService"
        Public Function ResolveClientUrl(ByVal Url As String) As String Implements IUrlResolutionService.ResolveClientUrl
            'no url properties at the moment, so don't need anything here
            Return Url
        End Function
#End Region

    End Class
End Namespace