'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2009
' by DotNetNuke Corporation
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'
Imports System
Imports System.ComponentModel
Imports System.Text
Imports System.Web
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports DotNetNuke.UI.Design.WebControls
Imports DotNetNuke.UI
Imports DotNetNuke.UI.Utilities
Imports System.Collections.Generic

Namespace DotNetNuke.UI.WebControls

	'
	'ParseChildren(GetType(Tabs), ChildrenAsProperties:=False, DefaultProperty:="Tabs")

	'To enable parsing of the collection items within a control's tags, the control adds the ParseChildren(true, "Tabs") attribute 
	'to the control. The first argument (true) of ParseChildrenAttribute specifies that the page parser should interpret nested content 
	'within the control's tags as properties and not as child controls. The second argument ("Tabs") provides the name of the inner 
	'default property. When you specify the second argument, content within the control's tags must correspond to the default inner 
	'property (Tab objects) and to nothing else. 
	'http://msdn2.microsoft.com/en-us/library/9txe1d4x.aspx
	'

	<Designer(GetType(DNNTabStripDesigner)), ParseChildren(True, "Tabs"), DefaultProperty("Tabs"), ToolboxData("<{0}:DNNTabStrip runat=server><{0}:DNNTab id=""Tab1"" Text=""Tab 1"" runat=""server"">Tab 1 Content</{0}:DNNTab><{0}:DNNTab id=""Tab2"" Text=""Tab 2"" runat=""server"">Tab 2 Content</{0}:DNNTab></{0}:DNNTabStrip>")> _
	Public Class DNNTabStrip
		Inherits WebControl
		Implements DotNetNuke.UI.Utilities.IClientAPICallbackEventHandler
		Implements IPostBackEventHandler
		Implements IPostBackDataHandler

        Const TAB_CONTENT_DELIMITER As String = "~`~`~`~`~`~`"

		Public Enum eTabRenderMode
			All
			PostBack
			CallBack
		End Enum
#Region "Constructors"
        Public Sub New()
            MyBase.New(HtmlTextWriterTag.Div)
        End Sub
#End Region

#Region "Member Variables"
        Private m_objTabs As TabStripTabCollection     'Generic.List(Of Tab)
        Private m_objPostedTabData As Hashtable = New Hashtable
        Private m_objDefaultDNNTabLabel As DNNTabLabel = New DNNTabLabel
        Private Const TAB_RENDERED As Integer = 1
        Private Const TAB_SELECTED As Integer = 2
        Private Const TAB_ENABLED As Integer = 4
#End Region

#Region "Properties"
        <Bindable(True), DefaultValue(""), ClientProperty(), ClientPropertyName("bid")> _
        Public Property BehaviorID() As String
            Get
                Return CType(ViewState("BehaviorID"), String)
            End Get
            Set(ByVal Value As String)
                ViewState("BehaviorID") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' If callbacks are supported/enabled, this javascript function will be invoked
        ''' with each status change of the xmlhttp request.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' The Javascript transport does not raise any events.
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/7/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), PersistenceMode(PersistenceMode.Attribute), Category("Behavior"), ClientProperty(), ClientPropertyName("callbackSF")> _
        Public Property CallbackStatusFunction() As String
            Get
                Return (CType(ViewState("CallbackStatusFunction"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("CallbackStatusFunction") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' This javascript function will be invoked whenever the user clicks on a tab
        ''' other than the currently selected one
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' To cancel a tab from changing return false from this function
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/7/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), PersistenceMode(PersistenceMode.Attribute), Category("Behavior"), DefaultValue(""), ClientProperty(), ClientPropertyName("tabClickF")> _
        Public Property TabClickFunction() As String
            Get
                Return (CType(ViewState("TabClickFunction"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("TabClickFunction") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' This javascript function will be invoked whenever the selectedIndex changes
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/7/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), PersistenceMode(PersistenceMode.Attribute), Category("Behavior"), DefaultValue(""), ClientProperty(), ClientPropertyName("selIdxF")> _
        Public Property SelectedIndexChangedFunction() As String
            Get
                Return (CType(ViewState("SelectedIndexChangedFunction"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("SelectedIndexChangedFunction") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Location of ClientAPI js files 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/7/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Property ClientAPIScriptPath() As String
            Get
                Return DotNetNuke.UI.Utilities.ClientAPI.ScriptPath
            End Get
            Set(ByVal Value As String)
                DotNetNuke.UI.Utilities.ClientAPI.ScriptPath = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Location of dnn.controls.dnntabstrip.js file
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/7/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Property TabStripScriptPath() As String
            Get
                If Len(ViewState("TabScriptPath")) = 0 Then
                    Return ClientAPIScriptPath
                Else
                    Return CStr(ViewState("TabScriptPath"))
                End If
            End Get
            Set(ByVal Value As String)
                ViewState("TabScriptPath") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Allows developer to force the rendering of the Menu in DownLevel mode
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/7/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <DefaultValue(False), Category("Behavior")> _
        Public Property ForceDownLevel() As Boolean
            Get
                Return CType(ViewState("ForceDownLevel"), Boolean)
            End Get
            Set(ByVal Value As Boolean)
                ViewState("ForceDownLevel") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Allows developer to force the rendering of the control as it is shown to a crawler
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/7/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <DefaultValue(False), Category("Behavior")> _
        Public Property IsCrawler() As Boolean
            Get
                If Len(ViewState("IsCrawler")) = 0 AndAlso Not System.Web.HttpContext.Current Is Nothing Then
                    Return System.Web.HttpContext.Current.Request.Browser.Crawler
                Else
                    Return CBool(ViewState("IsCrawler"))
                End If

            End Get
            Set(ByVal Value As Boolean)
                ViewState("IsCrawler") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns whether the TabStrip will render DownLevel or not
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/7/2006	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden), Browsable(False)> _
        Public ReadOnly Property IsDownLevel() As Boolean
            Get
                If ForceDownLevel OrElse IsCrawler OrElse _
                 DotNetNuke.UI.Utilities.ClientAPI.BrowserSupportsFunctionality(Utilities.ClientAPI.ClientFunctionality.DHTML) = False Then
                    Return True
                Else
                    Return False
                End If
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Assigns how the tabs will be rendered.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' All - All tabs along with the html of the controls are sent down to the client.   
        ''' When user changes the tab dhtml is used to display
        ''' Postback - Only the selected tab and the html of the controls within it are sent 
        ''' to the client.  When the user clicks another tab a postback occurs causing a new tab 
        ''' to display.
        ''' Callback - Only the selected tab and the html of the controls within it are sent 
        ''' to the client.  When the user clicks another tab a callback occurs to retrieve the html
        ''' for the new tab.
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/7/2006	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <DefaultValue(eTabRenderMode.CallBack), Category("Behavior"), ClientProperty(), ClientPropertyName("trm")> _
        Public Property TabRenderMode() As eTabRenderMode
            Get
                If ViewState("TabRenderMode") Is Nothing Then
                    Return eTabRenderMode.CallBack
                Else
                    Return (CType(ViewState("TabRenderMode"), eTabRenderMode))
                End If
                'Else
                'Return False
                'End If
            End Get
            Set(ByVal Value As eTabRenderMode)
                ViewState("TabRenderMode") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the index of the tab that is currently selected
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/7/2006	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private m_iSelectedIndex As Integer = -1
        <DefaultValue(0), RefreshProperties(RefreshProperties.All)> _
        Public Property SelectedIndex() As Integer
            Get
                If m_iSelectedIndex <> -1 Then
                    Return m_iSelectedIndex
                ElseIf Len(ViewState("SelectedIndex")) = 0 Then
                    Return 0
                Else
                    Return CInt(ViewState("SelectedIndex"))
                End If
            End Get
            Set(ByVal Value As Integer)
                If Not m_objTabs Is Nothing AndAlso Value > Me.Tabs.Count - 1 Then
                    Throw New Exception("Invalid SelectedIndex")
                Else
                    m_iSelectedIndex = Value
                    ViewState("SelectedIndex") = Value
                End If
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the tab that is currently selected
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/7/2006	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden), Browsable(False)> _
        Public Property SelectedTab() As DNNTab
            Get
                If Me.Tabs.Count > SelectedIndex Then
                    Return CType(Tabs.Item(SelectedIndex), DNNTab)
                End If
                Return Nothing
            End Get
            Set(ByVal value As DNNTab)
                For i As Integer = 0 To Tabs.Count - 1
                    If Tabs.Item(i).ID = value.ID Then
                        Me.SelectedIndex = i
                        Exit For
                    End If
                Next
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' This property Exposes the tabstrip's tab collection
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' DesignerSerializationVisibilityAttribute - Setting the Content parameter specifies that a visual designer should serialize the contents of the property. In the example, the property contains Tab objects.
        ''' PersistenceModeAttribute - Passing the InnerDefaultProperty parameter specifies that a visual designer should persist the property to which the attribute is applied as an inner default property. This means that a visual designer persists the property within the control's tags. The attribute can be applied to only one property, because only one property can be persisted within the control's tags. The property value is not wrapped in a special tag.
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/7/2006	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <DesignerSerializationVisibility(DesignerSerializationVisibility.Content), PersistenceMode(PersistenceMode.InnerDefaultProperty), MergableProperty(False)> _
        Public Overridable ReadOnly Property Tabs() As TabStripTabCollection
            Get
                If m_objTabs Is Nothing Then
                    m_objTabs = New TabStripTabCollection(Me)       'New Generic.List(Of Tab)
                End If
                Return CType(m_objTabs, TabStripTabCollection)
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Default properties to use for a tab's label
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' This is more efficient to use (less payload) than specifying the same properties
        ''' on each individual tab
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/7/2006	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Appearance"), Description("The style to be applied to Label"), DesignerSerializationVisibility(DesignerSerializationVisibility.Content), NotifyParentProperty(True)> _
        Public ReadOnly Property DefaultLabel() As DNNTabLabel
            Get
                Return m_objDefaultDNNTabLabel
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Default css class name to use for a tab's container
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' This is more efficient to use (less payload) than specifying the same css class
        ''' name on each individual tab
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/7/2006	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Appearance")> _
        Public Property DefaultContainerCssClass() As String
            Get
                If ViewState("DefaultContainerCssClass") Is Nothing Then
                    Return ""
                Else
                    Return (CType(ViewState("DefaultContainerCssClass"), String))
                End If
            End Get
            Set(ByVal Value As String)
                ViewState("DefaultContainerCssClass") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' css class name to use for all tab's container
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	1/15/2007	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Appearance")> _
        Public Property CssTabContainer() As String
            Get
                If ViewState("CssTabContainer") Is Nothing Then
                    Return ""
                Else
                    Return (CType(ViewState("CssTabContainer"), String))
                End If
            End Get
            Set(ByVal Value As String)
                ViewState("CssTabContainer") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' css class name to use for element holding all content
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	1/15/2007	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Appearance")> _
        Public Property CssContentContainer() As String
            Get
                If ViewState("CssContentContainer") Is Nothing Then
                    Return ""
                Else
                    Return (CType(ViewState("CssContentContainer"), String))
                End If
            End Get
            Set(ByVal Value As String)
                ViewState("CssContentContainer") = Value
            End Set
        End Property


        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Image to display during a callback
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/9/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), DefaultValue(""), Description("Image to display during a callback"), ClientProperty(), ClientPropertyName("workImage")> _
        Public Property WorkImage() As String
            Get
                Return CType(ViewState("WorkImage"), String)
            End Get
            Set(ByVal Value As String)
                ViewState("WorkImage") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' This property determines what type of callback is used
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' The Simple callback short-circuits at the Page Init event while the ProcessPage
        ''' will run the page through the entire page's lifecycle.
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/9/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), DefaultValue(ClientAPICallBackResponse.CallBackTypeCode.Simple), Description("Image to display during a callback"), Category("Behavior"), ClientProperty(), ClientPropertyName("cbtype")> _
        Public Property CallBackType() As ClientAPICallBackResponse.CallBackTypeCode
            Get
                If Len(ViewState("CallBackTypeCode")) > 0 Then
                    Return CType(ViewState("CallBackTypeCode"), ClientAPICallBackResponse.CallBackTypeCode)
                Else
                    Return ClientAPICallBackResponse.CallBackTypeCode.Simple
                End If
            End Get
            Set(ByVal Value As ClientAPICallBackResponse.CallBackTypeCode)
                ViewState("CallBackTypeCode") = Value
            End Set
        End Property

        <Bindable(True), DefaultValue(DotNetNuke.UI.WebControls.Alignment.Top), PersistenceMode(PersistenceMode.Attribute)> _
        Public Property TabAlignment() As DotNetNuke.UI.WebControls.Alignment
            Get
                If Len(ViewState("TabAlignment")) > 0 Then
                    Return (CType(ViewState("TabAlignment"), Alignment))       '?
                Else
                    Return Alignment.Top
                End If
            End Get
            Set(ByVal Value As DotNetNuke.UI.WebControls.Alignment)
                ViewState("TabAlignment") = Value
            End Set
        End Property
#End Region

#Region "Event Handlers"

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' The init handler is responsible to register the dnnvariable control.
        ''' We are also loading the posted values in this method
        ''' </summary>
        ''' <param name="e"></param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/7/2006	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub DNNTabStrip_Init(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Init
            If DotNetNuke.UI.Utilities.ClientAPI.NeedsDNNVariable(Me) Then
                'This is to allow us to add a control to our parent's control collection...  kindof a hack
                AddHandler Me.Page.Load, AddressOf ParentOnLoad
                'Else
                '	LoadPostedVars()
            End If
            LoadPostedVars()
            RaisePreLoadPostDataEvents()
        End Sub

        Private Sub RaisePreLoadPostDataEvents()
            For Each objTab As DNNTab In Me.Tabs
                If objTab.IsPostBack Then
                    objTab.RaisePreLoadPostData()
                End If
            Next
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' The prerender event handler is responsible for registering the required scripts along with 
        ''' setting up the postback logic
        ''' </summary>
        ''' <param name="sender"></param>
        ''' <param name="e"></param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/8/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub DNNTabStrip_PreRender(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.PreRender
            RegisterClientScript()
            Page.RegisterRequiresPostBack(Me)

            If DotNetNuke.UI.Utilities.ClientAPI.BrowserSupportsFunctionality(Utilities.ClientAPI.ClientFunctionality.DHTML) Then
                DotNetNuke.UI.Utilities.ClientAPI.RegisterClientReference(Me.Page, ClientAPI.ClientNamespaceReferences.dnn)
                ClientAPI.RegisterClientVariable(Me.Page, Me.ClientID & "_json", "{" & Me.Tabs.ToJSON() & "}", True)
            End If
        End Sub

        Protected Overrides Sub Render(ByVal writer As System.Web.UI.HtmlTextWriter)
            MyBase.Render(writer)
            If IsDownLevel = False Then
                Dim properties As Hashtable = Me.MarshalledProperties() 'get runtime generated properties (non attribute based)
                ScriptGenerator.GetMarshalledPropertyJSON(m_objDefaultDNNTabLabel, properties)  'also include tablabel properties
                Dim json As String = ScriptGenerator.GetMarshalledPropertyJSON(Me, properties)
                BaseWebControl.RegisterInitialize(Me, "initTabStrip", json)
            End If
        End Sub

#End Region

#Region "Overrides"

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' This method is overriden to make sure the custom attributes for our control
        ''' get written
        ''' </summary>
        ''' <param name="writer"></param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/9/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Protected Overrides Sub AddAttributesToRender(ByVal writer As System.Web.UI.HtmlTextWriter)
            MyBase.AddAttributesToRender(writer)
            '    'Me.DefaultLabel.WriteAttributes(Me)
            '    'If Len(Me.DefaultLabel.CssClass) > 0 Then writer.AddAttribute("css", Me.DefaultLabel.CssClass)
            '    'If Len(Me.DefaultLabel.CssClassSelected) > 0 Then writer.AddAttribute("csssel", Me.DefaultLabel.CssClassSelected)
            '    'If Len(Me.DefaultLabel.CssClassHover) > 0 Then writer.AddAttribute("csshover", Me.DefaultLabel.CssClassHover)
            '    'If Len(Me.DefaultLabel.CssClassDisabled) > 0 Then writer.AddAttribute("cssdisabled", Me.DefaultLabel.CssClassDisabled)
            '    'If Len(Me.WorkImage) > 0 Then writer.AddAttribute("workImage", Me.WorkImage)

            '    writer.AddAttribute("tabs", GetTabIds)
            '    'If PopulateNodesFromClient Then
            '    If Me.IsDownLevel = False Then
            '        Select Case TabRenderMode
            '            Case eTabRenderMode.CallBack
            '                'writer.AddAttribute("cbtype", CInt(CallBackType).ToString)
            '                If DotNetNuke.UI.Utilities.ClientAPI.BrowserSupportsFunctionality(Utilities.ClientAPI.ClientFunctionality.XMLHTTP) Then
            '                    Dim strCallback As String = DotNetNuke.UI.Utilities.ClientAPI.GetCallbackEventReference(Me, "'[TABID]'", "this.callBackSuccess", "this", "this.callBackFail", "this.callBackStatus", "[POST]", ClientAPICallBackResponse.CallBackTypeCode.Simple)

            '                    'TODO:  Enhance the callback to accept a string for last param to avoid this hack
            '                    'minor hack to allow substitution of callbacktypes on tab level
            '                    'dnn.xmlhttp.doCallBack('MyDNNTabStrip MyDNNTabStrip','[TABID]',this.callBackSuccess,this,this.callBackFail,this.callBackStatus,null,'[POST]',0);
            '                    'dnn.xmlhttp.doCallBack('MyDNNTabStrip MyDNNTabStrip','[TABID]',this.callBackSuccess,this,this.callBackFail,this.callBackStatus,null,'[POST]',[CBTYPE]);
            '                    Dim intPos As Integer = strCallback.IndexOf("'[POST]'") + "'[POST]'".Length + 1
            '                    strCallback = strCallback.Substring(0, intPos) & "'[CBTYPE]'" & strCallback.Substring(intPos + 1)
            '                    writer.AddAttribute("callback", strCallback)
            '                ElseIf Me.IsDownLevel Then
            '                    writer.AddAttribute("callback", ClientAPI.GetPostBackClientHyperlink(Me, "[TABID]" & ClientAPI.COLUMN_DELIMITER & "OnDemand"))
            '                End If
            '                'If Len(CallbackStatusFunction) > 0 Then
            '                '    writer.AddAttribute("callbackSF", CallbackStatusFunction)
            '                'End If
            '            Case eTabRenderMode.PostBack
            '                writer.AddAttribute("callback", ClientAPI.GetPostBackClientHyperlink(Me, "[TABID]" & ClientAPI.COLUMN_DELIMITER & "OnDemand"))
            '            Case eTabRenderMode.All

            '        End Select

            '        'If Len(TabClickFunction) > 0 Then
            '        '    writer.AddAttribute("tabClickF", TabClickFunction)
            '        'End If

            '        'If Len(SelectedIndexChangedFunction) > 0 Then
            '        '    writer.AddAttribute("selIdxF", SelectedIndexChangedFunction)
            '        'End If

            '        'writer.AddAttribute("trm", CInt(Me.TabRenderMode).ToString)
            '    End If
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' The overridden sub is responsible for rendering our tab's labels along with
        ''' the tabs content panels that either are selected, previously rendered, or are 
        ''' supposed to be rendered based off of TabRenderMode
        ''' </summary>
        ''' <param name="writer"></param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/9/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Protected Overrides Sub RenderContents(ByVal writer As HtmlTextWriter)
            'MyBase.RenderContents(writer)

            Select Case Me.TabAlignment
                Case Alignment.Left, Alignment.Top
                    RenderTabs(writer)
            End Select

            writer.AddAttribute("id", Me.ClientID & "_c")
            If Len(Me.CssContentContainer) > 0 Then
                writer.AddAttribute("class", Me.CssContentContainer)
            End If

            'If ClientAPI.IsInCallback(Me.Page) Then  'new callback logic to delimit portions we want
            '    writer.Write(TAB_CONTENT_DELIMITER)
            'End If
            'writer.RenderBeginTag("div ")    '<--- TRICK ASP.NET to not send down TABLE for FireFox/Mozilla
            writer.RenderBeginTag("div")    '<--- TRICK ASP.NET to not send down TABLE for FireFox/Mozilla
            For Each objTab As DNNTab In Me.Tabs
                If objTab.IsPostBack OrElse objTab.IsSelected OrElse objTab.TabRenderMode = eTabRenderMode.All Then
                    objTab.RaiseSetupDefaultsEvent(False)
                    If objTab.Visible = False Then
                        objTab.Visible = True
                        objTab.Style.Add("display", "none")
                    ElseIf objTab.IsSelected Then
                        If Not objTab.Style.Item("display") Is Nothing Then
                            objTab.Style.Remove("display")
                        End If
                    End If
                    objTab.RenderControl(writer)
                End If
            Next
            writer.RenderEndTag()
            'If ClientAPI.IsInCallback(Me.Page) Then 'new callback logic to delimit portions we want
            '    writer.Write(TAB_CONTENT_DELIMITER)
            'End If

            Select Case Me.TabAlignment
                Case Alignment.Bottom, Alignment.Right
                    RenderTabs(writer)
            End Select

        End Sub

        Private Sub RenderTabs(ByVal writer As HtmlTextWriter)
            'writer.AddStyleAttribute("float", "left")
            writer.AddAttribute("id", Me.ClientID & "_lc")
            If Len(Me.CssTabContainer) > 0 Then
                writer.AddAttribute("class", Me.CssTabContainer)
            End If
            writer.RenderBeginTag("div ")

            For Each objTab As DNNTab In Me.Tabs
                objTab.RenderLabel(writer)
            Next
            writer.RenderEndTag()
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Make sure controls added between opening and closing tags are of type DNNTab
        ''' </summary>
        ''' <param name="obj"></param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/9/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Protected Overrides Sub AddParsedSubObject(ByVal obj As Object)
            If TypeOf obj Is DNNTab Then
                MyBase.AddParsedSubObject(obj)
            End If
        End Sub
#End Region

#Region "Public Methods"

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' This funciton is called during a callback.  It is responsible for returning the
        ''' html of the passed in tab
        ''' </summary>
        ''' <param name="eventArgument"></param>
        ''' <returns></returns>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/9/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Function RaiseClientAPICallbackEvent(ByVal eventArgument As String) As String Implements Utilities.IClientAPICallbackEventHandler.RaiseClientAPICallbackEvent
            Dim objTab As DNNTab = Me.Tabs.FindTab(eventArgument)
            If Not objTab Is Nothing Then
                Return GetTabContents(objTab)
            End If
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Handles tab's postback
        ''' </summary>
        ''' <param name="eventArgument"></param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/9/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Overridable Sub RaisePostBackEvent(ByVal eventArgument As String) Implements IPostBackEventHandler.RaisePostBackEvent
            Dim args() As String = Split(eventArgument, ClientAPI.COLUMN_DELIMITER)

            Dim objTab As DNNTab = Tabs.FindTab(args(0))
            If Not (objTab Is Nothing) Then
                If args.Length > 1 Then
                    Select Case args(1)
                        Case "OnDemand"
                            Me.SelectedTab = objTab
                    End Select
                Else
                    'assume an event with no specific argument to be a ondemand
                    Me.SelectedTab = objTab
                End If
            End If
        End Sub

        Public Function LoadPostData(ByVal postDataKey As String, ByVal postCollection As System.Collections.Specialized.NameValueCollection) As Boolean Implements System.Web.UI.IPostBackDataHandler.LoadPostData

        End Function

        Public Sub RaisePostDataChangedEvent() Implements System.Web.UI.IPostBackDataHandler.RaisePostDataChangedEvent

        End Sub

#End Region

#Region "Private Methods"
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' This method is responsible for obtaining the HTML of a given tab.
        ''' </summary>
        ''' <param name="objTab"></param>
        ''' <returns></returns>
        ''' <remarks>
        ''' The original technique used here is the same as the one found on this site.
        ''' http://aspnet.4guysfromrolla.com/articles/102203-1.2.aspx
        ''' 
        ''' EventValidation must be disabled in order to work (.NET 2.0 specific)
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/9/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Function GetTabContents(ByVal objTab As DNNTab) As String
            objTab.RaiseSetupDefaultsEvent(True)

            Dim objSB As System.Text.StringBuilder = New System.Text.StringBuilder
            Dim objSW As System.IO.StringWriter = New System.IO.StringWriter(objSB)
            Dim objHW As HtmlTextWriter = New HtmlTextWriter(objSW)
            Dim objForm As HtmlControls.HtmlForm
            Dim objPage As Page

            objPage = Me.Page
            objForm = Me.FindForm(Me)

            If objForm Is Nothing Then
                objForm = New HtmlControls.HtmlForm
                objPage.Controls.Add(objForm)
            Else
                objForm.Controls.Clear()
            End If

            objTab.ID = objTab.UniqueID
            objForm.Controls.Add(New LiteralControl(TAB_CONTENT_DELIMITER))
            objForm.Controls.Add(objTab)
            objForm.Controls.Add(New LiteralControl(TAB_CONTENT_DELIMITER))
            'Me.SelectedTab = objTab    'JH 1/29/07
            objTab.SetParent(Me)
            objTab.Enabled = True

            'Experimental rendering using new page object
            'instead of clearing the form's controls.  Unfortunately, there is
            'no way without reflection invoking a private member to associate 
            'the httpcontext object to a new page object.
            'initial idea behind came from http://aspnet.4guysfromrolla.com/articles/102203-1.2.aspx

            'objPage = New Page
            'If System.Environment.Version.Major >= 2 Then
            '    'EventValidation must be disabled in order to work...  since we are not .NET 2.0 specific yet we cannot do this here, so it needs to be done in web.config
            '    objPage.GetType.InvokeMember("EnableEventValidation", Reflection.BindingFlags.SetProperty, Nothing, objPage, New Object() {False})
            'End If

            'objPage.EnableViewState = False
            'objForm = New HtmlControls.HtmlForm
            'Dim objValidators As Collection = New Collection

            'For Each objValidator As IValidator In Me.Page.Validators
            '    objValidators.Add(objValidator)
            'Next

            'objPage.DesignerInitialize()    'This causes the Page class's protected InitRecursive() method to be called, which preps a number of properties needed for rendering.
            'objPage.GetType.InvokeMember("SetIntrinsics", Reflection.BindingFlags.InvokeMethod Or Reflection.BindingFlags.NonPublic Or Reflection.BindingFlags.Instance, Nothing, objPage, New Object() {HttpContext.Current})
            'objPage.Controls.Add(objForm)
            'objTab.ID = objTab.UniqueID
            'objForm.Controls.Add(New LiteralControl(TAB_CONTENT_DELIMITER))
            'objForm.Controls.Add(objTab)
            'objForm.Controls.Add(New LiteralControl(TAB_CONTENT_DELIMITER))
            'Me.SelectedTab = objTab
            'objTab.SetParent(Me)
            'objTab.Enabled = True

            'Dim oCtl As Control
            'For Each objValidator As Control In objValidators
            '    'Me.Page.Validators.Remove(objValidator)
            '    'CType(objValidator, Control).Page = objPage
            '    'objPage.Validators.Add(objValidator)
            '    objTab.Controls.Add(objValidator)
            'Next

            objForm.RenderControl(objHW)

            Dim ret As Dictionary(Of String, String) = New Dictionary(Of String, String)
            Dim strReturn As String = Split(objSB.ToString, TAB_CONTENT_DELIMITER)(1)    'parse out only the contents of the tab's panel by using a delimiter
            ret.Add("text", strReturn)

            Dim objScripts As HTMLElementCollection = Me.GetScripts(objSB.ToString)    'GetFormHTML())
            If objScripts.Count > 0 Then
                ret.Add("scripts", objScripts.ToJSON())
                'strReturn &= ClientAPI.COLUMN_DELIMITER & objScripts.ToJSON()
            End If
            'Dim ser As System.Web.Script.Serialization.JavaScriptSerializer = New System.Web.Script.Serialization.JavaScriptSerializer()
            Dim vars As Generic.Dictionary(Of String, String) = ClientAPI.GetClientVariableList(objPage)
            ret.Add("vars", MSAJAX.Serialize(vars))

            Return MSAJAX.Serialize(ret)
            'Return strReturn
            'Me.Tabs.FindTab(objTab.ID).RenderControl(objHW)
            'Return objSB.ToString

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Loads state of tabs through the ClientAPIs variables set
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/9/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub LoadPostedVars()
            Dim aryPairs As String() = Split(DotNetNuke.UI.Utilities.ClientAPI.GetClientVariable(Me.Page, Me.ClientID & "_tabs"), ",")
            Dim aryValues As String()
            For Each sPair As String In aryPairs
                aryValues = Split(sPair, "=")
                If aryValues.Length = 2 Then
                    m_objPostedTabData.Add(aryValues(0), aryValues(1))
                End If
            Next
            AssignSelectedIndex()
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Assigns the selected index for the currently selected tab
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/9/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub AssignSelectedIndex()
            For i As Integer = 0 To Me.Tabs.Count - 1
                If m_objPostedTabData.ContainsKey(Me.Tabs(i).ID) Then
                    If (CInt(m_objPostedTabData(Me.Tabs(i).ID)) And TAB_SELECTED) <> 0 Then
                        Me.SelectedIndex = i
                        'Exit For
                    End If
                    If (CInt(m_objPostedTabData(Me.Tabs(i).ID)) And TAB_RENDERED) <> 0 Then
                        Me.Tabs(i).SetTabState(True)
                    End If
                    If (CInt(m_objPostedTabData(Me.Tabs(i).ID)) And TAB_ENABLED) <> 0 Then
                        Me.Tabs(i).Enabled = True
                    Else
                        Me.Tabs(i).Enabled = False
                    End If
                End If
            Next
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Obtains a list of client-side tabids
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/9/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Function GetTabIds() As String
            Dim sRet As String = ""
            For Each objTab As DNNTab In Me.Tabs
                sRet &= objTab.ClientID & ","
            Next
            If Len(sRet) > 0 Then
                Return sRet.Substring(0, sRet.Length - 1)
            End If
            Return ""
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' This routine is necessary in some cases to register the DNNVariable hidden input
        ''' </summary>
        ''' <param name="Sender"></param>
        ''' <param name="e"></param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/9/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub ParentOnLoad(ByVal Sender As Object, ByVal e As System.EventArgs)
            DotNetNuke.UI.Utilities.ClientAPI.RegisterDNNVariableControl(Me)
            'LoadPostedVars()
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Registers client-side script for the control
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	8/9/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub RegisterClientScript()
            If IsDownLevel = False Then
                DotNetNuke.UI.Utilities.ClientAPI.RegisterClientReference(Me.Page, DotNetNuke.UI.Utilities.ClientAPI.ClientNamespaceReferences.dnn_dom)
                'DotNetNuke.UI.Utilities.ClientAPI.RegisterClientReference(Me.Page, DotNetNuke.UI.Utilities.ClientAPI.ClientNamespaceReferences.dnn_xml)
                Dim blnCallbackExists As Boolean = Me.TabRenderMode = eTabRenderMode.CallBack
                For Each objTab As DNNTab In Me.Tabs
                    If objTab.TabRenderMode = eTabRenderMode.CallBack Then blnCallbackExists = True
                    If blnCallbackExists Then Exit For
                Next
                If blnCallbackExists AndAlso DotNetNuke.UI.Utilities.ClientAPI.BrowserSupportsFunctionality(Utilities.ClientAPI.ClientFunctionality.XMLHTTP) Then
                    DotNetNuke.UI.Utilities.ClientAPI.RegisterClientReference(Me.Page, DotNetNuke.UI.Utilities.ClientAPI.ClientNamespaceReferences.dnn_xmlhttp)
                End If
                RegisterClientScriptBlock(Me.Page, "dnn.controls.js")
                RegisterClientScriptBlock(Me.Page, "dnn.controls.dnntabstrip.js")
                'If Not ClientAPI.IsClientScriptBlockRegistered(Me.Page, "dnn.controls.dnntabstrip.js") Then
                '    ClientAPI.RegisterClientScriptBlock(Me.Page, "dnn.controls.dnntabstrip.js", "<script src=""" & TabStripScriptPath & "dnn.controls.dnntabstrip.js""></script>")
                'End If
                'ClientAPI.RegisterStartUpScript(Page, Me.ClientID & "_startup", "<script type="text/javascript">dnn.controls.initTabStrip($get('" & Me.ClientID & "'));</script>")			 'wrong place
            End If
        End Sub
#End Region

        Protected Function MarshalledProperties() As Hashtable
            Dim properties As Hashtable = New Hashtable
            properties.Add("tabs", GetTabIds)
            'If PopulateNodesFromClient Then
            If Me.IsDownLevel = False Then
                Select Case TabRenderMode
                    Case eTabRenderMode.CallBack
                        If DotNetNuke.UI.Utilities.ClientAPI.BrowserSupportsFunctionality(Utilities.ClientAPI.ClientFunctionality.XMLHTTP) Then
                            Dim strCallback As String = DotNetNuke.UI.Utilities.ClientAPI.GetCallbackEventReference(Me, "'[TABID]'", "this.callBackSuccess", "this", "this.callBackFail", "this.callBackStatus", "[POST]", ClientAPICallBackResponse.CallBackTypeCode.Simple)

                            'TODO:  Enhance the callback to accept a string for last param to avoid this hack
                            'minor hack to allow substitution of callbacktypes on tab level
                            'dnn.xmlhttp.doCallBack('MyDNNTabStrip MyDNNTabStrip','[TABID]',this.callBackSuccess,this,this.callBackFail,this.callBackStatus,null,'[POST]',0);
                            'dnn.xmlhttp.doCallBack('MyDNNTabStrip MyDNNTabStrip','[TABID]',this.callBackSuccess,this,this.callBackFail,this.callBackStatus,null,'[POST]',[CBTYPE]);
                            Dim intPos As Integer = strCallback.IndexOf("'[POST]'") + "'[POST]'".Length + 1
                            strCallback = strCallback.Substring(0, intPos) & "'[CBTYPE]'" & strCallback.Substring(intPos + 1)
                            properties.Add("callback", strCallback)
                        ElseIf Me.IsDownLevel Then
                            properties.Add("callback", ClientAPI.GetPostBackClientHyperlink(Me, "[TABID]" & ClientAPI.COLUMN_DELIMITER & "OnDemand"))
                        End If
                    Case eTabRenderMode.PostBack
                        properties.Add("callback", ClientAPI.GetPostBackClientHyperlink(Me, "[TABID]" & ClientAPI.COLUMN_DELIMITER & "OnDemand"))
                    Case eTabRenderMode.All

                End Select
            End If

            Return properties
        End Function

        Private Function GetScripts(ByVal HTML As String) As HTMLElementCollection
            Dim strExpression As String '= "(?i:(?:<(?<element>script[^/ >]*)(?:\s(?!/)+(?:(?<attr>[^=]+)=(?:""|')(?<attrv>[^""\']+)(" & "?:""|')))*)(?:[^/]*/>|[^/]{0}>(?<text>[\s\S]*)(?<close></\k<element>>+)))"
            '(?i:
            '	(?<element>(?:<script
            '		(?:\s*
            '		(?:
            '			(?<attr>[^=>]*?)
            '			=(?:"|')
            '			(?<attrv>[^"|']*?)
            '			(?:"|')
            '		))*
            '        )
            '	(
            '(?(?=\s*?/>)\s*?/>
            '|
            '                (?:\s*?>
            '	(?:[\s\r\n]*?<!--)?(?<text>[\s\S]*?)
            '                </script>))
            '	))
            ')
            strExpression = _
"(?i:" & _
"	(?<element>(?:<script" & _
"		(?:\s*" & _
"		(?:" & _
"			(?<attr>[^=>]*?)" & _
"			=(?:""|')" & _
"			(?<attrv>[^""|']*?)" & _
"			(?:""|')" & _
"		))*" & _
"        )" & _
"	(" & _
"(?(?=\s*?/>)\s*?/>" & _
"|" & _
"                (?:\s*?>" & _
"	(?:[\s\r\n]*?<!--)?(?<text>[\s\S]*?)" & _
"                </script>))" & _
"	)" & _
"))"

            Dim oRE As System.Text.RegularExpressions.Regex = New System.Text.RegularExpressions.Regex(strExpression, RegularExpressions.RegexOptions.IgnorePatternWhitespace Or RegularExpressions.RegexOptions.IgnoreCase Or RegularExpressions.RegexOptions.Singleline)

            Dim strGroup As String
            Dim oCol As HTMLElementCollection = New HTMLElementCollection
            Dim objElement As HTMLElement
            Dim objAttr As Collection   'Todo: the association between attr and attrv is a minor hack here... think of something better!
            Dim intAttr As Integer
            For Each oMatch As RegularExpressions.Match In oRE.Matches(HTML)
                objElement = Nothing

                For iGroup As Integer = 0 To oMatch.Groups.Count - 1
                    strGroup = oRE.GroupNameFromNumber(iGroup)
                    If strGroup = "attr" Then objAttr = New Collection
                    intAttr = 1
                    For Each oCapture As RegularExpressions.Capture In oMatch.Groups(iGroup).Captures
                        Select Case strGroup
                            Case "element"
                                objElement = New HTMLElement(oCapture.Value)
                                objElement.Raw = oMatch.Value
                            Case "attr"
                                objAttr.Add(oCapture.Value)
                            Case "attrv"
                                If CStr(objAttr(intAttr)) = "src" Then
                                    'need to replace &amp; with &  (webresource.axd for IE6)
                                    objElement.Attributes.Add(objAttr(intAttr), System.Web.HttpUtility.HtmlDecode(oCapture.Value))
                                Else
                                    objElement.Attributes.Add(objAttr(intAttr), oCapture.Value)
                                End If
                                intAttr += 1
                            Case "text"
                                objElement.Text = oCapture.Value
                        End Select

                    Next
                Next
                If Not objElement Is Nothing Then
                    oCol.Add(objElement)
                End If
            Next
            Return oCol
        End Function

        Private Function GetFormHTML() As String
            Dim objSB As System.Text.StringBuilder = New System.Text.StringBuilder
            Dim objSW As System.IO.StringWriter = New System.IO.StringWriter(objSB)
            Dim objHW As HtmlTextWriter = New HtmlTextWriter(objSW)
            Dim sFormHTML As String = ""

            'todo:  regexp to match scripts, regexp to match __dnnvariable
            Dim oCtl As Control = FindForm(Me)
            If Not oCtl Is Nothing Then
                oCtl.Controls.Clear()
                oCtl.RenderControl(objHW)
                sFormHTML = objSB.ToString
            End If
            Return sFormHTML
        End Function

        'TODO: make public in capi!
        Private Shared Function FindForm(ByVal oCtl As Control) As HtmlControls.HtmlForm
            Do While Not TypeOf oCtl Is HtmlControls.HtmlForm
                If oCtl Is Nothing OrElse TypeOf oCtl Is Page Then Return Nothing
                oCtl = oCtl.Parent
            Loop
            Return CType(oCtl, HtmlControls.HtmlForm)
        End Function

    End Class
End Namespace
