'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2009
' by DotNetNuke Corporation
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System
Imports System.Collections
Imports System.ComponentModel
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Diagnostics
Imports System.Xml
Imports DotNetNuke.UI.Utilities
Imports DotNetNuke.UI.WebControls

Namespace DotNetNuke.UI.WebControls
	<Designer(GetType(Design.WebControls.DNNTextSuggestDesigner)), ToolboxData("<{0}:DNNTextSuggest runat=server></{0}:DNNTextSuggest>")> _
	 Public Class DNNTextSuggest
		Inherits TextBox
		Implements IPostBackEventHandler
		Implements IClientAPICallbackEventHandler

		Public Enum eIDTokenChar
			None
			Brackets
			Paranthesis
		End Enum

#Region "Events / Delegates"
		Public Delegate Sub DNNTextSuggestEventHandler(ByVal source As Object, ByVal e As DNNTextSuggestEventArgs)
		Public Delegate Sub DNNDNNNodeClickHandler(ByVal source As Object, ByVal e As DNNTextSuggestEventArgs)

		Public Event NodeClick As DNNDNNNodeClickHandler
		Public Event PopulateOnDemand As DNNTextSuggestEventHandler
#End Region

#Region "Member Variables"
		Private m_objNodes As DNNNodeCollection
#End Region

#Region "Constructors"
		''' -----------------------------------------------------------------------------
		''' <summary>
		''' 
		''' </summary>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	5/6/2005	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Sub New()
		End Sub
#End Region

#Region "Properties"
        <Bindable(True), DefaultValue(""), ClientProperty(), ClientPropertyName("bid")> _
        Public Property BehaviorID() As String
            Get
                Return CType(ViewState("BehaviorID"), String)
            End Get
            Set(ByVal Value As String)
                ViewState("BehaviorID") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Allows developer to force the rendering of the Menu in DownLevel mode
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/22/2005	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Behavior"), Description("Allows developer to force the rendering of the Menu in DownLevel mode"), DefaultValue(False)> _
        Public Property ForceDownLevel() As Boolean
            Get
                Return CType(ViewState("ForceDownLevel"), Boolean)
            End Get
            Set(ByVal Value As Boolean)
                ViewState("ForceDownLevel") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns whether the TextSuggest will render DownLevel or not
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/22/2005	Commented
        '''		[Jon Henning]	3/9/2005	Requiring XML support on client for uplevel
        '''		[Jon Henning]	8/21/2007	No longer Requiring XML support on client for uplevel
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Browsable(False)> _
        Public ReadOnly Property IsDownLevel() As Boolean
            Get
                If ForceDownLevel OrElse _
                 ClientAPI.BrowserSupportsFunctionality(ClientAPI.ClientFunctionality.DHTML) = False Then
                    Return True
                Else
                    Return False
                End If
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Location of ClientAPI js files 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/22/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Paths"), Description("Location of ClientAPI js files"), DefaultValue("")> _
        Public Property ClientAPIScriptPath() As String
            Get
                Return ClientAPI.ScriptPath
            End Get
            Set(ByVal Value As String)
                ClientAPI.ScriptPath = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Location of dnn.controls.DNNTextSuggest.js file
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' Since 1.1 this path will be the same as the ClientAPI path.
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/22/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Paths"), Description("Location of dnn.controls.DNNTextSuggest.js file"), DefaultValue("")> _
        Public Property TextSuggestScriptPath() As String
            Get
                If Len(ViewState("TextSuggestScriptPath")) = 0 Then
                    Return ClientAPIScriptPath
                Else
                    Return CStr(ViewState("TextSuggestScriptPath"))
                End If
            End Get
            Set(ByVal Value As String)
                ViewState("TextSuggestScriptPath") = Value
            End Set
        End Property


        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Location of system images (i.e. spacer.gif)
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/22/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Paths"), Description("Directory to find the images for the TextSuggest."), DefaultValue("images/"), ClientProperty(), ClientPropertyName("sysimgpath")> _
  Public Property SystemImagesPath() As String
            Get
                If Len(ViewState("SystemImagesPath")) = 0 Then
                    Return "images/"
                Else
                    Return CStr(ViewState("SystemImagesPath"))
                End If
            End Get
            Set(ByVal Value As String)
                ViewState("SystemImagesPath") = Value
            End Set
        End Property


        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' This property really makes no sense for this control and should not be here, 
        ''' but due to concerns for backwards compatibility I am keeping.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public ReadOnly Property DNNNodes() As DNNNodeCollection
            Get
                If m_objNodes Is Nothing Then
                    m_objNodes = New DNNNodeCollection(Me.ClientID)
                End If
                Return m_objNodes
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Holds collection of selected node objects
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	4/6/2005	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Browsable(False)> _
        Public ReadOnly Property SelectedNodes() As DNNNodeCollection
            Get
                Dim objNodes As DNNNodeCollection = New DNNNodeCollection("")
                Dim objNode As DNNNode
                If Page.IsPostBack Then
                    Dim aryEntries() As String = Me.Text.Split(CChar(Me.Delimiter))
                    Dim strText As String
                    Dim strKey As String
                    For Each strEntry As String In aryEntries
                        'wish I knew regular expressions better...
                        If Len(strEntry) > 0 Then
                            strText = strEntry
                            strKey = ""
                            Select Case Me.IDToken
                                Case eIDTokenChar.None
                                Case eIDTokenChar.Brackets
                                    'text [key]
                                    Dim intTextEnd As Integer = strEntry.LastIndexOf(" [")
                                    Dim intKeyBegin As Integer = intTextEnd + " [".Length
                                    Dim intKeyEnd As Integer = strEntry.LastIndexOf("]")
                                    If intTextEnd > -1 AndAlso intKeyEnd > intKeyBegin Then
                                        strText = strEntry.Substring(0, intTextEnd)
                                        strKey = strEntry.Substring(intKeyBegin, intKeyEnd - intKeyBegin)
                                    End If
                                Case eIDTokenChar.Paranthesis
                            End Select
                            objNode = New DNNNode(strText)
                            objNode.Key = strKey
                            objNodes.Add(objNode)
                        End If
                    Next
                End If
                Return objNodes
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' If ClickAction for a node is set to navigate this is the target frame that 
        '''	will do the navigating.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Behavior"), Description("If ClickAction for a node is set to navigate this is the target frame that will do the navigating."), DefaultValue(""), ClientProperty(), ClientPropertyName("target")> _
    Public Property Target() As String
            Get
                Return (CType(ViewState("Target"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("Target") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Default Classname for node.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Appearance"), Description("Default Classname for node."), DefaultValue(""), ClientProperty(), ClientPropertyName("css")> _
        Public Property DefaultNodeCssClass() As String
            Get
                Return (CType(ViewState("DefaultNodeCssClass"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("DefaultNodeCssClass") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Default Classname for child node.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Appearance"), Description("Default Classname for child node."), DefaultValue(""), ClientProperty(), ClientPropertyName("csschild")> _
        Public Property DefaultChildNodeCssClass() As String
            Get
                Return (CType(ViewState("DefaultChildNodeCssClass"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("DefaultChildNodeCssClass") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Default Classname for node when hovered.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Appearance"), Description("Default Classname for node when hovered."), DefaultValue(""), ClientProperty(), ClientPropertyName("csshover")> _
        Public Property DefaultNodeCssClassOver() As String
            Get
                Return (CType(ViewState("DefaultNodeCssClassOver"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("DefaultNodeCssClassOver") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Default Classname for node when selected.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Appearance"), Description("Default Classname for node when selected."), DefaultValue(""), ClientProperty(), ClientPropertyName("csssel")> _
          Public Property DefaultNodeCssClassSelected() As String
            Get
                Return (CType(ViewState("DefaultNodeCssClassSelected"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("DefaultNodeCssClassSelected") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Default Classname container holding all of suggestion nodes.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	5/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Appearance"), Description("Default Classname container holding all of suggestion nodes."), DefaultValue(""), ClientProperty(), ClientPropertyName("tscss")> _
  Public Property TextSuggestCssClass() As String
            Get
                Return (CType(ViewState("TextSuggestCssClass"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("TextSuggestCssClass") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Allows you to have a common JS function be invoked for all nodes, unless
        ''' a different JS function is provided on the node level.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' If the client-side function returns false, the action associated to the node 
        ''' selection will be canceled
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	4/6/2005	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Behavior"), Description("Allows you to have a common JS function be invoked for all nodes, unless a different JS function is provided on the node level."), DefaultValue(""), ClientProperty(), ClientPropertyName("js")> _
  Public Property JSFunction() As String
            Get
                Return (CType(ViewState("JSFunction"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("JSFunction") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' If callbacks are supported/enabled, this javascript function will be invoked
        ''' with each status change of the xmlhttp request.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' The Javascript transport does not raise any events.
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	4/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Behavior"), Description("If callbacks are supported/enabled, this javascript function will be invoked with each status change of the xmlhttp request."), DefaultValue(""), ClientProperty(), ClientPropertyName("callbackSF")> _
        Public Property CallbackStatusFunction() As String
            Get
                Return (CType(ViewState("CallbackStatusFunction"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("CallbackStatusFunction") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Specifies a delimiter to be used to allow for multiple entries to be added.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' This should be a safe character that is not going to be used as an entry itself.
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/24/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Behavior"), Description("Specifies a delimiter to be used to allow for multiple entries to be added."), DefaultValue(""), ClientProperty(), ClientPropertyName("del")> _
  Public Property Delimiter() As Char
            Get
                Return (CType(ViewState("Delimiter"), Char))
            End Get
            Set(ByVal Value As Char)
                ViewState("Delimiter") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Specifies a type of character to be used to surround/delimit the underlying 
        '''	value/id of the selected item
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' An example of the text generated with this set to Brackets would be Smith [123]
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/24/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Behavior"), Description("Specifies a type of character to be used to surround/delimit the underlying value/id of the selected item."), DefaultValue(eIDTokenChar.None)> _
        Public Property IDToken() As eIDTokenChar
            Get
                Return (CType(ViewState("IDToken"), eIDTokenChar))
            End Get
            Set(ByVal Value As eIDTokenChar)
                ViewState("IDToken") = CInt(Value)
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Number of characters typed before a lookup is performed
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/24/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Behavior"), Description("Minimum number of characters typed before a lookup will be invoked"), DefaultValue(1), ClientProperty(), ClientPropertyName("minChar")> _
        Public Property MinCharacterLookup() As Integer
            Get
                If Len(ViewState("MinCharacterLookup")) = 0 Then
                    Return 1
                Else
                    Return (CType(ViewState("MinCharacterLookup"), Integer))
                End If
            End Get
            Set(ByVal Value As Integer)
                ViewState("MinCharacterLookup") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Maximum number of rows to display.  
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        '''	This is important since it will allow the client side code to determine when
        '''	a new lookup is needed.  As a developer you need to return MaxSuggestRows + 1
        '''	results down to the client in order for it to determine when a lookup is required.
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/24/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Behavior"), Description("Maximum number of rows to display."), DefaultValue(10), ClientProperty(), ClientPropertyName("maxRows")> _
        Public Property MaxSuggestRows() As Integer
            Get
                If Len(ViewState("MaxSuggestRows")) = 0 Then
                    Return 10
                Else
                    Return (CType(ViewState("MaxSuggestRows"), Integer))
                End If
            End Get
            Set(ByVal Value As Integer)
                ViewState("MaxSuggestRows") = Value
            End Set
        End Property


        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Number of milliseconds to wait after keypress before a lookup takes place
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        '''	Defaults to half a second (500)
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/24/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Behavior"), Description("Number of milliseconds to wait after keypress before a lookup takes place."), DefaultValue(500), ClientProperty(), ClientPropertyName("ludelay")> _
        Public Property LookupDelay() As Integer
            Get
                If Len(ViewState("LookupDelay")) = 0 Then
                    Return 500
                Else
                    Return (CType(ViewState("LookupDelay"), Integer))
                End If
            End Get
            Set(ByVal Value As Integer)
                ViewState("LookupDelay") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Number of milliseconds to wait after lostfocus before a menu is hidden
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        '''	Defaults to half a second (500)
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	9/29/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Behavior"), Description("Number of milliseconds to wait after focus is lost before a hiding the results."), DefaultValue(500), ClientProperty(), ClientPropertyName("lfdelay")> _
  Public Property LostFocusDelay() As Integer
            Get
                If Len(ViewState("LostFocusDelay")) = 0 Then
                    Return 500
                Else
                    Return (CType(ViewState("LostFocusDelay"), Integer))
                End If
            End Get
            Set(ByVal Value As Integer)
                ViewState("LostFocusDelay") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Determines if lookup uses a case sensitve match
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/24/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Category("Behavior"), Description("Determines if lookup uses a case sensitve match."), DefaultValue(False), ClientProperty(), ClientPropertyName("casesens")> _
  Public Property CaseSensitive() As Boolean
            Get
                If Len(ViewState("CaseSensitive")) = 0 Then
                    Return False
                Else
                    Return (CType(ViewState("CaseSensitive"), Boolean))
                End If
            End Get
            Set(ByVal Value As Boolean)
                ViewState("CaseSensitive") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' This property determines what type of callback is used
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' The Simple callback short-circuits at the Page Init event while the ProcessPage
        ''' will run the page through the entire page's lifecycle.
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	9/14/2006	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), DefaultValue(ClientAPICallBackResponse.CallBackTypeCode.Simple), Description("Image to display during a callback"), Category("Behavior")> _
        Public Property CallBackType() As ClientAPICallBackResponse.CallBackTypeCode
            Get
                If Len(ViewState("CallBackTypeCode")) > 0 Then
                    Return CType(ViewState("CallBackTypeCode"), ClientAPICallBackResponse.CallBackTypeCode)
                Else
                    Return ClientAPICallBackResponse.CallBackTypeCode.Simple
                End If
            End Get
            Set(ByVal Value As ClientAPICallBackResponse.CallBackTypeCode)
                ViewState("CallBackTypeCode") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' This property allows for data on page to be posted in callback
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	9/17/2008	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), Description("This property allows for data on page to be posted in callback"), Category("Behavior")> _
        Public Property CallBackPostChildrenOf() As String
            Get
                If Len(ViewState("CallBackPostChildrenOf")) > 0 Then
                    Return CStr(ViewState("CallBackPostChildrenOf"))
                Else
                    Return ""
                End If
            End Get
            Set(ByVal Value As String)
                ViewState("CallBackPostChildrenOf") = Value
            End Set
        End Property

#End Region

#Region "OverLoaded Methods"

        'Protected Overrides Sub AddAttributesToRender(ByVal writer As System.Web.UI.HtmlTextWriter)
        '	MyBase.AddAttributesToRender(writer)
        '	'writer.AddAttribute(HtmlTextWriterAttribute.Class, Me.CssClass)

        '          'writer.AddAttribute("sysimgpath", Me.SystemImagesPath)
        '          'If Len(Me.Target) > 0 Then writer.AddAttribute("target", Me.Target)

        '	'css attributes
        '          'If Len(Me.TextSuggestCssClass) > 0 Then writer.AddAttribute("tscss", Me.TextSuggestCssClass)
        '          'If Len(Me.DefaultNodeCssClass) > 0 Then writer.AddAttribute("css", Me.DefaultNodeCssClass)
        '          'If Len(Me.DefaultChildNodeCssClass) > 0 Then writer.AddAttribute("csschild", Me.DefaultChildNodeCssClass)
        '          'If Len(Me.DefaultNodeCssClassOver) > 0 Then writer.AddAttribute("csshover", Me.DefaultNodeCssClassOver)
        '          'If Len(Me.DefaultNodeCssClassSelected) > 0 Then writer.AddAttribute("csssel", Me.DefaultNodeCssClassSelected)

        '          'If Len(Me.JSFunction) > 0 Then writer.AddAttribute("js", Me.JSFunction)
        '          'If Len(Me.Delimiter) > 0 Then writer.AddAttribute("del", Me.Delimiter)
        '          'Select Case Me.IDToken
        '          '	Case eIDTokenChar.None
        '          '	Case eIDTokenChar.Brackets
        '          '		writer.AddAttribute("idtok", "[~]")
        '          '	Case eIDTokenChar.Paranthesis
        '          '		writer.AddAttribute("idtok", "(~)")
        '          'End Select

        '          'If Me.MinCharacterLookup > 1 Then writer.AddAttribute("minchar", Me.MinCharacterLookup.ToString)
        '          'If Me.MaxSuggestRows <> 10 Then writer.AddAttribute("maxrows", Me.MaxSuggestRows.ToString)
        '          'If Me.LookupDelay <> 500 Then writer.AddAttribute("ludelay", Me.LookupDelay.ToString)
        '          'If Me.LostFocusDelay <> 500 Then writer.AddAttribute("lfdelay", Me.LostFocusDelay.ToString)

        '          'If Me.CaseSensitive Then writer.AddAttribute("casesens", "1")


        '          'writer.AddAttribute("postback", ClientAPI.GetPostBackEventReference(Me, "[TEXT]" & ClientAPI.COLUMN_DELIMITER & "Click"))

        '          'If ClientAPI.BrowserSupportsFunctionality(ClientAPI.ClientFunctionality.XMLHTTP) Then
        '          '	writer.AddAttribute("callback", ClientAPI.GetCallbackEventReference(Me, "this.getText()", "this.callBackSuccess", "this", "this.callBackFail", "this.callBackStatus", CallBackType))
        '          'End If

        '          'If Len(Me.CallbackStatusFunction) > 0 Then
        '          '	writer.AddAttribute("callbackSF", Me.CallbackStatusFunction)
        '          'End If

        '          'If Len(Me.JSFunction) > 0 Then
        '          '	writer.AddAttribute("js", Me.JSFunction)
        '          'End If
        'End Sub

        Protected Function MarshalledProperties() As Hashtable
            Dim properties As Hashtable = New Hashtable

            Select Case Me.IDToken
                Case eIDTokenChar.None
                Case eIDTokenChar.Brackets
                    properties.Add("idtok", "[~]")
                Case eIDTokenChar.Paranthesis
                    properties.Add("idtok", "(~)")
            End Select

            properties.Add("postback", ClientAPI.GetPostBackEventReference(Me, "[TEXT]" & ClientAPI.COLUMN_DELIMITER & "Click"))

            If ClientAPI.BrowserSupportsFunctionality(ClientAPI.ClientFunctionality.XMLHTTP) Then
                properties.Add("callback", ClientAPI.GetCallbackEventReference(Me, "this.getText()", "this.callBackSuccess", "this", "this.callBackFail", "this.callBackStatus", CallBackPostChildrenOf, CallBackType))
            End If

            Return properties
        End Function


#End Region

#Region "Event Handlers"
		''' -----------------------------------------------------------------------------
		''' <summary>
		''' 
		''' </summary>
		''' <param name="eventArgument"></param>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	5/6/2005	Created
		'''		[Jon Henning]	2/21/2006	Fixed arg to not pass Click text
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Overridable Sub RaisePostBackEvent(ByVal eventArgument As String) Implements IPostBackEventHandler.RaisePostBackEvent
			Dim args() As String = Split(eventArgument, ClientAPI.COLUMN_DELIMITER)

			If args.Length > 1 Then
				Select Case args(1)
					Case "Click"
						Dim oArg As DNNTextSuggestEventArgs = New DNNTextSuggestEventArgs(Me.DNNNodes, args(0))
						OnNodeClick(oArg)
				End Select
			End If
		End Sub

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' 
		''' </summary>
		''' <param name="e"></param>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	10/6/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Overridable Sub OnNodeClick(ByVal e As DNNTextSuggestEventArgs)
			RaiseEvent NodeClick(Me, e)
		End Sub

		Private Sub DNNTextSuggest_PreRender(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.PreRender
			RegisterClientScript()
			Page.RegisterRequiresPostBack(Me)
		End Sub

		Protected Overrides Sub Render(ByVal writer As System.Web.UI.HtmlTextWriter)
			MyBase.Render(writer)
			If IsDownLevel = False Then
                Dim properties As Hashtable = MarshalledProperties() 'get runtime generated properties (non attribute based)
                Dim json As String = ScriptGenerator.GetMarshalledPropertyJSON(Me, properties)
                BaseWebControl.RegisterInitialize(Me, "initTextSuggest", json)
            End If
		End Sub

		Public Function RaiseClientAPICallbackEvent(ByVal eventArgument As String) As String Implements IClientAPICallbackEventHandler.RaiseClientAPICallbackEvent
			RaiseEvent PopulateOnDemand(Me, New DNNTextSuggestEventArgs(Me.DNNNodes, eventArgument))
            Return Me.DNNNodes.ToJSON
		End Function

#End Region

#Region "Methods"

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Determines if client supports an uplevel rendering, and if so it registers
		''' the appropriate scripts.
		''' </summary>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	4/6/2005	Commented
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Sub RegisterClientScript()
			If IsDownLevel = False Then
				ClientAPI.RegisterClientReference(Me.Page, ClientAPI.ClientNamespaceReferences.dnn_dom)
                'ClientAPI.RegisterClientReference(Me.Page, ClientAPI.ClientNamespaceReferences.dnn_xml)
				ClientAPI.RegisterClientReference(Me.Page, ClientAPI.ClientNamespaceReferences.dnn_dom_positioning)
				ClientAPI.RegisterClientReference(Me.Page, ClientAPI.ClientNamespaceReferences.dnn_xmlhttp)

                RegisterClientScriptBlock(Me.Page, "dnn.controls.js")
                RegisterClientScriptBlock(Me.Page, "dnn.controls.dnntextsuggest.js")
                'If Not ClientAPI.IsClientScriptBlockRegistered(Me.Page, "dnn.controls.DNNTextSuggest.js") Then
                '	ClientAPI.RegisterClientScriptBlock(Me.Page, "dnn.controls.dnnTextSuggest.js", "<script src=""" & TextSuggestScriptPath & "dnn.controls.dnnTextSuggest.js""></script>")
                'End If
                'ClientAPI.RegisterStartUpScript(Page, Me.ClientID & "_startup", "<script type="text/javascript">dnn.controls.initTextSuggest($get('" & Me.ClientID & "'));</script>")
			End If
		End Sub

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Finds Node by passed in Key. 
		''' </summary>
		''' <param name="strID">ID of node</param>
		''' <returns></returns>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	11/17/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Function FindNode(ByVal strID As String) As DNNNode
			Return Me.DNNNodes.FindNode(strID)
		End Function

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' Finds Node by passed in Key. 
		''' </summary>
		''' <param name="strKey">Key of node</param>
		''' <returns></returns>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[Jon Henning]	11/17/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Function FindNodeByKey(ByVal strKey As String) As DNNNode
			Return Me.DNNNodes.FindNodeByKey(strKey)
		End Function

		Public Sub LoadXml(ByVal strXml As String)
			m_objNodes = New DNNNodeCollection(strXml, "")
		End Sub

#End Region

	End Class
End Namespace
