'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2009
' by DotNetNuke Corporation
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System
Imports System.Web.UI
Imports System.ComponentModel

Namespace DotNetNuke.UI.WebControls

	<ParseChildren(False), PersistChildren(True)> _
	Public Class DNNToolBarButton
		Inherits Control
		Implements ICustomTypeDescriptor
		'		Implements IStateManager
		Private m_blnMarked As Boolean = False
		Private m_objState As StateBag
		Private m_objToolbar As DNNToolBar

		Friend CommonActions() As String = {"js", "navigate"}

#Region "Constructors"
		Public Sub New()
		End Sub
#End Region

#Region "Properties"

		<Browsable(False)> _
		Private ReadOnly Property Owner() As DNNToolBar
			Get
				If m_objToolbar Is Nothing Then
					m_objToolbar = CType(Me.Parent, DNNToolBar)
				End If
				Return m_objToolbar
			End Get
		End Property

		'<DefaultValue("")> _
		Public Property Key() As String
			Get
				'if it is a common action then allow custom key, otherwise use action as key
				If Array.IndexOf(CommonActions, Me.ControlAction) > -1 Then
					If MyState("Key") Is Nothing Then
						Return ""
					Else
						Return (CType(MyState("Key"), String))
					End If
				Else
					Return Me.ControlAction
				End If
			End Get
			Set(ByVal Value As String)
				MyState("Key") = Value
			End Set
		End Property

		<DefaultValue(""), Category("Appearance")> _
		Public Property CssClass() As String
			Get
				If MyState("CssClass") Is Nothing Then
					Return ""
				Else
					Return (CType(MyState("CssClass"), String))
				End If
			End Get
			Set(ByVal Value As String)
				MyState("CssClass") = Value
			End Set
		End Property

		<DefaultValue(""), Category("Appearance")> _
		Public Property CssClassHover() As String
			Get
				If MyState("CssClassHover") Is Nothing Then
					Return ""
				Else
					Return (CType(MyState("CssClassHover"), String))
				End If
			End Get
			Set(ByVal Value As String)
				MyState("CssClassHover") = Value
			End Set
		End Property

		<DefaultValue(""), Category("Behavior"), TypeConverter(GetType(Design.WebControls.DNNToolBarButtonActionTypeConverter))> _
		Public Property ControlAction() As String
			Get
				If MyState("ControlAction") Is Nothing Then
					Return ""
				Else
					Return (CType(MyState("ControlAction"), String))
				End If
			End Get
			Set(ByVal Value As String)
				MyState("ControlAction") = Value
				TypeDescriptor.Refresh(Me)				 'make sure we refresh the property grid
			End Set
		End Property

		<DefaultValue(""), Category("Appearance")> _
		Public Property Text() As String
			Get
				If MyState("Text") Is Nothing Then
					Return ""
				Else
					Return (CType(MyState("Text"), String))
				End If
			End Get
			Set(ByVal Value As String)
				MyState("Text") = Value
			End Set
		End Property

		<DefaultValue(""), Category("Appearance")> _
		Public Property ToolTip() As String
			Get
				If MyState("ToolTip") Is Nothing Then
					Return ""
				Else
					Return (CType(MyState("ToolTip"), String))
				End If
			End Get
			Set(ByVal Value As String)
				MyState("ToolTip") = Value
			End Set
		End Property

		<DefaultValue(""), Category("Appearance")> _
		Public Property ImageUrl() As String
			Get
				If MyState("ImageUrl") Is Nothing Then
					Return ""
				Else
					Return (CType(MyState("ImageUrl"), String))
				End If
			End Get
			Set(ByVal Value As String)
				MyState("ImageUrl") = Value
			End Set
		End Property

		<DefaultValue(""), Category("Behavior")> _
		Public Property NavigateUrl() As String
			Get
				If MyState("NavigateUrl") Is Nothing Then
					Return ""
				Else
					Return (CType(MyState("NavigateUrl"), String))
				End If
			End Get
			Set(ByVal Value As String)
				MyState("NavigateUrl") = Value
			End Set
		End Property

		<DefaultValue(""), Category("Behavior")> _
		Public Property JSFunction() As String
			Get
				If MyState("JSFunction") Is Nothing Then
					Return ""
				Else
					Return (CType(MyState("JSFunction"), String))
				End If
			End Get
			Set(ByVal Value As String)
				MyState("JSFunction") = Value
			End Set
		End Property

		Private Property MyState(ByVal strKey As String) As Object
			Get
				Return ViewState(strKey)
			End Get
			Set(ByVal Value As Object)
				ViewState(strKey) = Value
				NotifyDesigner()
			End Set
		End Property

#End Region

#Region "Public Methods"
		Public Function ToJSON() As String
			Dim oHash As Hashtable = New Hashtable
			Dim sb As System.Text.StringBuilder = New System.Text.StringBuilder
			If Len(Me.CssClass) > 0 Then
				oHash.Add("css", SafeJSONString(Me.CssClass))
			End If
			If Len(Me.CssClassHover) > 0 Then
				oHash.Add("cssh", SafeJSONString(Me.CssClassHover))
			End If
			If Len(Me.ImageUrl) > 0 Then
				oHash.Add("img", SafeJSONString(Me.ImageUrl))
			End If

			If Len(Me.Key) > 0 Then
				oHash.Add("key", SafeJSONString(Me.Key))
			End If
			If Len(Me.JSFunction) > 0 AndAlso Me.ControlAction = "js" Then
				oHash.Add("js", SafeJSONString(Me.JSFunction))
			End If
			If Len(Me.NavigateUrl) > 0 AndAlso Me.ControlAction = "navigate" Then
				oHash.Add("url", SafeJSONString(Me.NavigateUrl))
			End If
			If Len(Me.ControlAction) > 0 Then
				oHash.Add("ca", SafeJSONString(Me.ControlAction))
			End If
			If Len(Me.Text) > 0 Then
				oHash.Add("txt", SafeJSONString(Me.Text))
			End If
			If Len(Me.ToolTip) > 0 Then
				oHash.Add("alt", SafeJSONString(Me.ToolTip))
			End If
			If Me.Visible = False Then
				oHash.Add("vis", CInt(Me.Visible))
			End If
			sb.Append("{")
			For Each strKey As String In oHash.Keys
				If sb.Length > 1 Then sb.Append(",")
				sb.Append(strKey & ":" & CStr(oHash(strKey)))
			Next
			sb.Append("}")
			Return sb.ToString
		End Function

		Private Function SafeJSONString(ByVal strString As String) As String
			Return "'" & Replace(strString, "'", "\'") & "'"
		End Function
#End Region

#Region "Private Methods"
		Private Sub NotifyDesigner()
            If Not Owner Is Nothing AndAlso WebControls.IsDesignMode Then
                Owner.NotifyDesigner()
            End If
		End Sub
#End Region

#Region "Overrides"
		Protected Overrides Sub Render(ByVal writer As System.Web.UI.HtmlTextWriter)
			'No output to render
		End Sub
#End Region

#Region "State Management"
		'		Public ReadOnly Property IsTrackingViewState() As Boolean Implements IStateManager.IsTrackingViewState
		'			Get
		'				Return m_blnMarked
		'			End Get
		'		End Property

		'		Public Sub TrackViewState() Implements IStateManager.TrackViewState
		'			m_blnMarked = True
		'		End Sub

		'		Public Function SaveViewState() As Object Implements IStateManager.SaveViewState
		'			' save m_objState state
		'			Dim objState As Object = Nothing
		'			If Not (m_objState Is Nothing) Then
		'				objState = CType(m_objState, IStateManager).SaveViewState()
		'			End If
		'			Return objState
		'		End Function

		'		Public Sub LoadViewState(ByVal state As Object) Implements IStateManager.LoadViewState
		'			If Not (state Is Nothing) Then
		'				CType(ViewState, IStateManager).LoadViewState(state)
		'			End If
		'		End Sub

		'		Protected ReadOnly Property ViewState() As StateBag
		'			Get
		'				If m_objState Is Nothing Then
		'					m_objState = New StateBag(True)
		'					If IsTrackingViewState Then
		'						CType(m_objState, IStateManager).TrackViewState()
		'					End If
		'				End If
		'				Return m_objState
		'			End Get
		'		End Property

		'		Friend Sub SetDirty()
		'			If Not m_objState Is Nothing Then
		'				Dim key As String
		'				For Each key In m_objState.Keys
		'					m_objState.SetItemDirty(key, True)
		'				Next
		'			End If
		'		End Sub
#End Region

#Region "ICustomTypeDescriptor Implementation"

		Public Function GetAttributes() As System.ComponentModel.AttributeCollection Implements System.ComponentModel.ICustomTypeDescriptor.GetAttributes
			Return TypeDescriptor.GetAttributes(Me.GetType)
		End Function

		Public Function GetClassName() As String Implements System.ComponentModel.ICustomTypeDescriptor.GetClassName
			Return TypeDescriptor.GetClassName(Me.GetType)
		End Function

		Public Function GetComponentName() As String Implements System.ComponentModel.ICustomTypeDescriptor.GetComponentName
			Return TypeDescriptor.GetComponentName(Me.GetType)
		End Function

		Public Function GetConverter() As System.ComponentModel.TypeConverter Implements System.ComponentModel.ICustomTypeDescriptor.GetConverter
			Return TypeDescriptor.GetConverter(Me.GetType)
		End Function

		Public Function GetDefaultEvent() As System.ComponentModel.EventDescriptor Implements System.ComponentModel.ICustomTypeDescriptor.GetDefaultEvent
			Return TypeDescriptor.GetDefaultEvent(Me.GetType)
		End Function

		Public Function GetDefaultProperty() As System.ComponentModel.PropertyDescriptor Implements System.ComponentModel.ICustomTypeDescriptor.GetDefaultProperty
			Return TypeDescriptor.GetDefaultProperty(Me.GetType)
		End Function

		Public Function GetEditor(ByVal editorBaseType As System.Type) As Object Implements System.ComponentModel.ICustomTypeDescriptor.GetEditor
			Return TypeDescriptor.GetEditor(Me.GetType, editorBaseType)
		End Function

		Public Function GetEvents(ByVal attributes() As System.Attribute) As System.ComponentModel.EventDescriptorCollection Implements System.ComponentModel.ICustomTypeDescriptor.GetEvents
			Return TypeDescriptor.GetEvents(Me.GetType, attributes)
		End Function

		Public Function GetEvents2() As System.ComponentModel.EventDescriptorCollection Implements System.ComponentModel.ICustomTypeDescriptor.GetEvents
			Return TypeDescriptor.GetEvents(Me.GetType)
		End Function

		Public Function GetProperties(ByVal attributes() As System.Attribute) As System.ComponentModel.PropertyDescriptorCollection Implements System.ComponentModel.ICustomTypeDescriptor.GetProperties
			Dim objNewCol As New PropertyDescriptorCollection(Nothing)
			Dim objExistingCol As PropertyDescriptorCollection
			Dim pd As PropertyDescriptor

			objExistingCol = TypeDescriptor.GetProperties(Me.GetType, attributes)
			For Each pd In objExistingCol
				Select Case pd.Name
					Case "NavigateUrl"
						If Me.ControlAction = "navigate" Then
							pd.GetValue(Me)
							objNewCol.Add(pd)
						End If
					Case "JSFunction"
						If Me.ControlAction = "js" Then
							pd.GetValue(Me)
							objNewCol.Add(pd)
						End If
					Case "Key"
						If Array.IndexOf(CommonActions, Me.ControlAction) > -1 Then
							pd.GetValue(Me)
							objNewCol.Add(pd)
						End If
					Case Else
						pd.GetValue(Me)
						objNewCol.Add(pd)
				End Select
			Next
			Return objNewCol

		End Function

		Private Sub HandlePropertyFilters(ByVal pd As PropertyDescriptor, ByVal x As PropertyDescriptorCollection)
			pd.GetValue(Me)
			x.Add(pd)
		End Sub

		Public Function GetProperties2() As System.ComponentModel.PropertyDescriptorCollection Implements System.ComponentModel.ICustomTypeDescriptor.GetProperties
			Return TypeDescriptor.GetProperties(Me.GetType)
		End Function

		Public Function GetPropertyOwner(ByVal pd As System.ComponentModel.PropertyDescriptor) As Object Implements System.ComponentModel.ICustomTypeDescriptor.GetPropertyOwner
			Return Me
		End Function

#End Region

	End Class
End Namespace