'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2006
' by Perpetual Motion Interactive Systems Inc. ( http://www.perpetualmotion.ca )
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'
Imports System.Web.UI
Imports System.Collections
Imports System.Reflection
Imports System.ComponentModel
Imports System.Drawing.Design

Namespace DotNetNuke.UI.WebControls
	<DefaultMember("Item")> _
	Public NotInheritable Class DNNToolBarCollection
		Implements IList, ICollection, IEnumerable

		Private owner As DNNToolbarComponent

		Friend Sub New(ByVal owner As DNNToolbarComponent)
			Me.owner = owner
		End Sub

		' Properties
		Public ReadOnly Property Count() As Integer Implements ICollection.Count
			Get
				If Me.owner.HasControls Then
					Return Me.owner.Controls.Count
				End If
				Return 0
			End Get
		End Property

		Public ReadOnly Property IsReadOnly() As Boolean Implements IList.IsReadOnly
			Get
				Return False
			End Get
		End Property


		Public ReadOnly Property IsSynchronized() As Boolean Implements ICollection.IsSynchronized
			Get
				Return False
			End Get
		End Property


		Default Public ReadOnly Property Item(ByVal index As Integer) As DNNToolBar
			Get
				Return DirectCast(Me.owner.Controls.Item(index), DNNToolBar)
			End Get
		End Property

		Public ReadOnly Property SyncRoot() As Object Implements ICollection.SyncRoot
			Get
				Return Me
			End Get
		End Property

		Private ReadOnly Property IsFixedSize() As Boolean Implements System.Collections.IList.IsFixedSize
			Get
				Return False
			End Get
		End Property

		Private Property _item(ByVal index As Integer) As Object Implements System.Collections.IList.Item
			Get
				Return Me.owner.Controls.Item(index)
			End Get
			Set(ByVal value As Object)
				Me.RemoveAt(index)
				Me.AddAt(index, DirectCast(value, DNNToolBar))
			End Set
		End Property

		' Methods
		Public Function Add(ByVal row As DNNToolBar) As Integer
			Me.AddAt(-1, row)
			Return (Me.owner.Controls.Count - 1)
		End Function

		Public Sub AddAt(ByVal index As Integer, ByVal row As DNNToolBar)
			Me.owner.Controls.AddAt(index, row)
		End Sub

		Public Sub AddRange(ByVal rows As DNNToolBar())
			If (rows Is Nothing) Then
				Throw New ArgumentNullException("tabs")
			End If
			Dim row1 As DNNToolBar
			For Each row1 In rows
				Me.Add(row1)
			Next
		End Sub

		Public Sub Clear() Implements IList.Clear
			If Me.owner.HasControls Then
				Me.owner.Controls.Clear()
			End If
		End Sub

		Public Sub CopyTo(ByVal array As Array, ByVal index As Integer) Implements ICollection.CopyTo
			Dim enumerator1 As IEnumerator = Me.GetEnumerator
			Do While enumerator1.MoveNext
				index += 1
				array.SetValue(enumerator1.Current, index)
			Loop
		End Sub

		Public Function GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
			Return Me.owner.Controls.GetEnumerator
		End Function

		Public Function GetRowIndex(ByVal row As DNNToolBar) As Integer
			If Me.owner.HasControls Then
				Return Me.owner.Controls.IndexOf(row)
			End If
			Return -1
		End Function

		Public Sub Remove(ByVal row As DNNToolBar)
			Me.owner.Controls.Remove(row)
		End Sub


		Public Sub RemoveAt(ByVal index As Integer) Implements IList.RemoveAt
			Me.owner.Controls.RemoveAt(index)
		End Sub

		Private Function Add(ByVal o As Object) As Integer Implements System.Collections.IList.Add
			Return Me.Add(DirectCast(o, DNNToolBar))
		End Function

		Private Function Contains(ByVal o As Object) As Boolean Implements System.Collections.IList.Contains
			Return Me.owner.Controls.Contains(DirectCast(o, DNNToolBar))
		End Function

		Private Function IndexOf(ByVal o As Object) As Integer Implements System.Collections.IList.IndexOf
			Return Me.owner.Controls.IndexOf(DirectCast(o, DNNToolBar))
		End Function

		Private Sub Insert(ByVal index As Integer, ByVal o As Object) Implements System.Collections.IList.Insert
			Me.AddAt(index, DirectCast(o, DNNToolBar))
		End Sub

		Private Sub Remove(ByVal o As Object) Implements System.Collections.IList.Remove
			Me.Remove(DirectCast(o, DNNToolBar))
		End Sub

		Public Function FindToolBar(ByVal strTarget As String) As DNNToolBar
			For Each objTab As DNNToolBar In Me
				If objTab.Target = strTarget Then
					Return objTab
				End If
			Next
			Return Nothing
		End Function

	End Class
End Namespace