'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2009
' by DotNetNuke Corporation
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System
Imports System.Collections
Imports System.ComponentModel
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Diagnostics
Imports System.Xml
Imports DotNetNuke.UI.Utilities
Imports System.Collections.Generic

Namespace DotNetNuke.UI.WebControls

    Public Enum eCheckBoxMode
        MultiSelect = 0
        MultiSelectChildren = 1
        SingleSelect = 2
    End Enum

	Public Class DNNTreeBuilder
		Inherits ControlBuilder

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' 
		''' </summary>
		''' <param name="tagName"></param>
		''' <param name="attribs"></param>
		''' <returns></returns>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[jbrinkman]	5/6/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Overloads Overrides Function GetChildControlType(ByVal tagName As String, ByVal attribs As IDictionary) As Type
			If tagName.ToUpper.EndsWith("TreeNode") Then
				Return GetType(TreeNode)
			End If
			Return Nothing
		End Function

	End Class

	<ControlBuilderAttribute(GetType(DNNTreeBuilder)), _
	Designer(GetType(DotNetNuke.UI.Design.WebControls.DNNTreeDesigner)), _
	DefaultProperty("Nodes"), _
	ToolboxData("<{0}:DNNTree runat=server></{0}:DNNTree>")> _
	Public Class DnnTree
		Inherits WebControl
		Implements IPostBackEventHandler
		Implements IPostBackDataHandler
		Implements DotNetNuke.UI.Utilities.IClientAPICallbackEventHandler

#Region "Events / Delegates"
		Public Delegate Sub DNNTreeEventHandler(ByVal source As Object, ByVal e As DNNTreeEventArgs)
		Public Delegate Sub DNNTreeNodeClickHandler(ByVal source As Object, ByVal e As DNNTreeNodeClickEventArgs)

		Public Event Expand As DNNTreeEventHandler
		Public Event Collapse As DNNTreeEventHandler
		Public Event NodeClick As DNNTreeNodeClickHandler
		Public Event PopulateOnDemand As DNNTreeEventHandler
#End Region

#Region "Member Variables"
        Private m_Nodes As TreeNodeCollection
        Private m_objImages As NodeImageCollection
        Private m_TreeWriter As IDNNTreeWriter
        'Private m_DNNAnimation As DNNAnimation = New DNNAnimation
#End Region

#Region "Constructors"
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Sub New()
        End Sub
#End Region

#Region "Properties"
        <Bindable(True), DefaultValue(""), ClientProperty(), ClientPropertyName("bid")> _
        Public Property BehaviorID() As String
            Get
                Return CType(ViewState("BehaviorID"), String)
            End Get
            Set(ByVal Value As String)
                ViewState("BehaviorID") = Value
            End Set
        End Property

        <DefaultValue(eCheckBoxMode.MultiSelect), ClientProperty(), ClientPropertyName("cbm")> _
        Public Property CheckBoxMode() As eCheckBoxMode
            Get
                If Len(ViewState("CheckBoxMode")) > 0 Then
                    Return CType(ViewState("CheckBoxMode"), eCheckBoxMode)
                End If
                Return eCheckBoxMode.MultiSelect
            End Get
            Set(ByVal Value As eCheckBoxMode)
                ViewState("CheckBoxMode") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Allows developer to force the rendering of the tree in DownLevel mode
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/22/2005	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Property ForceDownLevel() As Boolean
            Get
                Return CType(ViewState("ForceDownLevel"), Boolean)
            End Get
            Set(ByVal Value As Boolean)
                ViewState("ForceDownLevel") = Value
            End Set
        End Property

        Public Property IsCrawler() As Boolean
            Get
                If Len(ViewState("IsCrawler")) = 0 Then
                    Return System.Web.HttpContext.Current.Request.Browser.Crawler
                Else
                    Return CBool(ViewState("IsCrawler"))
                End If

            End Get
            Set(ByVal Value As Boolean)
                ViewState("IsCrawler") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns whether the tree will render DownLevel or not
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/22/2005	Commented
        '''		[Jon Henning]	3/9/2005	Requiring XML support on client for uplevel
        '''		[Jon Henning]	11/28/2005	rendering downlevel if it is a crawler
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Browsable(False)> _
        Public ReadOnly Property IsDownLevel() As Boolean
            Get
                If ForceDownLevel OrElse IsCrawler OrElse _
                 DotNetNuke.UI.Utilities.ClientAPI.BrowserSupportsFunctionality(Utilities.ClientAPI.ClientFunctionality.DHTML) = False OrElse _
                 DotNetNuke.UI.Utilities.ClientAPI.BrowserSupportsFunctionality(Utilities.ClientAPI.ClientFunctionality.XML) = False _
                Then
                    Return True
                Else
                    Return False
                End If
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Location of ClientAPI js files 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/22/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Property ClientAPIScriptPath() As String
            Get
                Return DotNetNuke.UI.Utilities.ClientAPI.ScriptPath
            End Get
            Set(ByVal Value As String)
                ' Commented out because it was breaking the build.. (SM 26 Oct)  
                ' uncommented out, don't understand how this could have broke build - JH 22 Feb 2005
                DotNetNuke.UI.Utilities.ClientAPI.ScriptPath = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Location of dnn.controls.dnntree.js file
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' Since 1.1 this path will be the same as the ClientAPI path.
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/22/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Property TreeScriptPath() As String
            Get
                If Len(ViewState("TreeScriptPath")) = 0 Then
                    Return ClientAPIScriptPath
                Else
                    Return CStr(ViewState("TreeScriptPath"))
                End If
            End Get
            Set(ByVal Value As String)
                ViewState("TreeScriptPath") = Value
            End Set
        End Property


        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Location of system images (i.e. spacer.gif)
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	2/22/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Description("Directory to find the images for the menu.  Need to have spacer.gif here!"), DefaultValue("images/"), ClientProperty(), ClientPropertyName("sysimgpath"), UrlProperty()> _
        Public Property SystemImagesPath() As String
            Get
                If Len(ViewState("SystemImagesPath")) = 0 Then
                    Return "images/"
                Else
                    Return CStr(ViewState("SystemImagesPath"))
                End If
            End Get
            Set(ByVal Value As String)
                ViewState("SystemImagesPath") = Value
            End Set
        End Property


        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Browsable(True), PersistenceMode(PersistenceMode.InnerProperty)> _
        Public ReadOnly Property TreeNodes() As TreeNodeCollection
            Get
                If m_Nodes Is Nothing Then
                    m_Nodes = New TreeNodeCollection(Me.ClientID, Me)
                End If
                Return m_Nodes
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Holds collection of selected node objects
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	4/6/2005	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Browsable(False)> _
        Public ReadOnly Property SelectedTreeNodes() As Collection
            Get
                Return Me.TreeNodes.FindSelectedNodes()
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), DefaultValue(10), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("indentw")> _
    Public Property IndentWidth() As Integer
            Get
                Return CType(ViewState("IndentWidth"), Integer)
            End Get
            Set(ByVal Value As Integer)
                ViewState("IndentWidth") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), DefaultValue(""), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("colimg"), UrlProperty()> _
        Public Property CollapsedNodeImage() As String
            Get
                Return CType(ViewState("CollapsedNodeImage"), String)
            End Get
            Set(ByVal Value As String)
                ViewState("CollapsedNodeImage") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), DefaultValue(""), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("expimg"), UrlProperty()> _
        Public Property ExpandedNodeImage() As String
            Get
                Return CType(ViewState("ExpandedNodeImage"), String)
            End Get
            Set(ByVal Value As String)
                ViewState("ExpandedNodeImage") = Value
            End Set
        End Property

        <Bindable(True), DefaultValue("dnnanim.gif"), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("workimg")> _
        Public Property WorkImage() As String
            Get
                Return CType(ViewState("WorkImage"), String)
            End Get
            Set(ByVal Value As String)
                ViewState("WorkImage") = Value
            End Set
        End Property

        <Bindable(True), DefaultValue(5), PersistenceMode(PersistenceMode.Attribute), Category("Behavior"), ClientProperty(), ClientPropertyName("animf")> _
        Public Property AnimationFrames() As Integer
            Get
                If Len(ViewState("AnimationFrames")) > 0 Then
                    Return CInt(ViewState("AnimationFrames"))
                Else
                    Return 5
                End If
            End Get
            Set(ByVal Value As Integer)
                ViewState("AnimationFrames") = Value
            End Set
        End Property

        '<Category("Appearance"), Description("The animation to be applied to showing/hiding the menus"), DesignerSerializationVisibility(DesignerSerializationVisibility.Content), NotifyParentProperty(True)> _
        'Public ReadOnly Property Animation() As DNNAnimation
        '    Get
        '        Return m_DNNAnimation
        '    End Get
        'End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), DefaultValue(False), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("checkboxes")> _
        Public Property CheckBoxes() As Boolean
            Get
                Return CType(ViewState("CheckBoxes"), Boolean)
            End Get
            Set(ByVal Value As Boolean)
                ViewState("CheckBoxes") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), DefaultValue(""), ClientProperty(), ClientPropertyName("target")> _
        Public Property Target() As String
            Get
                Return (CType(ViewState("Target"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("Target") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Browsable(True), PersistenceMode(PersistenceMode.InnerProperty)> _
          Public ReadOnly Property ImageList() As NodeImageCollection
            Get
                If m_objImages Is Nothing Then
                    m_objImages = New NodeImageCollection
                    If IsTrackingViewState Then
                        CType(m_objImages, IStateManager).TrackViewState()
                    End If
                End If
                Return m_objImages
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), DefaultValue(""), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("css")> _
        Public Property DefaultNodeCssClass() As String
            Get
                Return Me.CssClass
            End Get
            Set(ByVal Value As String)
                Me.CssClass = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), DefaultValue(""), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("csschild")> _
        Public Property DefaultChildNodeCssClass() As String
            Get
                Return (CType(ViewState("DefaultChildNodeCssClass"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("DefaultChildNodeCssClass") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), DefaultValue(""), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("csshover")> _
        Public Property DefaultNodeCssClassOver() As String
            Get
                Return (CType(ViewState("DefaultNodeCssClassOver"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("DefaultNodeCssClassOver") = Value
            End Set
        End Property

        <Bindable(True), DefaultValue(""), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("csssel")> _
        Public Property DefaultNodeCssClassSelected() As String
            Get
                Return (CType(ViewState("DefaultNodeCssClassSelected"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("DefaultNodeCssClassSelected") = Value
            End Set
        End Property

        <Bindable(True), DefaultValue(""), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("cssicon")> _
        Public Property DefaultIconCssClass() As String
            Get
                Return (CType(ViewState("DefaultIconCssClass"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("DefaultIconCssClass") = Value
            End Set
        End Property

        <Bindable(True), DefaultValue(12), PersistenceMode(PersistenceMode.Attribute), ClientProperty(), ClientPropertyName("expcolimgw")> _
        Public Property ExpandCollapseImageWidth() As Integer
            Get
                If Len(ViewState("ExpColImgWidth")) > 0 Then
                    Return CType(ViewState("ExpColImgWidth"), Integer)
                Else
                    Return 12
                End If
            End Get
            Set(ByVal Value As Integer)
                ViewState("ExpColImgWidth") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private ReadOnly Property TreeWriter() As IDNNTreeWriter
            Get
                If m_TreeWriter Is Nothing Then
                    If Me.IsDownLevel Then
                        m_TreeWriter = New DNNTreeWriter
                    Else
                        m_TreeWriter = New DNNTreeUpLevelWriter
                    End If
                End If
                Return m_TreeWriter
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Allows you to have a common JS function be invoked for all nodes, unless
        ''' a different JS function is provided on the node level.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' If the client-side function returns false, the action associated to the node 
        ''' selection will be canceled
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	4/6/2005	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <DefaultValue(""), ClientProperty(), ClientPropertyName("js")> _
        Public Property JSFunction() As String
            Get
                Return (CType(ViewState("JSFunction"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("JSFunction") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Allows the nodes to be populated on the client 
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	4/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(False), DefaultValue(True), PersistenceMode(PersistenceMode.Attribute)> _
          Public Property PopulateNodesFromClient() As Boolean
            Get
                'If DotNetNuke.UI.Utilities.ClientAPI.BrowserSupportsFunctionality(Utilities.ClientAPI.ClientFunctionality.XMLHTTP) Then
                If ViewState("PopNodesFromClient") Is Nothing Then
                    Return True
                Else
                    Return (CType(ViewState("PopNodesFromClient"), Boolean))
                End If
                'Else
                'Return False
                'End If
            End Get
            Set(ByVal Value As Boolean)
                ViewState("PopNodesFromClient") = Value
            End Set
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' If callbacks are supported/enabled, this javascript function will be invoked
        ''' with each status change of the xmlhttp request.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' The Javascript transport does not raise any events.
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	4/6/2005	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Bindable(True), PersistenceMode(PersistenceMode.Attribute)> _
        Public Property CallbackStatusFunction() As String
            Get
                Return (CType(ViewState("CallbackStatusFunction"), String))
            End Get
            Set(ByVal Value As String)
                ViewState("CallbackStatusFunction") = Value
            End Set
        End Property

#End Region

#Region "OverLoaded Methods"

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Responsible for rendering the DNNTree and in an uplevel rendering of the tree, sending down the xml for the child nodes in a ClientAPI variable.
        ''' </summary>
        ''' <param name="writer"></param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        '''		[Jon Henning 4/6/2005	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Protected Overloads Overrides Sub Render(ByVal writer As HtmlTextWriter)
            'If Me.IsDownLevel = False Then DotNetNuke.UI.Utilities.ClientAPI.RegisterClientVariable(Me.Page, Me.ClientID & "_xml", Me.TreeNodes.ToXml, True)
            TreeWriter.RenderTree(writer, Me)
            Dim oCtl As Control
            For Each oCtl In Me.Controls
                oCtl.RenderControl(writer)
            Next
            If Me.IsDownLevel = False Then
                Dim properties As Hashtable = TreeWriter.MarshalledProperties() 'get runtime generated properties (non attribute based)
                Dim json As String = ScriptGenerator.GetMarshalledPropertyJSON(Me, properties)
                BaseWebControl.RegisterInitialize(Me, "initTree", json)
            End If
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <param name="e"></param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Protected Overloads Overrides Sub OnInit(ByVal e As EventArgs)
            MyBase.OnInit(e)
            If DotNetNuke.UI.Utilities.ClientAPI.NeedsDNNVariable(Me) Then
                'This is to allow us to add a control to our parent's control collection...  kindof a hack
                AddHandler Me.Page.Load, AddressOf ParentOnLoad
            Else
                'LoadPostedXml()
                LoadPostedJSON()
            End If
        End Sub

        Protected Sub ParentOnLoad(ByVal Sender As Object, ByVal e As System.EventArgs)
            DotNetNuke.UI.Utilities.ClientAPI.RegisterDNNVariableControl(Me)
            'LoadPostedXml()
            LoadPostedJSON()
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Protected Overloads Overrides Function SaveViewState() As Object
            Dim _baseState As Object = MyBase.SaveViewState
            Dim _imagesState As Object = CType(ImageList, IStateManager).SaveViewState
            Dim _newState(1) As Object
            _newState(0) = _baseState
            _newState(1) = _imagesState
            Return _newState
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <param name="state"></param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Protected Overloads Overrides Sub LoadViewState(ByVal state As Object)
            If Not (state Is Nothing) Then
                Dim _newState As Object() = CType(state, Object())
                If Not (_newState(0) Is Nothing) Then
                    MyBase.LoadViewState(_newState(0))
                End If
                If Not (_newState(1) Is Nothing) Then
                    CType(ImageList, IStateManager).LoadViewState(_newState(1))
                End If
            End If
            If IsDownLevel AndAlso Len(ViewState("xml")) > 0 Then m_Nodes = New TreeNodeCollection(CStr(ViewState("xml")), "", Me)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Protected Overloads Overrides Sub TrackViewState()
            MyBase.TrackViewState()
            'CType(Root, IStateManager).TrackViewState()
        End Sub
#End Region

#Region "Event Handlers"
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <param name="eventArgument"></param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Overridable Sub RaisePostBackEvent(ByVal eventArgument As String) Implements IPostBackEventHandler.RaisePostBackEvent
            Dim args() As String = Split(eventArgument, ClientAPI.COLUMN_DELIMITER)

            Dim Node As TreeNode = TreeNodes.FindNode(args(0))

            If Not (Node Is Nothing) Then
                If args.Length > 1 Then
                    Select Case args(1)
                        Case "Click"
                            If Me.CheckBoxes = False Then
                                Dim objNode As TreeNode
                                For Each objNode In TreeNodes.FindSelectedNodes()
                                    objNode.Selected = False
                                Next
                            End If
                            If Node.ClickAction = eClickAction.Expand Then
                                If Node.DNNNodes.Count = 0 AndAlso Me.PopulateNodesFromClient Then                       'Node.DNNNodes.Count = 0 NEW!!!
                                    RaiseEvent PopulateOnDemand(Me, New DNNTreeEventArgs(Node))
                                End If
                                If Node.IsExpanded Then
                                    Node.Collapse()
                                Else
                                    Node.Expand()
                                End If
                            End If
                            Node.Click()
                        Case "Checked"
                            Node.Selected = Not Node.Selected
                        Case "OnDemand"
                            RaiseEvent PopulateOnDemand(Me, New DNNTreeEventArgs(Node))
                    End Select
                Else
                    If Node.IsExpanded Then
                        Node.Collapse()
                    Else
                        Node.Expand()
                    End If
                    If Node.DNNNodes.Count = 0 AndAlso Me.PopulateNodesFromClient Then                    'Node.DNNNodes.Count = 0 NEW!!!
                        RaiseEvent PopulateOnDemand(Me, New DNNTreeEventArgs(Node))
                    End If
                End If
            End If
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <param name="e"></param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Overridable Sub OnExpand(ByVal e As DNNTreeEventArgs)
            RaiseEvent Expand(Me, e)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <param name="e"></param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Overridable Sub OnCollapse(ByVal e As DNNTreeEventArgs)
            RaiseEvent Collapse(Me, e)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <param name="e"></param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	10/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Overridable Sub OnNodeClick(ByVal e As DNNTreeNodeClickEventArgs)
            RaiseEvent NodeClick(Me, e)
        End Sub

        Private Sub DnnTree_PreRender(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.PreRender
            RegisterClientScript()
            Page.RegisterRequiresPostBack(Me)
            UpdateNodes(Me.TreeNodes)            'update all imageindex properties

            If Me.IsDownLevel = False Then
                'DotNetNuke.UI.Utilities.ClientAPI.RegisterClientVariable(Me.Page, Me.ClientID & "_xml", Me.TreeNodes.ToXml, True)
                DotNetNuke.UI.Utilities.ClientAPI.RegisterClientVariable(Me.Page, Me.ClientID & "_json", "{" & Me.TreeNodes.ToJSON() & "}", True)
            Else
                ViewState("xml") = Me.TreeNodes.ToXml
                If DotNetNuke.UI.Utilities.ClientAPI.BrowserSupportsFunctionality(Utilities.ClientAPI.ClientFunctionality.DHTML) Then
                    DotNetNuke.UI.Utilities.ClientAPI.RegisterClientReference(Me.Page, ClientAPI.ClientNamespaceReferences.dnn)
                    If Me.SelectedTreeNodes.Count > 0 Then
                        DotNetNuke.UI.Utilities.ClientAPI.RegisterClientVariable(Me.Page, Me.ClientID & "_selNode", CType(Me.SelectedTreeNodes(1), TreeNode).ToJSON(False), True)
                    End If
                End If

            End If

        End Sub

        Public Function LoadPostData(ByVal postDataKey As String, ByVal postCollection As System.Collections.Specialized.NameValueCollection) As Boolean Implements System.Web.UI.IPostBackDataHandler.LoadPostData
            'We need to process the individual checkboxes
            'If Me.CheckBoxes Then SelectNodes(Nothing, postCollection)

        End Function

        Public Sub RaisePostDataChangedEvent() Implements System.Web.UI.IPostBackDataHandler.RaisePostDataChangedEvent

        End Sub

        Public Function RaiseClientAPICallbackEvent(ByVal eventArgument As String) As String Implements Utilities.IClientAPICallbackEventHandler.RaiseClientAPICallbackEvent
            Dim aryArgs As String() = Split(eventArgument, ClientAPI.COLUMN_DELIMITER)
            Dim strNodeXml As String = "<root>" & aryArgs(0) & "</root>"
            Dim objNodes As TreeNodeCollection = New TreeNodeCollection(strNodeXml, "", Me)

            Dim objNode As TreeNode = objNodes.Item(0)
            If Not objNode Is Nothing Then
                RaiseEvent PopulateOnDemand(Me, New DNNTreeEventArgs(objNode))
                Dim objTNode As TreeNode = Me.FindNode(objNode.ID)              'if whole tree was populated (i.e. LoadXml, then use the node from the tree

                If Not objTNode Is Nothing Then
                    Return objTNode.TreeNodes.ToJSON()
                Else                 'if only passed in node object was updated then use that xml.
                    Return objNode.TreeNodes.ToJSON()
                End If
            Else
                Return Nothing
            End If
        End Function

#End Region

#Region "Methods"
        'Private Sub SelectNodes(ByVal node As TreeNode, ByVal selectedNodes As System.Collections.Specialized.NameValueCollection)
        '	Dim objTreeNodes As TreeNodeCollection
        '	If Not node Is Nothing Then
        '		If selectedNodes(node.ID & ":checkbox") Is Nothing Then
        '			'We need to perform a check to see if data has changed
        '			node.Selected = False
        '		Else
        '			node.Selected = True
        '		End If
        '		objTreeNodes = node.TreeNodes
        '	Else
        '		objTreeNodes = Me.TreeNodes
        '	End If

        '	For Each childNode As TreeNode In objTreeNodes
        '		SelectNodes(childNode, selectedNodes)
        '	Next
        'End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Responsible for inputting the hidden field necessary for the ClientAPI to pass variables back in forth
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	4/6/2005	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        <Obsolete("RegisterDNNVariableControl on control level is now obsolete.  Use RegisterDNNVariableControl.WebControls")> _
        Public Sub RegisterDNNVariableControl()
            DotNetNuke.UI.Utilities.ClientAPI.RegisterDNNVariableControl(Me)
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Determines if client supports an uplevel rendering, and if so it registers
        ''' the appropriate scripts.
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	4/6/2005	Commented
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Sub RegisterClientScript()
            If IsDownLevel = False Then
                DotNetNuke.UI.Utilities.ClientAPI.RegisterClientReference(Me.Page, DotNetNuke.UI.Utilities.ClientAPI.ClientNamespaceReferences.dnn_dom)
                'DotNetNuke.UI.Utilities.ClientAPI.RegisterClientReference(Me.Page, DotNetNuke.UI.Utilities.ClientAPI.ClientNamespaceReferences.dnn_xml)
                If Me.PopulateNodesFromClient Then               'AndAlso DotNetNuke.UI.Utilities.ClientAPI.BrowserSupportsFunctionality(Utilities.ClientAPI.ClientFunctionality.XMLHTTP) Then
                    DotNetNuke.UI.Utilities.ClientAPI.RegisterClientReference(Me.Page, DotNetNuke.UI.Utilities.ClientAPI.ClientNamespaceReferences.dnn_xmlhttp)
                End If
                RegisterSubmitComponent(Me.Page)
                RegisterClientScriptBlock(Me.Page, "dnn.controls.js")
                RegisterClientScriptBlock(Me.Page, "dnn.controls.dnntree.js")
                'If Me.Animation.AnimationType <> UI.WebControls.Animation.AnimationType.None Then
                '    ClientAPI.RegisterClientReference(Me.Page, ClientAPI.ClientNamespaceReferences.dnn_motion)
                'End If

                'If Not ClientAPI.IsClientScriptBlockRegistered(Me.Page, "dnn.controls.dnntree.js") Then
                '	ClientAPI.RegisterClientScriptBlock(Me.Page, "dnn.controls.dnntree.js", "<script src=""" & TreeScriptPath & "dnn.controls.dnntree.js""></script>")
                'End If
                'ClientAPI.RegisterStartUpScript(Page, Me.ClientID & "_startup", "<script type="text/javascript">dnn.controls.initTree($get('" & Me.ClientID & "'));</script>")				'wrong place
            End If
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Finds Node by passed in Key. 
        ''' </summary>
        ''' <param name="strID">ID of node</param>
        ''' <returns></returns>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	11/17/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Function FindNode(ByVal strID As String) As TreeNode
            Return Me.TreeNodes.FindNode(strID)
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Finds Node by passed in Key. 
        ''' </summary>
        ''' <param name="strKey">Key of node</param>
        ''' <returns></returns>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	11/17/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Function FindNodeByKey(ByVal strKey As String) As TreeNode
            Return Me.TreeNodes.FindNodeByKey(strKey)
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Finds Node by passed in ID and selects it.  Additionally it will expand 
        ''' all parent nodes.
        ''' </summary>
        ''' <param name="strID">ID of node</param>
        ''' <returns></returns>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	11/17/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Function SelectNode(ByVal strID As String) As TreeNode
            Dim objNode As TreeNode = Nothing

            If Me.TreeNodes.Count > 0 Then
                objNode = Me.FindNode(strID)
                If Not objNode Is Nothing Then
                    objNode.Selected = True
                    Dim objParent As TreeNode
                    objParent = objNode.Parent

                    While Not objParent Is Nothing
                        objParent.Expand()
                        objParent = objParent.Parent
                    End While

                End If
            End If

            Return objNode
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Finds Node by passed in Key and selects it.  Additionally it will expand 
        ''' all parent nodes.
        ''' </summary>
        ''' <param name="strKey">Key of node</param>
        ''' <returns></returns>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	11/17/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Function SelectNodeByKey(ByVal strKey As String) As TreeNode
            Dim objNode As TreeNode = Nothing

            If Me.TreeNodes.Count > 0 Then
                objNode = Me.FindNodeByKey(strKey)
                If Not objNode Is Nothing Then
                    objNode.Selected = True
                    Dim objParent As TreeNode
                    objParent = objNode

                    While Not objParent Is Nothing
                        objParent.Expand()
                        objParent = objParent.Parent
                    End While
                End If
            End If

            Return objNode
        End Function

        Private Sub LoadPostedXml()
            Dim strXML As String = ""
            If Me.IsDownLevel = False Then strXML = DotNetNuke.UI.Utilities.ClientAPI.GetClientVariable(Me.Page, Me.ClientID & "_xml")
            If Len(strXML) > 0 Then LoadXml(strXML)
        End Sub

        Private Sub LoadPostedJSON()
            Dim json As String = ""
            If Me.IsDownLevel = False Then json = DotNetNuke.UI.Utilities.ClientAPI.GetClientVariable(Me.Page, Me.ClientID & "_json")
            If String.IsNullOrEmpty(json) = False Then
                'Dim ser As System.Web.Script.Serialization.JavaScriptSerializer = New System.Web.Script.Serialization.JavaScriptSerializer()
                Dim dict As Dictionary(Of String, Object) = MSAJAX.Deserialize(Of Dictionary(Of String, Object))(json)
                If dict.ContainsKey("nodes") Then LoadJSON(dict)
                'Me.MenuNodes.LoadJSON(dict)
            End If
        End Sub

        Public Sub LoadJSON(ByVal dict As Dictionary(Of String, Object))
            m_Nodes = New TreeNodeCollection(dict, Me)
        End Sub

        Public Sub LoadXml(ByVal strXml As String)
            m_Nodes = New TreeNodeCollection(strXml, "", Me)
        End Sub

        Private Sub UpdateNodes(ByVal objNodes As TreeNodeCollection)
            Dim objNode As TreeNode
            'todo: xpath lookup for img attribute so we don't waste time looping.
            For Each objNode In objNodes
                If Len(objNode.Image) > 0 Then
                    If Me.ImageList.Contains(objNode.Image) = False Then                      'THIS CHECK FOR CONTAINS DOES NOT WORK!!!
                        objNode.ImageIndex = Me.ImageList.Add(objNode.Image)
                    Else
                        objNode.ImageIndex = Me.ImageList.IndexOf(objNode.Image)
                    End If
                    objNode.Image = Nothing
                End If
                If objNode.DNNNodes.Count > 0 Then UpdateNodes(objNode.TreeNodes)
            Next
        End Sub

        Public Sub ClearSelections()
            For Each node As TreeNode In Me.TreeNodes
                ClearSelections(node)
            Next
        End Sub
        Public Sub ClearSelections(ByVal Parent As TreeNode)
            Parent.Selected = False
            For Each node As TreeNode In Parent.TreeNodes
                ClearSelections(node)
            Next
        End Sub

#End Region

	End Class
End Namespace
