'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2009
' by DotNetNuke Corporation
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System
Imports System.ComponentModel
Imports System.Web.UI
Imports System.Web.UI.WebControls
Namespace DotNetNuke.UI.WebControls

	Public Class TreeNode : Inherits DNNNode : Implements IStateManager
		Friend Shared _separator As String = ":"
		Friend Shared ReadOnly _checkboxIDSufix As String = "checkbox"

		Private m_objNodes As TreeNodeCollection
		Private m_objDNNTree As DNNTree

		Public Sub New()
			MyBase.New()
		End Sub

		Public Sub New(ByVal strText As String)
			MyBase.New(strText)
		End Sub

		Friend Sub New(ByVal objXmlNode As System.Xml.XmlNode, ByVal ctlOwner As Control)
			MyBase.New(objXmlNode)
			m_objDNNTree = CType(ctlOwner, DNNTree)
		End Sub

		Friend Sub New(ByVal ctlOwner As Control)
			MyBase.New()
			m_objDNNTree = CType(ctlOwner, DNNTree)
		End Sub

		'This is here for backwards compatibility
		Public Shadows Property CssClass() As String
			Get
				Return MyBase.CSSClass
			End Get
			Set(ByVal Value As String)
				MyBase.CSSClass = Value
			End Set
		End Property

		'This is here for backwards compatibility
		Public Shadows Property NavigateUrl() As String
			Get
				Return MyBase.NavigateURL
			End Get
			Set(ByVal Value As String)
				MyBase.NavigateURL = Value
			End Set
		End Property

		<Browsable(True), PersistenceMode(PersistenceMode.InnerProperty)> _
		  Public ReadOnly Property TreeNodes() As TreeNodeCollection
			Get
				If m_objNodes Is Nothing Then
					m_objNodes = New TreeNodeCollection(Me.XmlNode, Me.DNNTree)
				End If
				Return m_objNodes
			End Get
		End Property

		<Browsable(False)> _
		  Public ReadOnly Property Parent() As TreeNode
			Get
				Return CType(Me.ParentNode, TreeNode)
			End Get
		End Property

		<Browsable(False)> _
		  Public ReadOnly Property DNNTree() As DNNTree
			Get
				Return CType(m_objDNNTree, DNNTree)
			End Get
		End Property

		<DefaultValue(False), Browsable(False)> _
		  Public ReadOnly Property IsExpanded() As Boolean
			Get
				Dim _expanded As Object
				If DNNTree.IsDownLevel = False Then
					Dim sExpanded As String = DotNetNuke.UI.Utilities.ClientAPI.GetClientVariable(m_objDNNTree.Page, m_objDNNTree.ClientID & "_" & Me.ClientID & ":expanded")
					If Len(sExpanded) > 0 Then
						_expanded = CBool(sExpanded)
					Else
						_expanded = CBool(Me.CustomAttribute("expanded", "0"))
					End If
				Else
					_expanded = CBool(Me.CustomAttribute("expanded", "0"))
				End If
				Return CType(_expanded, Boolean)
			End Get
		End Property


		'<Bindable(True), DefaultValue(False), PersistenceMode(PersistenceMode.Attribute)> _
		'  Public Property CheckBox() As Boolean
		'	Get
		'		Dim _checkBox As Object = Me.CustomAttribute("checkBox", 0)
		'		Return CType(_checkBox, Boolean)
		'	End Get
		'	Set(ByVal Value As Boolean)
		'		Me.CustomAttribute("checkBox", 0) = Value
		'	End Set
		'End Property

		<Bindable(True), DefaultValue(""), PersistenceMode(PersistenceMode.Attribute)> _
		  Public Property CssClassOver() As String
			Get
				Return Me.CSSClassHover
			End Get
			Set(ByVal Value As String)
				Me.CSSClassHover = Value
			End Set
		End Property

		<Bindable(True), DefaultValue(-1), PersistenceMode(PersistenceMode.Attribute)> _
		  Public Property ImageIndex() As Integer
			Get
				If Len(Me.CustomAttribute("imgIdx")) > 0 Then
					Return CInt(Me.CustomAttribute("imgIdx"))
				Else
					If Me.DNNTree.ImageList.Count > 0 Then
						Return 0						 'BACKWARDS COMPATIBILITY!!!! SHOULD BE -1
					Else
						Return -1
					End If
				End If
			End Get
			Set(ByVal Value As Integer)
				Me.CustomAttribute("imgIdx") = CStr(Value)
			End Set
		End Property

		'Public Property PopulateOnDemand() As Boolean
		'	Get
		'		Return CBool(Me.CustomAttribute("pond", False))
		'	End Get
		'	Set(ByVal Value As Boolean)
		'		Me.CustomAttribute("pond") = Value
		'	End Set
		'End Property

		Public Property LeftHTML() As String
			Get
				Return Me.CustomAttribute("lhtml", "")
			End Get
			Set(ByVal Value As String)
				Me.CustomAttribute("lhtml") = Value
			End Set
		End Property

		Public Property RightHTML() As String
			Get
				Return Me.CustomAttribute("rhtml", "")
			End Get
			Set(ByVal Value As String)
				Me.CustomAttribute("rhtml") = Value
			End Set
		End Property


		Public Shadows ReadOnly Property ParentNode() As TreeNode
			Get
				If Not Me.XmlNode.ParentNode Is Nothing AndAlso Me.XmlNode.ParentNode.NodeType <> System.Xml.XmlNodeType.Document Then Return New TreeNode(Me.XmlNode.ParentNode, m_objDNNTree) Else Return Nothing
			End Get
		End Property

		Public Sub MakeNodeVisible()
			If Not Me.Parent Is Nothing Then
				Me.Parent.Expand()
				Me.Parent.MakeNodeVisible()
			End If
		End Sub

		Private ReadOnly Property NodeWriter() As ITreeNodeWriter
			Get
				If m_objDNNTree.IsDownLevel Then
					Return New TreeNodeWriter
				Else
					Return Nothing					  ' New TreeNodeUpLevelWriter
				End If
			End Get
		End Property

		Public Sub Expand()
			If HasNodes Then
				Me.CustomAttribute("expanded") = "1"
				DNNTree.OnExpand(New DNNTreeEventArgs(Me))
			End If
		End Sub

		Public Sub Collapse()
			If HasNodes Then
				Me.CustomAttribute("expanded") = "0"
				DNNTree.OnCollapse(New DNNTreeEventArgs(Me))
			End If
		End Sub

		Public Sub Click()
			Me.Selected = True
			DNNTree.OnNodeClick(New DNNTreeNodeClickEventArgs(Me))
		End Sub

		Public Overridable Sub Render(ByVal writer As HtmlTextWriter)
			NodeWriter.RenderNode(writer, Me)
		End Sub

		Friend Sub SetDNNTree(ByVal objTree As DNNTree)
			m_objDNNTree = objTree
		End Sub

		'BACKWARDS COMPATIBILITY ONLY
#Region "IStateManager Interface"
		Public ReadOnly Property IsTrackingViewState() As Boolean Implements IStateManager.IsTrackingViewState
			Get
				Return False
			End Get
		End Property

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' 
		''' </summary>
		''' <param name="state"></param>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[jbrinkman]	5/6/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Sub LoadViewState(ByVal state As Object) Implements IStateManager.LoadViewState
		End Sub

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' 
		''' </summary>
		''' <returns></returns>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[jbrinkman]	5/6/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Function SaveViewState() As Object Implements IStateManager.SaveViewState
		End Function

		''' -----------------------------------------------------------------------------
		''' <summary>
		''' 
		''' </summary>
		''' <remarks>
		''' </remarks>
		''' <history>
		''' 	[jbrinkman]	5/6/2004	Created
		''' </history>
		''' -----------------------------------------------------------------------------
		Public Sub TrackViewState() Implements IStateManager.TrackViewState
		End Sub
#End Region

	End Class
End Namespace
