'
' DotNetNuke - http://www.dotnetnuke.com
' Copyright (c) 2002-2009
' by DotNetNuke Corporation
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'

Imports System.Xml
Imports DotNetNuke.UI.WebControls
Imports System.Web.UI
Imports System.Collections
Imports System.Collections.Generic

Namespace DotNetNuke.UI.WebControls
	Public Class TreeNodeCollection : Inherits DNNNodeCollection : Implements IStateManager

#Region "Member Variables"
        Private m_DNNTree As DnnTree
#End Region

#Region "Constructors"
        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Constructor to call when creating a Root Node
        ''' </summary>
        ''' <param name="strNamespace">Namespace of node hierarchy</param>
        ''' <param name="objControl">DnnTree control associated to TreeNodeCollection</param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	12/22/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal strNamespace As String, ByVal objControl As DnnTree)
            MyBase.New(strNamespace, objControl)
            m_DNNTree = objControl
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Loads node collection based off of XML string
        ''' </summary>
        ''' <param name="strXML">XML String</param>
        ''' <param name="strXSLFile">XSL FileName.  Leave empty if no transform needed</param>
        ''' <param name="objControl">DNN Tree control</param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	12/22/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal strXML As String, ByVal strXSLFile As String, ByVal objControl As DnnTree)
            MyBase.New(strXML, strXSLFile)
            m_DNNTree = objControl
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Constructor for all nodes that are not the root.  
        ''' </summary>
        ''' <param name="objXmlNode">Node whose children will be exposed by this class</param>
        ''' <param name="objControl">DNN Tree control</param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[Jon Henning]	12/22/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Sub New(ByVal objXmlNode As XmlNode, ByVal objControl As DnnTree)        ', ByVal objControl As Control)
            MyBase.New(objXmlNode, objControl)
            m_DNNTree = objControl
        End Sub

        Public Sub New(ByVal JSONDict As Dictionary(Of String, Object), ByVal Tree As DnnTree)
            MyBase.New(Tree.ClientID, JSONDict)
            m_DNNTree = Tree
        End Sub

#End Region

        Public Shadows Function Add() As Integer          'TreeNode
            Return Me.Add("")
        End Function

        Public Shadows Function Add(ByVal strText As String) As Integer       ' TreeNode
            Dim objNode As TreeNode = New TreeNode(m_DNNTree)
            Dim intIndex As Integer = Me.Add(objNode)
            objNode.Text = strText
            Return intIndex
        End Function

        Public Shadows Function Add(ByVal objNode As TreeNode) As Integer         ' TreeNode
            Dim intIndex As Integer = MyBase.Add(objNode)
            objNode.SetDNNTree(m_DNNTree)
            'ChildNodes.count is not guaranteed to be unique... GetHashCode should be a little safer (I assume it is medium trust safe...
            If Len(objNode.ID) = 0 Then objNode.ID = objNode.ParentNameSpace & "_" & objNode.GetHashCode 'Me.XmlNode.ChildNodes.Count
            Return intIndex
        End Function

        Public Shadows Function Add(ByVal strID As String, ByVal strKey As String, ByVal strText As String, ByVal strNavigateURL As String) As Integer
            Return Add(strID, strKey, strText, strNavigateURL, "", "", "", True, "", "", "", False, eClickAction.Navigate, "", -1)
        End Function

        Public Shadows Function Add(ByVal strID As String, ByVal strKey As String, ByVal strText As String, ByVal strNavigateURL As String, _
         ByVal strJSFunction As String, ByVal strTarget As String, ByVal strToolTip As String, ByVal blnEnabled As Boolean, _
         ByVal strCSSClass As String, ByVal strCSSClassSelected As String, ByVal strCSSClassHover As String, _
         ByVal blnSelected As Boolean, ByVal enumClickAction As eClickAction, _
         ByVal strCssClassOver As String, ByVal intImageIndex As Integer) As Integer         ' TreeNode

            Dim intIndex As Integer = Me.Add()
            Dim objNode As TreeNode = Me.Item(intIndex)

            If Len(strID) > 0 Then
                objNode.ID = strID
            Else
                objNode.ID = objNode.ParentNameSpace & "_" & Me.XMLNode.ChildNodes.Count
            End If
            objNode.Key = strKey
            objNode.Text = strText
            objNode.NavigateUrl = strNavigateURL
            objNode.JSFunction = strJSFunction
            objNode.Target = strTarget
            objNode.ToolTip = strToolTip
            objNode.Enabled = blnEnabled
            objNode.CssClass = strCSSClass
            objNode.CSSClassSelected = strCSSClassSelected
            objNode.CSSClassHover = strCSSClassHover
            objNode.Selected = blnSelected
            objNode.ClickAction = enumClickAction
            'objNode.CheckBox = blnCheckBox
            objNode.CssClassOver = strCssClassOver
            objNode.ImageIndex = intImageIndex

            Return intIndex

        End Function

        Default Public Shadows Property Item(ByVal index As Integer) As TreeNode
            Get
                Return New TreeNode(Me.XMLNode.ChildNodes(index), m_DNNTree)
            End Get
            Set(ByVal Value As TreeNode)
                Throw New Exception("Cannot Assign Node Directly")
            End Set
        End Property

        Public Overloads Function IndexOf(ByVal value As TreeNode) As Integer
            Dim i As Integer
            For i = 0 To Me.XMLNode.ChildNodes.Count
                If New TreeNode(Me.XMLNode.ChildNodes(i), m_DNNTree).ID = value.ID Then
                    Return i
                End If
            Next
        End Function

        Public Overloads Sub Insert(ByVal index As Integer, ByVal value As TreeNode)
            Me.XMLNode.InsertAfter(Me.XMLNode.ChildNodes(index), value.XmlNode)
        End Sub

        Public Overloads Sub Remove(ByVal value As TreeNode)
            Me.XMLNode.RemoveChild(value.XmlNode)
        End Sub

        Public Shadows Sub Clear()
            Dim i As Integer
            For i = Me.XMLNode.ChildNodes.Count - 1 To 0 Step -1
                Me.XMLNode.RemoveChild(Me.XMLNode.ChildNodes(i))
            Next
        End Sub

        Public Overloads Function Contains(ByVal value As TreeNode) As Boolean
            If Me.FindNode(value.ID) Is Nothing Then
                Return False
            Else
                Return True
            End If
        End Function

        Public Shadows Function FindNode(ByVal ID As String) As TreeNode
            Dim objNode As XmlNode = FindFast("id", ID, Me.XMLNode, True)
            If Not objNode Is Nothing Then Return New TreeNode(objNode, m_DNNTree) Else Return Nothing
        End Function

        Public Shadows Function FindNodeByKey(ByVal Key As String) As TreeNode
            Dim objNode As XmlNode = FindFast("key", Key, Me.XMLNode, True)
            If Not objNode Is Nothing Then Return New TreeNode(objNode, m_DNNTree) Else Return Nothing
        End Function

        Public Overrides Function FindSelectedNodes() As Collection
            Dim colTreeNodes As Collection = New Collection          'TreeNodeCollection = New TreeNodeCollection(m_strNamespace, m_DNNTree)
            If Not Me.XMLNode Is Nothing Then
                Dim objNodeList As XmlNodeList = Me.XMLNode.SelectNodes("//n[@selected='1']")
                Dim objNode As XmlNode
                For Each objNode In objNodeList
                    colTreeNodes.Add(New TreeNode(objNode, m_DNNTree))
                Next
            End If
            Return colTreeNodes
        End Function


        'Public Overloads Function ToXml() As String
        '	Return Me.XMLDoc.OuterXml
        'End Function

#Region "ICollection Implementation"
        Public Shadows Function GetEnumerator() As IEnumerator
            Return New TreeNodeEnumerator(Me.XMLNode, m_DNNTree)
        End Function
#End Region

        'BACKWARDS COMPATIBILITY ONLY
#Region "IStateManager Interface"
        Public ReadOnly Property IsTrackingViewState() As Boolean Implements IStateManager.IsTrackingViewState
            Get
                Return False
            End Get
        End Property

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <param name="state"></param>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Sub LoadViewState(ByVal state As Object) Implements IStateManager.LoadViewState
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Function SaveViewState() As Object Implements IStateManager.SaveViewState
        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[jbrinkman]	5/6/2004	Created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Sub TrackViewState() Implements IStateManager.TrackViewState
        End Sub
#End Region


    End Class

    Class TreeNodeEnumerator
        Implements IEnumerator

        Private m_objXmlNode As XmlNode
        Private m_DNNTree As DnnTree
        Private m_intCursor As Integer

        Sub New(ByVal objRoot As XmlNode, ByVal objControl As DnnTree)
            m_objXmlNode = objRoot
            m_DNNTree = objControl
            m_intCursor = -1
        End Sub

        Public Sub Reset() Implements IEnumerator.Reset
            m_intCursor = -1
        End Sub

        Public Function MoveNext() As Boolean Implements IEnumerator.MoveNext
            If m_intCursor < m_objXmlNode.ChildNodes.Count Then
                m_intCursor = m_intCursor + 1
            End If

            If (m_intCursor = m_objXmlNode.ChildNodes.Count) Then
                Return False
            Else
                Return True
            End If
        End Function

        Public ReadOnly Property Current() As Object Implements IEnumerator.Current
            Get
                If ((m_intCursor < 0) Or (m_intCursor = m_objXmlNode.ChildNodes.Count)) Then
                    Throw New InvalidOperationException
                Else
                    Return New TreeNode(m_objXmlNode.ChildNodes(m_intCursor), m_DNNTree)
                End If
            End Get
        End Property
    End Class
End Namespace